@@ -1,4 +1,3 @@
-from .main import *
 from .para import *
 from .tensor_scaling import rescale_lambda_centerized_workhorse
 from .tensor_scaling import rescale_lambda_centerized
@@ -15,14 +14,17 @@ from .simnibs import *
 import pynibs.models
 from .exp import *
 from .opt import *
+from .neuron import *
+from .mesh import *
 
 try:
     from .pckg import libeep
 except (ImportError, SyntaxError):
-    # print("libeep not found. visor import will not work.")
     pass
 try:
     from paraview.simple import *
 except ImportError:
-    # print("Paraview not installed. Plot functionalities will not work.")
-    pass
\ No newline at end of file
+    pass
+
+__version__ = "0.2022.8"
+__datadir__ = os.path.join(os.path.os.path.dirname(__file__), '..', 'tests','data')
\ No newline at end of file
