@@ -11,24 +11,12 @@ import pandas as pd
 import numpy as np
 import matplotlib.pyplot as plt
 
-from .exp.exp import read_csv
-from .exp.exp import sort_by_condition
-from .exp.exp import read_exp_stimulations
+import pynibs
+
 from scipy.spatial import Delaunay
 
-from .subject import load_subject
-from .util import add_center
-from .util import euler_angles_to_rotation_matrix
-from .util import in_hull
-from .util import rot_to_quat
-from .util import rotation_matrix_to_euler_angles
-from .util import unique_rows
-
 from collections import OrderedDict
 
-from ortools.constraint_solver import routing_enums_pb2
-from ortools.constraint_solver import pywrapcp
-
 
 def get_coil_dipole_pos(coil_fn, matsimnibs):
     """
@@ -38,7 +26,7 @@ def get_coil_dipole_pos(coil_fn, matsimnibs):
     ----------
     coil_fn : str
         Filename of coil .ccd file
-    matsimnibs : ndarray of float
+    matsimnibs : np.ndarray of float
         Transformation matrix
 
     Returns
@@ -64,9 +52,9 @@ def check_coil_position(points, hull):
 
     Parameters
     ----------
-    points : ndarray of float [N_points x 3]
+    points : np.ndarray of float [N_points x 3]
         Coordinates (x,y,z) of magnetic dipoles
-    hull : Delaunay object or np.array of float [N_surface_points x 3]
+    hull : Delaunay object or np.ndarray of float [N_surface_points x 3]
         Head surface data
 
     Returns
@@ -100,9 +88,9 @@ def calc_coil_transformation_matrix(LOC_mean, ORI_mean, LOC_var, ORI_var, V):
 
     Parameters
     ----------
-    LOC_mean : ndarray of float [3]
+    LOC_mean : np.ndarray of float [3]
         Mean location of TMS coil
-    ORI_mean : ndarray of float [3 x 3]
+    ORI_mean : np.ndarray of float [3 x 3]
         Mean orientations of TMS coil
 
         .. math::
@@ -134,7 +122,7 @@ def calc_coil_transformation_matrix(LOC_mean, ORI_mean, LOC_var, ORI_var, V):
     """
 
     # calculate rotation matrix for angle variation (angle_var in deg)
-    rotation_matrix = euler_angles_to_rotation_matrix(ORI_var * np.pi / 180.)
+    rotation_matrix = pynibs.euler_angles_to_rotation_matrix(ORI_var * np.pi / 180.)
 
     # determine new orientation
     ori = np.dot(ORI_mean, rotation_matrix)
@@ -149,7 +137,8 @@ def calc_coil_transformation_matrix(LOC_mean, ORI_mean, LOC_var, ORI_var, V):
     return mat
 
 
-def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientation='quaternions', folder_pdfplots=None):
+def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientation='quaternions',
+                           folder_pdfplots=None):
     """
     Determines the probability density functions of the transformed coil position (x', y', z') and quaternions of
     the coil orientations (x'', y'', z'')
@@ -169,21 +158,21 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
 
     Returns
     -------
-    pdf_paras_location : list of list of np.arrays [n_conditions]
+    pdf_paras_location : list of list of np.ndarrays [n_conditions]
         Pdf parameters (limits and shape) of the coil position for x', y', and z' for each:
 
             - beta_paras ... [p, q, a, b] (2 shape parameters and limits)
             - moments    ... [data_mean, data_std, beta_mean, beta_std]
             - p_value    ... p-value of the Kolmogorov Smirnov test
             - uni_paras  ... [a, b] (limits)
-    pdf_paras_orientation_euler : list of np.array [n_conditions]
+    pdf_paras_orientation_euler : list of np.ndarray [n_conditions]
         Pdf parameters (limits and shape) of the coil orientation Psi, Theta, and Phi for each:
 
             - beta_paras ... [p, q, a, b] (2 shape parameters and limits)
             - moments    ... [data_mean, data_std, beta_mean, beta_std]
             - p_value    ... p-value of the Kolmogorov Smirnov test
             - uni_paras  ... [a, b] (limits)
-    OP_mean : List of [3 x 4] np.array [n_conditions]
+    OP_mean : List of [3 x 4] np.ndarray [n_conditions]
         List of mean coil position and orientation for different conditions (global coordinate system)
 
         .. math::
@@ -193,14 +182,14 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
             |  &   |   &   |   &  |   \\\\
             \\end{bmatrix}
 
-    OP_zeromean : list of [3 x 4 x n_con_each] np.arrays [n_conditions]
+    OP_zeromean : list of [3 x 4 x n_con_each] np.ndarray [n_conditions]
         List over conditions containing zero-mean coil orientations and positions
-    V : list of [3 x 3] np.arrays [n_conditions]
+    V : list of [3 x 3] np.ndarrays [n_conditions]
         Transformation matrix of coil positions from global coordinate system to transformed coordinate system
-    P_transform : list of np.array [n_conditions]
+    P_transform : list of np.ndarray [n_conditions]
         List over conditions containing transformed coil positions [x', y', z'] of all stimulations
         (zero-mean, rotated by SVD)
-    quaternions : list of np.array [n_conditions]
+    quaternions : list of np.ndarray [n_conditions]
         List over conditions containing imaginary part of quaternions [x'', y'', z''] of all stimulations
     """
     import pygpc
@@ -212,9 +201,10 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
         folder_pdfplots = ''
 
     if fn_rescon and fn_simpos:
-        positions_all, conditions, position_list, _, _ = read_exp_stimulations(fn_rescon, fn_simpos)
+        positions_all, conditions, position_list, _, _ = pynibs.read_exp_stimulations(fn_rescon, fn_simpos)
 
-        # sort POSITIONS according to CONDITIONS, idx_con is alphabetically sorted, first index of condition[*] appeareance
+        # sort POSITIONS according to CONDITIONS, idx_con is alphabetically sorted, first index of condition[*]
+        # appeareance
         conditions_unique, idx_con, n_con_each = np.unique(conditions,
                                                            return_index=True,
                                                            return_inverse=False,
@@ -226,28 +216,28 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
         n_condition = len(conditions_unique)
         n_zaps = len(positions_all)
 
-        l = 0
+        pos_idx = 0
         k = 0
 
         # Orientation and position tensors (list over conditions containing arrays of [3 x 4 x n_con_each])
-        OP = [np.zeros((3, 4, n_con_each[i])) for i in range(n_condition)]
+        op = [np.zeros((3, 4, n_con_each[i])) for i in range(n_condition)]
 
         # read data from global position_list
         for i in range(n_condition):
-            while position_list[l][len(position_list[0]) - 3] == conditions_unique[i]:
-                OP[i][:, :, k] = np.array(positions_all[l])[0:3, 0:4]
-                l = l + 1
+            while position_list[pos_idx][len(position_list[0]) - 3] == conditions_unique[i]:
+                op[i][:, :, k] = np.array(positions_all[pos_idx])[0:3, 0:4]
+                pos_idx = pos_idx + 1
                 k = k + 1
-                if l == n_zaps:
+                if pos_idx == n_zaps:
                     break
             k = 0
 
         # determine matrix of mean location and orientation (list over conditions containing arrays of [3 x 4])
-        OP_mean = [np.mean(OP[i], axis=2) for i in range(n_condition)]
+        op_mean = [np.mean(op[i], axis=2) for i in range(n_condition)]
 
     elif fn_exp:
-        exp = read_csv(fn_exp)
-        exp_cond = sort_by_condition(exp)
+        exp = pynibs.read_csv(fn_exp)
+        exp_cond = pynibs.sort_by_condition(exp)
 
         conditions_unique = [exp_cond[i_cond]['condition'][0] for i_cond in range(len(exp_cond))]
 
@@ -255,14 +245,14 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
         n_condition = len(exp_cond)
 
         # Orientation and position tensors (list over conditions containing arrays of [3 x 4 x n_con_each])
-        OP = [np.zeros((3, 4, n_con_each[i])) for i in range(n_condition)]
+        op = [np.zeros((3, 4, n_con_each[i])) for i in range(n_condition)]
 
         for i in range(n_condition):
             for k in range(n_con_each[i]):
-                OP[i][:, :, k] = exp_cond[i]['coil_mean_matrix'][k][0:3, :]
+                op[i][:, :, k] = exp_cond[i]['coil_mean_matrix'][k][0:3, :]
 
         # determine matrix of mean location and orientation (list over conditions containing arrays of [3 x 4])
-        OP_mean = [np.mean(OP[i], axis=2) for i in range(n_condition)]
+        op_mean = [np.mean(op[i], axis=2) for i in range(n_condition)]
 
     else:
         raise AssertionError('Please provide experiment.csv or results_condition.csv and simpos.csv files!')
@@ -274,15 +264,15 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
     # [   |      |      |     |  ]         [   |      |      |     |  ]
 
     # Orientation and position tensors with zeromean (list over conditions containing arrays of [3 x 4 x n_con_each])
-    OP_zeromean = [np.zeros((3, 4, n_con_each[i])) for i in range(n_condition)]
+    op_zeromean = [np.zeros((3, 4, n_con_each[i])) for i in range(n_condition)]
 
     # Transformed coil position in (x', y', z') space (list over conditions containing arrays of [n_con_each x 3])
-    P_transform = [0] * n_condition
+    p_transform = [0] * n_condition
 
     # SVD matrices
-    U = [0] * n_condition
+    u = [0] * n_condition
     s = [0] * n_condition
-    V = [0] * n_condition
+    v = [0] * n_condition
 
     # pdf parameters
     pdf_paras_location = [[[] for _ in range(3)] for _ in range(n_condition)]
@@ -293,30 +283,31 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
     for i in range(n_condition):
 
         # shift data by mean
-        OP_zeromean[i][:, 3, :] = OP[i][:, 3, :] - np.tile(OP_mean[i][:, 3][:, np.newaxis], (1, n_con_each[i])) # OP[i][0:3,3,:] - np.tile(OP_mean[i][:, 3][:, np.newaxis], (1, n_con_each[i]))
+        op_zeromean[i][:, 3, :] = op[i][:, 3, :] - np.tile(op_mean[i][:, 3][:, np.newaxis], (1, n_con_each[i]))
+        # OP[i][0:3,3,:] - np.tile(OP_mean[i][:, 3][:, np.newaxis], (1, n_con_each[i]))
 
         # perform SVD to determine principal axis
-        U[i], s[i], V[i] = np.linalg.svd(OP_zeromean[i][:, 3, :].T, full_matrices=True)
+        u[i], s[i], v[i] = np.linalg.svd(op_zeromean[i][:, 3, :].T, full_matrices=True)
 
         # project locations on principal axis
-        P_transform[i] = np.dot(OP_zeromean[i][:, 3, :].T, V[i].T)
+        p_transform[i] = np.dot(op_zeromean[i][:, 3, :].T, v[i].T)
 
         # rotate coil orientations to mean orientation (zero-mean)
-        OP_zeromean[i][:, 0:3, :] = np.dot(OP[i][:, 0:3, :].transpose(2, 1, 0), OP_mean[i][:, 0:3]).transpose(2, 1, 0)
+        op_zeromean[i][:, 0:3, :] = np.dot(op[i][:, 0:3, :].transpose(2, 1, 0), op_mean[i][:, 0:3]).transpose(2, 1, 0)
 
         # Determine quaternions or Euler angles
         orientations[i] = np.zeros((n_con_each[i], 3))
+        xlabel, ylabel = '', ''
 
         for j in range(n_con_each[i]):
-
             if orientation == 'quaternions':
-                orientations[i][j, :] = rot_to_quat(OP_zeromean[i][:, 0:3, j])[1:]
+                orientations[i][j, :] = pynibs.rot_to_quat(op_zeromean[i][:, 0:3, j])[1:]
                 ori_names = ['xq', 'yq', 'zq']
                 xlabel = ['$x_q$', '$y_q$', '$z_q$']
                 ylabel = ['$p(x_q)$', '$p(y_q)$', '$p(z_q)$']
 
             elif orientation == 'euler':
-                orientations[i][j, :] = rotation_matrix_to_euler_angles(OP_zeromean[i][:, :, j]) * 180.0 / np.pi
+                orientations[i][j, :] = pynibs.rotation_matrix_to_euler_angles(op_zeromean[i][:, :, j]) * 180.0 / np.pi
                 ori_names = ['Psi', 'Theta', 'Phi']
                 xlabel = ['$\Psi$', '$\Theta$', '$\Phi$']
                 ylabel = ['$p(\Psi)$', '$p(\Theta)$', '$p(\Phi)$']
@@ -326,7 +317,7 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
 
         # fit beta pdfs to x', y' and z' locations and orientations
         for j, name in enumerate(pos_names):
-            pdf_paras_location[i][j] = pygpc.fit_betapdf(P_transform[i][:, j],
+            pdf_paras_location[i][j] = pygpc.fit_betapdf(p_transform[i][:, j],
                                                          BETATOL=0.05,
                                                          PUNI=0.9,
                                                          PLOT=make_pdf_plot,
@@ -353,7 +344,7 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
                                                                                         name +
                                                                                         '_betafit'))
 
-    return pdf_paras_location, pdf_paras_orientation_euler, OP_mean, OP_zeromean, V, P_transform, orientations
+    return pdf_paras_location, pdf_paras_orientation_euler, op_mean, op_zeromean, v, p_transform, orientations
 
 
 def test_coil_position_gpc(parameters):
@@ -370,7 +361,7 @@ def test_coil_position_gpc(parameters):
     from pygpc.RandomParameter import RandomParameter
 
     # load subject
-    subject = load_subject(parameters["fn_subject"])
+    subject = pynibs.load_subject(parameters["fn_subject"])
 
     coil_uncertainty = False
     parameters_random = OrderedDict()
@@ -382,7 +373,7 @@ def test_coil_position_gpc(parameters):
             if p in ["x", "y", "z", "psi", "theta", "phi"]:
                 coil_uncertainty = True
 
-    dim = len(parameters_random)
+    # dim = len(parameters_random)
 
     if coil_uncertainty:
 
@@ -398,7 +389,7 @@ def test_coil_position_gpc(parameters):
             triangles = node_number_list[elm_type == 2, 0:3]
 
         triangles = triangles[triangles_regions == 5]
-        surface_points = unique_rows(np.reshape(points[triangles], (3 * triangles.shape[0], 3)))
+        surface_points = pynibs.unique_rows(np.reshape(points[triangles], (3 * triangles.shape[0], 3)))
         limits_scaling_factor = .1
 
         # Generate Delaunay triangulation object
@@ -461,8 +452,8 @@ def test_coil_position_gpc(parameters):
             parameters_corr[parameters_corr_idx[param_to_change_idx]]['pdf_limits'] = limits_new
 
             print((' > Changing pdf_limits of {} from {} -> {}'.format(parameters_corr_idx[param_to_change_idx],
-                                                                      limits_old,
-                                                                      limits_new)))
+                                                                       limits_old,
+                                                                       limits_new)))
 
             # overwrite old values
             parameters[parameters_corr_idx[param_to_change_idx]].pdf_limits = limits_new
@@ -477,7 +468,7 @@ def test_coil_position_gpc(parameters):
     return parameters
 
 
-def get_invalid_coil_parameters(self, param_dict, coil_position_mean, svd_v, del_obj, fn_coil,
+def get_invalid_coil_parameters(param_dict, coil_position_mean, svd_v, del_obj, fn_coil,
                                 fn_hdf5_coilpos=None):
     """
     Finds gpc parameter combinations, which place coil dipoles inside subjects head.
@@ -495,7 +486,7 @@ def get_invalid_coil_parameters(self, param_dict, coil_position_mean, svd_v, del
     svd_v: nparray [3 x 3]
         SVD matrix V
     del_obj: Delaunay object
-        Delaunay object of the skin surface (created with scipy.spatial.Delaunay)
+        Skin surface (created with scipy.spatial.Delaunay)
     fn_coil: str
         Filename of coil .ccd file
     fn_hdf5_coilpos: str
@@ -518,12 +509,12 @@ def get_invalid_coil_parameters(self, param_dict, coil_position_mean, svd_v, del
     limits_theta = param_dict['theta']['limits']
     limits_phi = param_dict['phi']['limits']
 
-    limits_pos_x = add_center(limits_pos_x)
-    limits_pos_y = add_center(limits_pos_y)
-    limits_pos_z = add_center(limits_pos_z)
-    limits_psi = add_center(limits_psi)
-    limits_theta = add_center(limits_theta)
-    limits_phi = add_center(limits_phi)
+    limits_pos_x = pynibs.add_center(limits_pos_x)
+    limits_pos_y = pynibs.add_center(limits_pos_y)
+    limits_pos_z = pynibs.add_center(limits_pos_z)
+    limits_psi = pynibs.add_center(limits_psi)
+    limits_theta = pynibs.add_center(limits_theta)
+    limits_phi = pynibs.add_center(limits_phi)
 
     temp_list = [[0, 1, 2], [0, 1, 2], [0, 1, 2], [0, 1, 2], [0, 1, 2], [0, 1, 2]]
     combinations = list(itertools.product(*temp_list))
@@ -549,7 +540,6 @@ def get_invalid_coil_parameters(self, param_dict, coil_position_mean, svd_v, del
         coil_dipoles = get_coil_dipole_pos(fn_coil, mat)
 
         if fn_hdf5_coilpos:
-
             with h5py.File(fn_hdf5_coilpos, 'w') as f:
                 f.create_dataset("/dipoles/", data=coil_dipoles)
 
@@ -584,7 +574,7 @@ def get_invalid_coil_parameters(self, param_dict, coil_position_mean, svd_v, del
                 f.write('</Xdmf>\n')
 
         # check hull and add to results
-        results.append(in_hull(coil_dipoles, del_obj))
+        results.append(pynibs.in_hull(coil_dipoles, del_obj))
 
     # find parameter which drives dipoles into head
     combinations = np.array(combinations)
@@ -621,7 +611,7 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
         If true, only one coil center per site is generated.
     fix_angles : boolean
         rename 22.5 -> 0, 0 -> -45, 67.5 -> 90, 90 -> 135
-    data_dict : dict of ndarray of float [n_stimsites] (optional), default: None
+    data_dict : dict ofnp.ndarray of float [n_stimsites] (optional), default: None
         Dictionary containing data corresponding to the stimulation sites (keys)
     conditions_ignored : str or list of str, Default=None
         Conditions, which are not going to be included in the plot
@@ -639,9 +629,9 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
     """
     assert not fn_hdf.endswith('/')
 
-    exp = read_csv(fn_exp)
+    exp = pynibs.read_csv(fn_exp)
 
-    exp_cond = sort_by_condition(exp, conditions_selected=conditions_selected)  # []
+    exp_cond = pynibs.sort_by_condition(exp, conditions_selected=conditions_selected)  # []
 
     # get the unique conditions in the correct order
     conds = [c['condition'][0] for c in exp_cond]
@@ -739,6 +729,8 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
     f.create_dataset('angles', data=angles)
     f.create_dataset('sites', data=np.string_(sites))  # this is a string array, not xdmf compatible
     f.create_dataset('sites_idx', data=sites_idx)
+
+    data = None
     if data_dict is not None:
         data = np.zeros((len(list(data_dict.keys())), 1))
         for i_data, cond in enumerate(conds):
@@ -816,6 +808,7 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
     f.write('</Xdmf>\n')
     f.close()
 
+
 def create_stimsite_from_list(fn_hdf, poslist, datanames=None, data=None, overwrite=False):
     """This takes a TMSLIST from simnibs and creates a .hdf5 + .xdmf tuple for all positions.
 
@@ -946,21 +939,22 @@ def create_stimsite_from_matsimnibs(fn_hdf, matsimnibs, datanames=None, data=Non
 
     Parameters
     -----------
-    fn_hdf: basestring
+    fn_hdf: string
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
-    matsimnibs: ndarray [4 x 4 x n_pos]
+    matsimnibs: np.ndarray
+        (4, 4, n_pos)
         Matsimnibs matrices containing the coil orientation (x,y,z) and position (p)
-        [ |  |  |  | ]
-        [ x  y  z  p ]
-        [ |  |  |  | ]
-        [ 0  0  0  1 ]
-    datanames: basestring or list of basestring
+            [ |  |  |  | ]
+            [ x  y  z  p ]
+            [ |  |  |  | ]
+            [ 0  0  0  1 ]
+    datanames: string or list of string
         Dataset names for _data_. Default: None.
-    data: np.ndarray
-        Dataset array with (len(poslist.pos), len(datanames()). Default: None.
-    overwrite : boolean
-        Overwrite existing files. Default:  False.
+    data: np.ndarray, optional
+        (len(poslist.pos), len(datanames).
+    overwrite : boolean, default: False
+        Overwrite existing files.
     """
     n_pos = matsimnibs.shape[2]
     centers = np.zeros((n_pos, 3))
@@ -988,13 +982,13 @@ def write_coil_pos_hdf5(fn_hdf, centers, m0, m1, m2, datanames=None, data=None,
     fn_hdf : basestring
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
-    centers : ndarray of float [n_pos x 3]
+    centers : np.ndarray of float [n_pos x 3]
         Coil positions
-    m0 : ndarray of float [n_pos x 3]
+    m0 : np.ndarray of float [n_pos x 3]
         Coil orientation x-axis (looking at the active (patient) side of the coil pointing to the right)
-    m1 : ndarray of float [n_pos x 3]
+    m1 : np.ndarray of float [n_pos x 3]
         Coil orientation y-axis (looking at the active (patient) side of the coil pointing up away from the handle)
-    m2 : ndarray of float [n_pos x 3]
+    m2 : np.ndarray of float [n_pos x 3]
         Coil orientation z-axis (looking at the active (patient) side of the coil pointing to the patient)
     datanames : basestring or list of basestring [n_data]
         Dataset names for _data_. Default: None.
@@ -1098,7 +1092,8 @@ def write_coil_pos_hdf5(fn_hdf, centers, m0, m1, m2, datanames=None, data=None,
     f.close()
 
 
-def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root_path="/0/0/", verbose=False, print_output=False):
+def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root_path="/0/0/", verbose=False,
+                            print_output=False):
     """
     Sorts coil positions according to Traveling Salesman problem
 
@@ -1120,6 +1115,9 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
     <file> .hdf5 file containing the sorted optimal coil position indices
     """
 
+    from ortools.constraint_solver import routing_enums_pb2
+    from ortools.constraint_solver import pywrapcp
+
     if verbose:
         print(f"Loading optimal coil indices from file: {fn_coil_pos_opt}")
 
@@ -1147,7 +1145,7 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
     pos_unique = np.unique(matsimnibs_opt[0, 3, :])
 
     for i, pos in enumerate(pos_unique):
-        unique_idx[matsimnibs_opt[0, 3, :]==pos] = i
+        unique_idx[matsimnibs_opt[0, 3, :] == pos] = i
 
     if verbose:
         print(f"Sorting same positions according to angles ...")
@@ -1161,12 +1159,12 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 
         if pos.shape[2] > 1:
             for i_p in range(pos.shape[2]):
-                    dot_prod = np.dot(pos[:3, 0, 0], pos[:3, 0, i_p])
-                    if dot_prod > 1:
-                        dot_prod = 1
-                    elif dot_prod < -1:
-                        dot_prod = -1
-                    angles[i_p] = np.arccos(dot_prod)/np.pi * 180
+                dot_prod = np.dot(pos[:3, 0, 0], pos[:3, 0, i_p])
+                if dot_prod > 1:
+                    dot_prod = 1
+                elif dot_prod < -1:
+                    dot_prod = -1
+                angles[i_p] = np.arccos(dot_prod) / np.pi * 180
 
             angles_sort_idx = np.argsort(angles)
             pos_sorted = pos[:, :, angles_sort_idx]
@@ -1175,7 +1173,7 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 
         matsimnibs_opt_sorted[:, :, unique_idx == i] = pos_sorted
         matsimnibs_opt_sorted_pert[:, :, unique_idx == i] = pos_sorted
-        matsimnibs_opt_sorted_pert[0, 3, unique_idx == i] += np.arange(len(angles))*1e-2
+        matsimnibs_opt_sorted_pert[0, 3, unique_idx == i] += np.arange(len(angles)) * 1e-2
 
     coords = matsimnibs_opt_sorted_pert[:3, 3, :].transpose()
     coords_list = [tuple(c) for c in coords]
@@ -1191,24 +1189,24 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
                 else:
                     # Euclidean distance
                     distances[from_counter][to_counter] = (int(
-                        math.hypot((from_node[0] - to_node[0]),
-                                   (from_node[1] - to_node[1]))))
+                            math.hypot((from_node[0] - to_node[0]),
+                                       (from_node[1] - to_node[1]))))
         return distances
 
-    def get_routes(solution, routing, manager):
+    def get_routes(s, r, m):
         """Get vehicle routes from a solution and store them in an array."""
-        # Get vehicle routes and store them in a two dimensional array whose
+        # Get vehicle routes and store them in a two-dimensional array whose
         # i,j entry is the jth location visited by vehicle i along its route.
-        routes = []
-        for route_nbr in range(routing.vehicles()):
-            index = routing.Start(route_nbr)
-            route = [manager.IndexToNode(index)]
-            while not routing.IsEnd(index):
-                index = solution.Value(routing.NextVar(index))
-                route.append(manager.IndexToNode(index))
-            routes.append(route)
+        routes_l = []
+        for route_nbr in range(r.vehicles()):
+            index = r.Start(route_nbr)
+            route = [m.IndexToNode(index)]
+            while not r.IsEnd(index):
+                index = s.Value(r.NextVar(index))
+                route.append(m.IndexToNode(index))
+            routes_l.append(route)
 
-        return routes
+        return routes_l
 
     def distance_callback(from_index, to_index):
         """Returns the distance between the two nodes."""
@@ -1237,7 +1235,7 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 
     # Setting first solution heuristic.
     search_parameters = pywrapcp.DefaultRoutingSearchParameters()
-    search_parameters.first_solution_strategy = (routing_enums_pb2.FirstSolutionStrategy.PATH_CHEAPEST_ARC)
+    search_parameters.first_solution_strategy = routing_enums_pb2.FirstSolutionStrategy.PATH_CHEAPEST_ARC
 
     # Solve the problem.
     if verbose:
@@ -1264,17 +1262,18 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
     if fn_out_hdf5 is not None:
         outpath = fn_out_hdf5
         shutil.copy(fn_coil_pos_opt, fn_out_hdf5)
-    
+
     if verbose:
         print(f"Saving output to: {outpath}")
 
     with h5py.File(outpath, "a") as f:
-        f[root_path+"coil_seq"][:] = np.vstack((idx_global_sorted, goal_fun_val_sorted, intensities_opt_sorted)).transpose() 
-    
+        f[root_path + "coil_seq"][:] = np.vstack((idx_global_sorted, goal_fun_val_sorted,
+                                                  intensities_opt_sorted)).transpose()
+
     if print_output:
         fig = plt.figure()
         ax = fig.add_subplot(projection='3d')
-        ax.scatter(coords[routes, 0], coords[routes, 1], coords[routes, 2], s=intensities_opt_sorted*11, c="k")
+        ax.scatter(coords[routes, 0], coords[routes, 1], coords[routes, 2], s=intensities_opt_sorted * 11, c="k")
         ax.plot3D(coords[routes, 0], coords[routes, 1], coords[routes, 2])
         ax.set_xlabel("x in mm")
         ax.set_ylabel("y in mm")
@@ -1290,9 +1289,9 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 #     -----------
 #     problem : Problem class instance
 #         GPC problem under investigation
-#     coil_position_mean : ndarray of float [3 x 4]
+#     coil_position_mean : np.ndarray of float [3 x 4]
 #         Mean coil position matrix containing the 3 orientation vectors and the location [ori_x, ori_y, ori_z, loc]
-#     svd_v: ndarray of float [3 x 3]
+#     svd_v: np.ndarray of float [3 x 3]
 #         List containing the SVD matrices V for each condition (principal axes of orientation)
 #
 #     Returns:
@@ -1312,7 +1311,7 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 #         triangles = node_number_list[elm_type == 2, 0:3]
 #
 #     triangles = triangles[triangles_regions == 5]
-#     surface_points = pynibs.unique_rows(np.reshape(points[triangles], (3 * triangles.shape[0], 3)))
+#     surface_points = pynibs.pynibs.unique_rows(np.reshape(points[triangles], (3 * triangles.shape[0], 3)))
 #     limits_scaling_factor = .1
 #
 #     # Generate Delaunay triangulation object
@@ -1387,8 +1386,6 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 #
 #     return param_dict
 
-
-
 # def calc_coil_position_pdf(fn_rescon, fn_simpos, folder_pdfplots):
 #     """ Determines the probability density functions of the transformed coil position (x', y', z') and Euler angles of
 #     the coil orientations (Psi, Theta, Phi)
@@ -1430,9 +1427,10 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 #         folder_pdfplots = ''
 #
 #     # load recorded positions and orientations
-#     positions_all, conditions, position_list, _, _ = read_exp_stimulations(fn_rescon, fn_simpos)
+#     positions_all, conditions, position_list, _, _ = pynibs.read_exp_stimulations(fn_rescon, fn_simpos)
 #
-#     # sort POSITIONS according to CONDITIONS, idx_con is alphabetically sorted, first index of condition[*] appeareance
+#     # sort POSITIONS according to CONDITIONS, idx_con is alphabetically sorted, first index of condition[*]
+#     appeareance
 #     conditions_unique, idx_con, n_con_each = np.unique(conditions,
 #                                                        return_index=True,
 #                                                        return_inverse=False,
@@ -1554,7 +1552,8 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 #     #
 #     # fig = plt.figure()
 #     # ax = fig.add_subplot(111, projection='3d')
-#     # ax.scatter(ORI_X_transform[0][:,0], ORI_X_transform[0][:,1], ORI_X_transform[0][:,2] , zdir='z', s=20, c=[1,1,0],
+#     # ax.scatter(ORI_X_transform[0][:,0], ORI_X_transform[0][:,1], ORI_X_transform[0][:,2] , zdir='z',
+#     s=20, c=[1,1,0],
 #     #  depthshade=True)
 #     # fig.savefig(folder + 'ORI_X_transform_0.png')
 #
@@ -1576,7 +1575,7 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
 #             R = np.hstack((ori_x_transform[i][j, :][:, np.newaxis],
 #                            ori_y_transform[i][j, :][:, np.newaxis],
 #                            ori_z_transform[i][j, :][:, np.newaxis]))
-#             euler_angles[i][j, :] = rotation_matrix_to_euler_angles(R) * 180.0 / np.pi
+#             euler_angles[i][j, :] = pynibs.rotation_matrix_to_euler_angles(R) * 180.0 / np.pi
 #             r_all.append(R)
 #         for j, name in enumerate(angle_names):
 #             np.random.seed(5)
