@@ -1,17 +1,18 @@
-import os
 import re
-import sys
+import warnings
+
 import pylab
 import pickle
 import datetime
 import numpy as np
 import pandas as pd
 from scipy.signal import butter, lfilter
+import scipy.io as spio
 from pynibs.util import get_cartesian_product
 from pynibs.util import generalized_extreme_value_distribution
-# from pynibs.exp.exp import testfun
 from lmfit import Model
 import matplotlib.pyplot as plt
+
 __package__ = "pynibs"
 
 try:
@@ -296,8 +297,9 @@ def exp0_log(x, x0, r):
 
 
 def get_mep_virtual_subject_TVS(x, p1=-5.0818, p2=-2.4677, p3=3.6466, p4=0.42639, p5=1.6665,
-                                mu_y_add=10**-5.0818, mu_y_mult=-0.9645334, mu_x_add=0.68827324,
-                                sigma_y_add=1.4739*1e-6, k=0.39316, sigma2_y_mult=2.2759*1e-2, sigma2_x_add=2.3671*1e-2,
+                                mu_y_add=10 ** -5.0818, mu_y_mult=-0.9645334, mu_x_add=0.68827324,
+                                sigma_y_add=1.4739 * 1e-6, k=0.39316, sigma2_y_mult=2.2759 * 1e-2,
+                                sigma2_x_add=2.3671 * 1e-2,
                                 subject_variability=False, trial_variability=True):
     """
     Creates random mep data using the 3 variability source model from Goetz et al. 2019.
@@ -351,31 +353,31 @@ def get_mep_virtual_subject_TVS(x, p1=-5.0818, p2=-2.4677, p3=3.6466, p4=0.42639
     mep : ndarray of float [n_x]
         Motor evoked potential values in mV
     """
-    x = x*100
+    x = x * 100
     n_x = len(x)
-    mu_y_add = 10**p1
+    mu_y_add = 10 ** p1
 
     # determine variabilities
     if trial_variability:
-        e_y_mult = 10**np.random.normal(loc=mu_y_mult, scale=np.sqrt(sigma2_y_mult), size=n_x)
-        e_x_add = 10**np.random.normal(loc=mu_x_add, scale=np.sqrt(sigma2_x_add), size=n_x)
+        e_y_mult = 10 ** np.random.normal(loc=mu_y_mult, scale=np.sqrt(sigma2_y_mult), size=n_x)
+        e_x_add = 10 ** np.random.normal(loc=mu_x_add, scale=np.sqrt(sigma2_x_add), size=n_x)
 
         # determine generalized value distribution of additive y variability
         p_e_y_add = generalized_extreme_value_distribution(x=np.linspace(5e-6, 1e-4, 100000),
                                                            mu=mu_y_add, sigma=sigma_y_add, k=k)
 
         # sample from generalized value distribution to determine additive y variability
-        e_y_add = np.random.choice(np.linspace(5e-6, 1e-4, 100000), p=p_e_y_add/np.sum(p_e_y_add), size=n_x)
+        e_y_add = np.random.choice(np.linspace(5e-6, 1e-4, 100000), p=p_e_y_add / np.sum(p_e_y_add), size=n_x)
 
     else:
-        e_y_mult = np.ones(n_x) * 10**mu_y_mult
-        e_x_add = np.ones(n_x) * 10**mu_x_add
+        e_y_mult = np.ones(n_x) * 10 ** mu_y_mult
+        e_x_add = np.ones(n_x) * 10 ** mu_x_add
         e_y_add = np.ones(n_x) * mu_y_add
 
-    x_prime = 2.224*1e-16 + x-10**p5+e_x_add
-    x_prime[x_prime < 0] = 2.224*1e-16
+    x_prime = 2.224 * 1e-16 + x - 10 ** p5 + e_x_add
+    x_prime[x_prime < 0] = 2.224 * 1e-16
 
-    mep = e_y_add + np.exp(np.log(10) * (e_y_mult - 7 + (p2 + 7)/(1+(10**p3)/(x_prime**(10**p4)))))
+    mep = e_y_add + np.exp(np.log(10) * (e_y_mult - 7 + (p2 + 7) / (1 + (10 ** p3) / (x_prime ** (10 ** p4)))))
     mep = mep * 1000
 
     return mep
@@ -433,7 +435,7 @@ def get_mep_virtual_subject_DVS(x, x0=0.5, r=10, amp=1, sigma_x=0, sigma_y=0, y0
 
     x_pert = x + e_x_add
 
-    mep = 10**(sigmoid4_log(x=x_pert, x0=x0, r=r, amp=amp, y0=y0) + e_y_add)
+    mep = 10 ** (sigmoid4_log(x=x_pert, x0=x0, r=r, amp=amp, y0=y0) + e_y_add)
 
     return mep
 
@@ -607,7 +609,7 @@ class Mep:
             # re-fit if std of any parameter is > delta
             if ((self.pstd / self.popt) > delta).any() and i_try < 3:
                 print('Unsuccessful fit ... relative STD of one parameter is > {}% ... trying next function!' \
-                    .format(delta * 100))
+                      .format(delta * 100))
                 i_try = i_try + 1
                 continue
 
@@ -618,7 +620,7 @@ class Mep:
         # determine motor threshold considering a threshold of 50 uV
         self.calc_motor_threshold(threshold=0.05)
 
-    def fit_mep_to_function_multistart(self, p0=None, constraints=None, fun=[sigmoid, exp0, linear]):
+    def fit_mep_to_function_multistart(self, p0=None, constraints=None, fun=None):
         """
         Fits MEP data to function.
 
@@ -656,6 +658,8 @@ class Mep:
         # intensities = intensities[omask] #+ np.random.randn(intensities.size)*1e-6
         # mep = mep[omask]
 
+        if fun is None:
+            fun = [sigmoid, exp0, linear]
         if p0 is None:
             p0 = []
 
@@ -741,7 +745,7 @@ class Mep:
             Dependent data the curve is fitted through
         p0 : nparray of float
             Initial guess of parameter values
-        constraints : dict
+        constraints : dict, optional
             Dictionary with parameter names as keys and [min, max] values as constraints
         verbose : bool
             Show output messages (True / False (Default))
@@ -1115,7 +1119,7 @@ def load_mepobj(fname):
 def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=0, channels=None, time_format="delta",
                      plot=False):
     """
-    Read mep data from .cfs file.
+    Read EMG data from CED .cfs or .txt file and returns MEP amplitudes.
 
     Parameters
     ----------
@@ -1137,16 +1141,18 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
 
     Returns
     -------
-    p2p_array : np.array of float [N_stim]
+    p2p_array : np.ndarray of float [N_stim]
         Peak to peak values of N sweeps
     time_mep_lst : list of datetime.timedelta
-        Mep-timestamps
-    mep_raw_data : np.array of float [N_channel x N_stim x N_samples]
+        MEP-timestamps
+    mep_raw_data : np.ndarray of float [N_channel x N_stim x N_samples]
         Raw (unfiltered) MEP data
-    mep_filt_data : np.array of float [N_channel x N_stim x N_samples]
+    mep_filt_data : np.ndarray of float [N_channel x N_stim x N_samples]
         Filtered MEP data (Butterworth lowpass filter)
-    time : np.array of float [N_samples]
+    time : np.ndarray of float [N_samples]
         Time axis corresponding to MEP data
+    mep_onset_array : np.ndarray of float [S_samples]
+        MEP onset after TMS pulse
     """
     # convert pulse time to datetime object in case of "delta"
     if time_format == "delta":
@@ -1156,7 +1162,6 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
     else:
         raise NotImplementedError("Specified time_format not implemented yet...")
 
-    print(mep_fn)
     if mep_fn.endswith('.cfs'):
         # get data from cfs file
         import biosig
@@ -1193,30 +1198,46 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
             pass
 
         # add first timestamp (not saved by signal) and shift other by isi
-        time_mep_list = [datetime.timedelta(seconds=0)] + [t + time_mep_list[1]-time_mep_list[0] for t in time_mep_list]
+        time_mep_list = [datetime.timedelta(seconds=0)] + [t + time_mep_list[1] - time_mep_list[0] for t in
+                                                           time_mep_list]
 
-        # get peal-to-peak values
+        # get peak-to-peak values
         # get the ratio of samples per sweep
         sweep_index = cfs_header.find('NumberOfSweeps')
         comma_index = cfs_header.find(',', sweep_index)
-        sweeps = int(cfs_header[sweep_index + 18:comma_index])
+        n_sweeps = int(cfs_header[sweep_index + 18:comma_index])
         record_index = cfs_header.find('NumberOfRecords')
         comma_index = cfs_header.find(',', record_index)
         records = int(cfs_header[record_index + 19:comma_index])
-        samples = int(records / sweeps)
-        if not isinstance(samples, int):
+        n_samples = int(records / n_sweeps)
+        if not isinstance(n_samples, int):
             print('Warning: Number of samples is not an integer.')
             # TODO: Correct get_mep_elements() sample number check. This does not work as expected (from Ole)
 
         # reshape numpy array
-        mep_raw_data = np.zeros((len(cfs_data_column), sweeps, samples))
+        mep_raw_arr = np.zeros((len(cfs_data_column), n_sweeps, n_samples))
 
         for i in cfs_data_column:
-            mep_raw_data[i, :, :] = np.reshape(mep_raw_data_tmp[:, i], (sweeps, samples))
+            mep_raw_arr[i, :, :] = np.reshape(mep_raw_data_tmp[:, i], (n_sweeps, n_samples))
 
         sampling_rate = get_mep_sampling_rate(mep_fn)
 
+    elif mep_fn.endswith('.mat'):
+        mep_data = spio.loadmat(mep_fn, struct_as_record=False, squeeze_me=True)
+
+        # find data
+        for k in mep_data.keys():
+            if isinstance(mep_data[k], spio.matlab.mio5_params.mat_struct):
+                mep_data = mep_data[k].__dict__
+                break
+
+        n_samples = mep_data['points']
+        mep_raw_arr = mep_data['values'].transpose(1, 2, 0)
+        time_mep_list = [datetime.timedelta(seconds=f.__dict__['start']) for f in mep_data['frameinfo']]
+        sampling_rate = get_mep_sampling_rate(mep_fn)
+
     elif mep_fn.endswith('.txt'):
+        warnings.warn(".txt import is deprecated - use .mat or .cfs.", DeprecationWarning)
         print("Reading MEP from .txt file")
         # The Signal text export looks like this:
         #
@@ -1233,76 +1254,76 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
         # With first column = time, second = 1st electrode, ...
         # This is an example of 2 sweeps, 3 samples each, sampling rate = 4000
 
+        # Find number of samples per sweep
         pattern = '"s"'
-        # with open(fn) as f:
         with open(mep_fn, 'r') as f:
             for line_nr, line in enumerate(f):
+                print(f'{line_nr}: {line}')
                 if pattern in line and line_nr > 0:
                     # find second occurance of "s" -> end of first sweep
-                    samples = line_nr
-                    break
+                    n_samples = line_nr
+                    print(f'{line_nr}: {line}')
                 if line != '\n':
-                    last_sweep = line
-            # raise ValueError("Signal file not readable.")
-        last_sweep = float(last_sweep[0:last_sweep.find('\t')])
+                    last_sample_time = line
+
+        # extract time (first column) of last samples
+        last_sample_time = float(last_sample_time[0:last_sample_time.find('\t')])
 
         # subtract 2 because first row is header ("s"\t"ADC 0"\t"ADC 1") and last row is blank
-        samples = samples - 2
+        n_samples = n_samples - 2
 
         df_mep = pd.read_csv(mep_fn,
                              delimiter="\t",
                              skip_blank_lines=True,
-                             skiprows=lambda x: x % (samples+2) == 0 and x > 0)
+                             skiprows=lambda x: x % (n_samples + 2) == 0 and x > 0)
 
-        sweeps = int(df_mep.shape[0] / samples)
-        mep_raw_data = np.zeros((len(cfs_data_column), sweeps, samples))
+        n_sweeps = int(df_mep.shape[0] / n_samples)
+        mep_raw_arr = np.zeros((len(cfs_data_column), n_sweeps, n_samples))
 
-        for i in range(sweeps):
-            mep_raw_data[:, i, :] = df_mep.iloc[i*samples:(i+1)*samples, 1:].transpose()
+        for i in range(n_sweeps):
+            mep_raw_arr[:, i, :] = df_mep.iloc[i * n_samples:(i + 1) * n_samples, 1:].transpose()
 
         # get sampling rate by dividing number of sweeps by timing
-        sampling_rate = (mep_raw_data.shape[2] - 1) / last_sweep
-        assert np.isclose(sampling_rate, round(sampling_rate))
-        sampling_rate = int(sampling_rate)
+        sampling_rate = int(mep_raw_arr.shape[2] - 1) / last_sample_time
 
         # build time_mep_list
         # we only have information about the single mep timings, so let's assume signal sticks strictly to the protocol
-        sample_len = last_sweep + 1/sampling_rate
+        sample_len = last_sample_time + 1 / sampling_rate
 
         # TODO: The ISI is missing here, do we want to add it to the subject object?
-        time_mep_list = [datetime.timedelta(seconds=i*sample_len) +
-                         tms_pulse_timedelta for i in range(mep_raw_data.shape[0])]
+        time_mep_list = [datetime.timedelta(seconds=i * sample_len) +
+                         tms_pulse_timedelta for i in range(mep_raw_arr.shape[1])]
 
     else:
-        raise ValueError("Unknown MEP file extension. Use .cfs or .txt .")
+        raise ValueError("Unknown MEP file extension. Use .cfs or .txt.")
 
     # get peak to peak value of every sweep and plot results in mep/plots/channels
     if channels is None:
         channels = [str(i) for i in cfs_data_column]
 
-    tmp = np.zeros((mep_raw_data.shape[0], mep_raw_data.shape[1], 2)).astype(object)
-    for i_channel in range(mep_raw_data.shape[0]):
+    tmp = np.zeros((mep_raw_arr.shape[0], mep_raw_arr.shape[1], 3)).astype(object)
+    for i_channel in range(mep_raw_arr.shape[0]):
         print(f"Calculating p2p values for channel: {channels[i_channel]}")
 
-        for i_zap in range(mep_raw_data.shape[1]):
-            tmp[i_channel, i_zap, 0], tmp[i_channel, i_zap, 1] = calc_p2p(sweep=mep_raw_data[i_channel, i_zap, :],
-                                                                          tms_pulse_time=tms_pulse_time,
-                                                                          sampling_rate=sampling_rate,
-                                                                          fn_plot=None)
+        for i_zap in range(mep_raw_arr.shape[1]):
+            tmp[i_channel, i_zap, 0], \
+            tmp[i_channel, i_zap, 1], \
+            tmp[i_channel, i_zap, 2] = calc_p2p(sweep=mep_raw_arr[i_channel, i_zap, :],
+                                                tms_pulse_time=tms_pulse_time,
+                                                sampling_rate=sampling_rate,
+                                                fn_plot=None)
 
-    # tmp = np.apply_along_axis(calc_p2p, axis=2, arr=mep_raw_data,
-    #                           tms_pulse_time=tms_pulse_time,
-    #                           sampling_rate=sampling_rate)
+    p2p_arr = np.zeros((tmp.shape[0], tmp.shape[1]))
+    mep_onset_arr = np.zeros((tmp.shape[0], tmp.shape[1]))
+    mep_filt_arr = np.zeros(mep_raw_arr.shape)
 
-    p2p_array = np.zeros((tmp.shape[0], tmp.shape[1]))
-    mep_filt_data = np.zeros(mep_raw_data.shape)
+    time = np.arange(mep_raw_arr.shape[2]) / sampling_rate
 
-    time = np.arange(mep_raw_data.shape[2])/sampling_rate
-
-    for i_c in cfs_data_column:
-        for i, t in enumerate(tmp[i_c, :, :]):
-            p2p_array[i_c, i] = tmp[i_c, i, 0]
-            mep_filt_data[i_c, i, :] = tmp[i_c, i, 1]
+    for idx_channel in cfs_data_column:
+        for i, t in enumerate(tmp[idx_channel, :, :]):
+            p2p_arr[idx_channel, i] = tmp[idx_channel, i, 0]
+            mep_onset_arr[idx_channel, i] = tmp[idx_channel, i, 2]
+            mep_filt_arr[idx_channel, i, :] = tmp[idx_channel, i, 1]
 
     if time_format == "delta":
         time_mep_list = [time_mep_list[i] - time_mep_list[0] for i in range(len(time_mep_list))]
@@ -1311,17 +1332,18 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
 
     # remove MEPs according to drop_mep_idx and reset time
     if drop_mep_idx is not None:
-        p2p_array = np.delete(p2p_array, drop_mep_idx)
+        p2p_arr = np.delete(p2p_arr, drop_mep_idx)
+        mep_onset_arr = np.delete(mep_onset_arr, drop_mep_idx)
         time_mep_list = np.delete(time_mep_list, drop_mep_idx)
 
-    keep_mep_idx = [i for i in range(mep_raw_data.shape[1]) if i not in np.array(drop_mep_idx)]
-    mep_raw_data = mep_raw_data[:, keep_mep_idx, :]
-    mep_filt_data = mep_filt_data[:, keep_mep_idx, :]
+    keep_mep_idx = [i for i in range(mep_raw_arr.shape[1]) if i not in np.array(drop_mep_idx)]
+    mep_raw_arr = mep_raw_arr[:, keep_mep_idx, :]
+    mep_filt_arr = mep_filt_arr[:, keep_mep_idx, :]
 
-    return [p2p_array, time_mep_list, mep_raw_data, mep_filt_data, time]
+    return p2p_arr, time_mep_list, mep_raw_arr, mep_filt_arr, time, mep_onset_arr
 
 
-def calc_p2p_old_exp0(sweep, start_mep=None, end_mep=None, tms_pulse_time = None, sampling_rate=None):
+def calc_p2p_old_exp0(sweep, start_mep=None, end_mep=None, tms_pulse_time=None, sampling_rate=None):
     """
     Calc peak-to-peak values of an mep sweep.
     This version was probably used in the ancient times of experiment 0.
@@ -1349,7 +1371,7 @@ def calc_p2p_old_exp0(sweep, start_mep=None, end_mep=None, tms_pulse_time = None
     p2p : float
         Peak-to-peak value of input curve
     """
-    raise DeprecationWarning("Use calc_p2p(). calc_p2p_old_exp0 only used for data-reproducibility")
+    warnings.warn(DeprecationWarning("Use calc_p2p(). calc_p2p_old_exp0 only used for data-reproducibility"))
 
     # Filter requirements.
     order = 6
@@ -1379,7 +1401,7 @@ def calc_p2p_old_exp0(sweep, start_mep=None, end_mep=None, tms_pulse_time = None
 
     index_max_end = sweep_filt.size  # int(0.234 / 0.4 * sweep.size) + 1
     if index_max_begin >= index_max_end:
-        index_max_begin = index_max_end-1
+        index_max_begin = index_max_end - 1
     # index_max_end = index_max_begin + end_mep
 
     # get maximum and max index
@@ -1428,7 +1450,7 @@ def calc_p2p_old_exp1(sweep, start_mep=18, end_mep=35, tms_pulse_time=None, samp
     p2p : float
         Peak-to-peak value of input curve
     """
-    raise DeprecationWarning("Use calc_p2p(). calc_p2p_old_exp1 only used for data-reproducibility")
+    warnings.warn(DeprecationWarning("Use calc_p2p(). calc_p2p_old_exp1 only used for data-reproducibility"))
 
     # Compute start and stop idx according to sampling rate
     start_mep = int((start_mep / 1000.) * sampling_rate)
@@ -1459,20 +1481,23 @@ def calc_p2p_old_exp1(sweep, start_mep=18, end_mep=35, tms_pulse_time=None, samp
     return sweep_max - sweep_min
 
 
-def calc_p2p(sweep, tms_pulse_time=.2, start_mep=18, end_mep=35, sampling_rate=4000, cutoff_freq=500, fn_plot=None):
+def calc_p2p(sweep, tms_pulse_time=.2, start_mep=20, end_mep=35, measurement_start_time=0, sampling_rate=4000,
+             cutoff_freq=500, fn_plot=None):
     """
-    Calc peak-to-peak values of an mep sweep.
+    Calc peak-to-peak values of and mep sweep.
 
     Parameters
     ----------
     sweep : np.array of float [Nx1]
         Input curve
     tms_pulse_time : float (Default: .2)
-        onset time in seconds of TMS pulse trigger
+        onset time of TMS pulse trigger in [s]
     start_mep : int (Default: 18)
-        start of p2p search window, relative to TMS pulse. In [ms].
+        start of p2p search window after TMS pulse. In [ms].
     end_mep : int (Default: 35)
-        start of p2p search window, relative to TMS pulse. In [ms].
+        end of p2p search window after TMS pulse. In [ms].
+    measurement_start_time : float (Default: 0)
+        start time of the EMG measurement. In [ms].
     sampling_rate : int (Default: 2000)
         Sampling rate in Hz
     cutoff_freq : int (Default: 500)
@@ -1486,57 +1511,67 @@ def calc_p2p(sweep, tms_pulse_time=.2, start_mep=18, end_mep=35, sampling_rate=4
         Peak-to-peak value of input curve
     sweep_filt : np.array of float
         Filtered input curve (Butter lowpass filter with specified cutoff_freq)
+    onset : float
+        MEP onset after tms_pulse_time
     """
+
+    def time_to_idx_conversion(t):
+        return int((t - measurement_start_time) * sampling_rate / 1000)
+
+    def idx_to_time_conversion(i):
+        return i * 1000 / sampling_rate + measurement_start_time
+
     # Compute start and stop idx according to sampling rate
-    start_mep = int((start_mep / 1000.) * sampling_rate)
-    end_mep = int((end_mep / 1000.) * sampling_rate)
+    if tms_pulse_time > 1:
+        warnings.warn(f"Is tms_pulse_time={tms_pulse_time} really in seconds?")
 
     # Filter requirements.
     order = 6
 
     sweep_filt = butter_lowpass_filter(sweep, cutoff_freq, sampling_rate, order)
 
-    # compute tms pulse idx in samplerate space
-    tms_pulse_sample = int(tms_pulse_time * sampling_rate)
-
-    # get index for begin of mep search window
-    # index_max_begin = np.argmin(sweep) + start_mep  # get TMS impulse # int(0.221 / 0.4 * sweep.size)
     # beginning of mep search window
-    srch_win_start = tms_pulse_sample + start_mep  # get TMS impulse # int(0.221 / 0.4 * sweep.size)
+    srch_win_start = time_to_idx_conversion(tms_pulse_time * 1000 + start_mep)  # get TMS impulse
 
     if srch_win_start >= sweep_filt.size:
         srch_win_start = sweep_filt.size - 1
-    # index_max_end = sweep_filt.size  # int(0.234 / 0.4 * sweep.size) + 1
-    srch_win_end = srch_win_start + end_mep
+    srch_win_end = time_to_idx_conversion(tms_pulse_time * 1000 + end_mep)
 
     # get maximum and max index
     sweep_max = np.amax(sweep_filt[srch_win_start:srch_win_end])
-    # if sweep_max < index_max_begin + 72:
-    #     a = 1
 
     # Using the same window as the max should make this more robust
-    # index_min_begin = mep_search_window_start
-    # index_min_end = mep_search_window_end
     sweep_min = np.amin(sweep_filt[srch_win_start:srch_win_end])
     p2p = sweep_max - sweep_min
 
+    # find onset in [s] of mep after tms pulse
+    onset_max = np.argmax(sweep[srch_win_start:srch_win_end])
+    onset_min = np.argmin(sweep[srch_win_start:srch_win_end])
+    if onset_min < onset_max:
+        onset_max = onset_min
+    onset_max += srch_win_start
+    onset_max /= sampling_rate
+    onset_max -= tms_pulse_time
+
     if fn_plot is not None:
-        t = np.arange(len(sweep))/sampling_rate
+        timepoints = [idx_to_time_conversion(i) for i in np.arange(len(sweep))]
         sweep_min_idx = np.argmin(sweep_filt[srch_win_start:srch_win_end]) + srch_win_start
         sweep_max_idx = np.argmax(sweep_filt[srch_win_start:srch_win_end]) + srch_win_start
 
         plt.figure(figsize=[4.07, 3.52])
-        plt.plot(t, sweep)
-        plt.plot(t, sweep_filt)
-        plt.scatter(t[sweep_min_idx], sweep_min, 15, color="r")
-        plt.scatter(t[sweep_max_idx], sweep_max, 15, color="r")
-        plt.plot(t, np.ones(len(t))*sweep_min, linestyle="--", color="r", linewidth=1)
-        plt.plot(t, np.ones(len(t))*sweep_max, linestyle="--", color="r", linewidth=1)
+        plt.plot(timepoints, sweep)
+        plt.plot(timepoints, sweep_filt)
+        plt.scatter(timepoints[sweep_min_idx], sweep_min, 15, color="r")
+        plt.scatter(timepoints[sweep_max_idx], sweep_max, 15, color="r")
+        plt.plot(timepoints, np.ones(len(timepoints)) * sweep_min, linestyle="--", color="r", linewidth=1)
+        plt.plot(timepoints, np.ones(len(timepoints)) * sweep_max, linestyle="--", color="r", linewidth=1)
         plt.grid()
         plt.legend(["raw", "filtered", "p2p"], loc='upper right')
 
-        plt.xlim([np.max((tms_pulse_time-0.01, np.min(t))), np.min((t[tms_pulse_sample+end_mep]+0.1, np.max(t)))])
-        plt.ylim([-1.1*np.abs(sweep_min), 1.1*np.abs(sweep_max)])
+        # plt.xlim([np.max((tms_pulse_time-0.01, np.min(t))), np.min((t[tms_pulse_sample+end_mep]+0.1, np.max(t)))])
+        plt.xlim([np.max((tms_pulse_time - 0.01, np.min(timepoints))),
+                  np.min((timepoints[srch_win_end] + 0.1, np.max(timepoints)))])
+        plt.ylim([-1.1 * np.abs(sweep_min), 1.1 * np.abs(sweep_max)])
 
         plt.xlabel("time in s", fontsize=11)
         plt.ylabel("MEP in mV", fontsize=11)
@@ -1545,37 +1580,67 @@ def calc_p2p(sweep, tms_pulse_time=.2, start_mep=18, end_mep=35, sampling_rate=4
         plt.savefig(fn_plot, dpi=300, transparent=True)
         plt.close()
 
-    return [p2p, sweep_filt]
+    return p2p, sweep_filt, onset_max
 
 
 def get_mep_sampling_rate(cfs_path):
     """
-    Returns sampling rate [Hz] for cfs file.
+    Returns sampling rate [Hz] for CED Signal EMG data in .cfs, .mat or .txt file.
 
     The sampling rate is saved in the cfs header like this:
     ...Samplingrate"\t: 3999.999810,\n...
 
     Parameters
     ----------
-    cfs_path : string with path to ...cfs-file
+    cfs_path : string with path to ...cfs-file or .txt file
 
     Returns
     --------
-    int : sampling rate
+    float : sampling rate
     """
-    # get header from cfs file
-    cfs_header = biosig.header(cfs_path)
+    if cfs_path.endswith('.cfs'):
+        # get header from cfs file
+        cfs_header = biosig.header(cfs_path)
 
-    # get start and end idx of 'samplingrate'
-    idx_a = re.search('Samplingrate', cfs_header)
+        # get start and end idx of 'samplingrate'
+        idx_a = re.search('Samplingrate', cfs_header)
 
-    # get idx of first '\n' after 'samplingrate'
-    idx_b = re.search(',\n', cfs_header[idx_a.end():])
+        # get idx of first '\n' after 'samplingrate'
+        idx_b = re.search(',\n', cfs_header[idx_a.end():])
 
-    sr_start = idx_a.end() + 4  # 'Samplingrate"\t: '
-    sr = int(round(float(cfs_header[sr_start:idx_b.start() + idx_a.end()])))
+        sr_start = idx_a.end() + 4  # 'Samplingrate"\t: '
+        return float(cfs_header[sr_start:idx_b.start() + idx_a.end()])
 
-    return sr
+
+    elif cfs_path.endswith('.mat'):
+        mep_data = spio.loadmat(cfs_path, struct_as_record=False, squeeze_me=True)
+        for k in mep_data.keys():
+            if isinstance(mep_data[k], spio.matlab.mio5_params.mat_struct):
+                mep_data = mep_data[k].__dict__
+                break
+        return 1 / mep_data['interval']
+
+    elif cfs_path.endswith('.txt'):
+        # search for end of first frame / sweep
+        pattern = '"s"'
+        with open(cfs_path, 'r') as f:
+            for line_nr, line in enumerate(f):
+                if pattern in line and line_nr > 0:
+                    # find second occurance of "s" -> end of first sweep
+                    n_samples = line_nr
+                    break
+                if line != '\n':
+                    last_sample_time = line
+        # extract time (first column) of last samples
+        last_sample_time = float(last_sample_time[0:last_sample_time.find('\t')])
+
+        # subtract 2 because first row is header ("s"\t"ADC 0"\t"ADC 1") and last row is blank
+        n_samples -= 2
+
+        # get sampling rate by dividing number of sweeps by timing
+        return n_samples / last_sample_time
+    else:
+        raise ValueError(f"cfs_path={cfs_path} must be .csf or .txt filename.")
 
 
 def butter_lowpass(cutoff, fs, order=5):
