@@ -1,5 +1,5 @@
 from .exp import *
-from .tmsnav import *
+from .localite import *
 from .Mep import *
 from .visor import *
 from .brainvis import *
