@@ -24,17 +24,17 @@ def read_channel_names(fname):
     channel_names = []
 
     while not eof:
-        l = f.readline()
+        lines = f.readline()
 
-        if l == "[Channel Infos]\n":
+        if lines == "[Channel Infos]\n":
             channel_info_start = 5 + i_line
 
         if i_line >= channel_info_start:
 
-            if l == "\n":
+            if lines == "\n":
                 break
 
-            channel_names.append(re.search("(?<=\=)(.*?)(?=\,)", l).group(0))
+            channel_names.append(re.search("(?<=\=)(.*?)(?=\,)", lines).group(0))
 
         i_line += 1
 
@@ -60,11 +60,11 @@ def read_sampling_frequency(fname):
     eof = False
 
     while not eof:
-        l = f.readline()
+        line = f.readline()
 
-        if "Sampling Rate [Hz]: " in l:
-            start_idx = l.find(":") + 2
-            sampling_frequency = float(l[start_idx:])
+        if "Sampling Rate [Hz]: " in line:
+            start_idx = line.find(":") + 2
+            sampling_frequency = float(line[start_idx:])
             break
 
     return sampling_frequency
