@@ -1,20 +1,19 @@
 """ Functions to import data from ANT Visor 2 / ANT EEG software go here """
+import os
 import warnings
 
-import os
 import h5py
 import numpy as np
-from pynibs import write_arr_to_hdf5
-from pynibs.util.util import quat_to_rot
-from scipy.spatial.transform import Rotation as R
-from scipy import signal
 import pandas as pd
+from scipy.spatial.transform import Rotation
+from scipy import signal
+
+import pynibs
 
 try:
     from pynibs.pckg import libeep
 except (ImportError, SyntaxError):
     pass
-    # print("lipbeep not found. visor import will not work.")
 
 
 def read_nlr(fname):
@@ -117,7 +116,7 @@ def get_instrument_marker(im_path, verbose=False):
         matsimnibs_raw[3, 3] = 1
         matsimnibs_raw[0:3, 3] = np.array([data[-1]['PosX'], data[-1]['PosY'], data[-1]['PosZ']]) * 1000
         quat = np.array([data[-1]['QuatX'], data[-1]['QuatY'], data[-1]['QuatZ'], data[-1]['QuatW']])
-        matsimnibs_raw[0:3, 0:3] = R.from_quat(quat).as_dcm()
+        matsimnibs_raw[0:3, 0:3] = Rotation.from_quat(quat).as_dcm()
         data[-1]["coil_mean_raw"] = matsimnibs_raw
 
     return data
@@ -197,7 +196,7 @@ def get_cnt_data(fn, channels='all', trigger_val='1', max_duration=10,
 
     # chunk into triggers
     trigger_idx = 0
-    arr_idx = 0
+    # arr_idx = 0
     last_zap_done = False
     trigger_zap = 0
     # we want the data between trigger and trigger+1
@@ -226,7 +225,7 @@ def get_cnt_data(fn, channels='all', trigger_val='1', max_duration=10,
             if not start[1] < end[1]:
                 if verbose:
                     print(f"Trigger {trigger_idx} and {trigger_idx + 1}: wrong sample number "
-                              f"({trigger_idx}: {start[1]}, {trigger_idx + 1}: {end[1]}]")
+                          f"({trigger_idx}: {start[1]}, {trigger_idx + 1}: {end[1]}]")
                 # the eeg cnt files and with a trigger. get data from trigger to end-offile
                 if trigger_idx == n_trig - 1:
                     end = (end[0], f.get_sample_count())
@@ -345,14 +344,14 @@ def filter_emg(emg, fs):
     Parameters
     ----------
     emg : list of np.array [n_stimuli]
-        Raw EMG data. Each list entry contains a np.array of size [n_samples x n_channel]
+        Raw EMG data. Each list entry contains a np.ndarray of size [n_samples x n_channel]
         Each channel is filtered in the same way.
     fs : float
         Sampling frequency
 
     Returns
     -------
-    emg_filt : list of np.array [n_stimuli]
+    emg_filt : list of np.ndarray [n_stimuli]
         Filtered EMG data
     """
 
@@ -401,3 +400,214 @@ def filter_emg(emg, fs):
             emg_filt[-1][:, i_channel] = signal.filtfilt(b_notch50, a_notch50, emg_filt[-1][:, i_channel])
 
     return emg_filt
+
+
+def merge_exp_data(subject, exp_id=0, mesh_idx=0, verbose=False):
+    """
+    Merges all experimental data from visor experiment into one .hdf5 file
+
+    Parameters
+    ----------
+    subject : Subject object
+        Subject object
+    exp_id : int
+        Experiment index
+    mesh_idx : int
+        Mesh index
+    verbose : bool
+        Print output
+
+    Returns
+    -------
+    <File>: .hdf5 file
+        File containing the stimulation and physiological data as pandas dataframes:
+        - "stim_data": Stimulation parameters (e.g. coil positions, etc.)
+        - "phys_data/info/EMG": Information about EMG data recordings (e.g. sampling frequency, etc.)
+        - "phys_data/info/EEG": Information about EEG data recordings (e.g. sampling frequency, etc.)
+        - "phys_data/raw/EMG": Raw EMG data
+        - "phys_data/raw/EEG": Raw EEG data
+        - "phys_data/postproc/EMG": Post-processed EMG data (e.g. filtered, p2p, etc.)
+        - "phys_data/postproc/EEG": Post-processed EEG data (e.g. filtered, p2p, etc.)
+    """
+    # mep_paths_lst = subject.exp[exp_id]['fn_data']
+
+    # im_lst = subject.exp[exp_id]['cond']
+    # nii_exp_path_lst = subject.exp[exp_id]['fn_mri_nii']
+    # nii_conform_path = subject.mesh[mesh_idx]['fn_mri_conform']
+    fn_exp_hdf5 = subject.exp[exp_id]['fn_exp_hdf5']
+    fn_current = subject.exp[exp_id]['fn_current'][0]
+    # fn_coil = subject.exp[exp_id]['fn_coil']
+    # fn_mesh_hdf5 = subject.mesh[mesh_idx]['fn_mesh_hdf5']
+    exp_id = exp_id
+
+    if os.path.exists(fn_exp_hdf5):
+        os.remove(fn_exp_hdf5)
+
+    # read stimulation parameters
+    # ===================================================================================
+    if 'fn_visor_cnt' in subject.exp[exp_id]:
+        print(f"Reading stimulation parameters from {subject.exp[exp_id]['fn_visor_cnt']}")
+
+        assert 'fn_fiducials' in subject.exp[exp_id]
+        assert 'fn_current' in subject.exp[exp_id]
+        assert len(subject.exp[exp_id]['fn_visor_cnt']) == 1, "Multiple coils not implemented for visor"
+        fn_visor_cnt = subject.exp[exp_id]['fn_visor_cnt'][0]
+
+        fn_fiducials = subject.exp[exp_id]['fn_fiducials'][0]
+
+        ims_list = pynibs.visor.get_instrument_marker(fn_visor_cnt)
+        ims_dict = pynibs.list2dict(ims_list)
+        n_stim = len(ims_list)
+
+        # read fiducials and transform to simnibs space
+        fiducials = pynibs.visor.read_nlr(fn_fiducials)
+
+        # fiducial correction
+        if 'fiducial_corr' in subject.exp[exp_id]:
+            fiducal_corr = np.array(subject.exp[exp_id]['fiducial_corr'])
+            if any(np.abs(fiducal_corr[fiducal_corr != 0]) < .1):
+                warnings.warn("fiducial_corr are expected to be given in mm.")
+            fiducials += fiducal_corr
+
+        fn_exp_nii = subject.exp[exp_id]['fn_mri_nii'][0][0]
+
+        matsimnibs_raw = np.dstack(ims_dict["coil_mean_raw"])
+
+        matsimnibs = pynibs.nnav2simnibs(fn_exp_nii=fn_exp_nii,
+                                  fn_conform_nii=subject.mesh[mesh_idx]['fn_mri_conform'],
+                                  m_nnav=matsimnibs_raw,
+                                  nnav_system="visor",
+                                  fiducials=fiducials,
+                                  verbose=verbose)
+
+        # read coil current
+        current = np.loadtxt(fn_current)
+
+        if subject.exp[exp_id]["cond"][0][0] != "":
+            raise NotImplementedError("Individual conditions and average coil position over it not implemented yet")
+
+        # create stim_data dataframe
+        stim_data = {"coil_mean": [matsimnibs[:, :, i] for i in range(n_stim)],
+                     "coil_type": [np.array(os.path.split(subject.exp[exp_id]["fn_coil"][0][0])[1]).astype(
+                             "|S")] * n_stim,
+                     "current": current,
+                     "condition": [f"{(i - 1):04d}" for i in ims_dict["StimulusID"]]}
+
+        df_stim_data = pd.DataFrame.from_dict(stim_data)
+        df_stim_data.to_hdf(fn_exp_hdf5, "stim_data")
+
+        print(f"Writing stim_data dataframe to {fn_exp_hdf5}")
+
+    else:
+        warnings.warn("No visor positions found.")
+
+    # read emg
+    # ===================================================================================
+    if 'fn_emg_cnt' in subject.exp[exp_id]:
+
+        print(f"Reading EMG data from {subject.exp[exp_id]['fn_emg_cnt'][0]}")
+
+        # which emg_channel to use
+        emg_channels = subject.exp[exp_id]['emg_channels']
+
+        if isinstance(emg_channels, list) and len(emg_channels) > 1:
+            warnings.warn("Multiple EMG channels are untested.")
+
+        emg_trigger_value = subject.exp[exp_id]['emg_trigger_value'][0]
+
+        max_duration = 10  # maximum EMG time series duration per after zap
+        try:
+            max_duration = subject.exp[exp_id]['emg_max_duration'][0]
+        except KeyError:
+            pass
+        fn_emg_cnt = subject.exp[exp_id]['fn_emg_cnt'][0]
+
+        # read info
+        cnt_info = pynibs.get_cnt_infos(fn_emg_cnt)
+
+        phys_data_info_emg = dict()
+        for key in cnt_info.keys():
+            phys_data_info_emg[key] = cnt_info[key]
+
+        phys_data_info_emg["max_duration"] = max_duration
+        phys_data_info_emg["emg_channels"] = emg_channels
+
+        df_phys_data_info_emg = pd.DataFrame.from_dict(phys_data_info_emg)
+        df_phys_data_info_emg.to_hdf(fn_exp_hdf5, "phys_data/info/EMG")
+        print(f"Writing EMG info dataframe (phys_data/info/EMG) to {fn_exp_hdf5}")
+
+        # read raw emg data from cnt file and write to hdf5 file
+        emg = pynibs.visor.get_cnt_data(fn_emg_cnt,
+                                        channels=emg_channels,
+                                        max_duration=max_duration,
+                                        trigger_val=emg_trigger_value,
+                                        verbose=verbose,
+                                        fn_hdf5=fn_exp_hdf5,
+                                        path_hdf5="phys_data/raw/EMG",
+                                        return_data=True)
+
+        print(f"Writing EMG raw dataframe (phys_data/raw/EMG) to {fn_exp_hdf5}")
+
+        # filter data
+        emg_filt = pynibs.visor.filter_emg(emg=emg, fs=phys_data_info_emg["sampling_rate"])
+        df_phys_data_postproc_emg = pd.DataFrame.from_dict({"filtered": emg_filt})
+
+        # calc p2p
+        # TODO: implement p2p function
+        # p2p = calc_p2p(emg_filt)
+        # df_phys_data_postproc_emg["p2p"] = p2p
+
+        df_phys_data_postproc_emg.to_hdf(fn_exp_hdf5, "phys_data/postproc/EMG")
+        print(f"Writing EMG postproc dataframe (phys_data/postproc/EMG) to {fn_exp_hdf5}")
+
+    # read eeg
+    # ===================================================================================
+    if 'fn_eeg_cnt' in subject.exp[exp_id]:
+        # which emg_channel to use?
+        max_duration = 10  # maximum EMG time series duration per after zap
+
+        try:
+            max_duration = subject.exp[exp_id]['eeg_max_duration'][0]
+        except KeyError:
+            pass
+
+        eeg_trigger_value = subject.exp[exp_id]['eeg_trigger_value'][0]
+
+        # eeg_channel can be int, str, list of int, list of str
+        eeg_channels = ['all']
+        try:
+            try:
+                # list of int
+                eeg_channels = subject.exp[exp_id]['eeg_channels']
+            except ValueError:
+                # list of str (gets casted to b'')
+                eeg_channels = subject.exp[exp_id]['eeg_channels'].astype(str).tolist()
+        except KeyError:  # key not defined, fall back to default
+            pass
+
+        fn_eeg_cnt = subject.exp[exp_id]['fn_eeg_cnt'][0]
+
+        phys_data_info_eeg = dict()
+        for key in cnt_info.keys():
+            phys_data_info_eeg[key] = cnt_info[key]
+
+        phys_data_info_eeg["max_duration"] = max_duration
+        phys_data_info_eeg["eeg_channels"] = eeg_channels
+
+        df_phys_data_info_eeg = pd.DataFrame.from_dict(phys_data_info_eeg)
+        df_phys_data_info_eeg.to_hdf(fn_exp_hdf5, "phys_data/info/EEG")
+        print(f"Writing EEG info dataframe (phys_data/info/EEG) to {fn_exp_hdf5}")
+
+        # read raw eeg data from cnt file and write to hdf5 file
+        pynibs.visor.get_cnt_data(fn_eeg_cnt,
+                                  channels=eeg_channels,
+                                  max_duration=max_duration,
+                                  trigger_val=eeg_trigger_value,
+                                  verbose=verbose,
+                                  fn_hdf5=fn_exp_hdf5,
+                                  path_hdf5="phys_data/raw/EEG",
+                                  return_data=False)
+
+        print(f"Writing EEG raw dataframe (phys_data/raw/EEG) to {fn_exp_hdf5}")
+
+    print("DONE")
