@@ -1,12 +1,16 @@
 import platform
-import os
 import shutil
-from .main import *
+import numpy as np
+import nibabel as nib
+import meshio
+import os
+from .mesh import map_data_to_surface, data_nodes2elements, data_elements2nodes
 from .util import *
 from .hdf5_io import write_geo_hdf5_surf
 from .hdf5_io import write_data_hdf5_surf
 from .roi import make_GM_WM_surface
 from .subject import *
+import nibabel
 
 
 def read_curv_data(fname_curv, fname_inf, raw=False):
@@ -136,15 +140,15 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
             # print [command, "{}/surf/{}.sphere {}{}.tif {}.sphere.myreg{}".format(subject,
             # hemi, template, i, hemi, i)]
             processes.add(subprocess.Popen(
-                [f'{command} '
-                 f'{os.path.split(subject)[1]}{os.sep}surf{os.sep}lh.sphere lh{template}{i}.tif '
-                 f'{"lh"}.sphere.{template}{i}'],
-                shell=True))
+                    [f'{command} '
+                     f'{os.path.split(subject)[1]}{os.sep}surf{os.sep}lh.sphere lh{template}{i}.tif '
+                     f'{"lh"}.sphere.{template}{i}'],
+                    shell=True))
             processes.add(subprocess.Popen(
-                [f"{command} "
-                 f"{os.path.split(subject)[1]}{os.sep}surf{os.sep}rh.sphere rh{template}{i}.tif "
-                 f"{'rh'}.sphere.{template}{i}"],
-                shell=True))
+                    [f"{command} "
+                     f"{os.path.split(subject)[1]}{os.sep}surf{os.sep}rh.sphere rh{template}{i}.tif "
+                     f"{'rh'}.sphere.{template}{i}"],
+                    shell=True))
 
             # check if n_cpu processes are reached
             if len(processes) >= n_cpu:
@@ -176,7 +180,7 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
         print(command)
         os.system(command)
 
-    print("##################### Iteration {} done. #####################\n".format(i))
+        print("##################### Iteration {} done. #####################\n".format(i))
 
     print('Creating average surfaces and volumes of average subject')
     make_average_subject(subjects=subjects,
@@ -318,19 +322,19 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
 
     # filename of mapped subject data in nodes in .curv format for freesurfer (will be generated)
     fn_data_sub_nodes_mapped_curv = [os.path.join(
-        os.path.split(fn_out_hdf5_geo)[0],
-        hemisphere + '.' +
-        os.path.split(subject_dir)[1] +
-        '_' +
-        data_label[i] +
-        '.curv')
+            os.path.split(fn_out_hdf5_geo)[0],
+            hemisphere + '.' +
+            os.path.split(subject_dir)[1] +
+            '_' +
+            data_label[i] +
+            '.curv')
         for i in range(len(data_label))]
 
     # filename of mapped subject data in nodes on average template in .curv format for freesurfer (will be generated)
     fn_data_sub_nodes_mapped_avg_curv = [os.path.join(
-        os.path.split(fn_data_sub_nodes_mapped_curv[i])[0],
-        os.path.splitext(os.path.split(fn_data_sub_nodes_mapped_curv[i])[1])[0] +
-        '_avg.curv')
+            os.path.split(fn_data_sub_nodes_mapped_curv[i])[0],
+            os.path.splitext(os.path.split(fn_data_sub_nodes_mapped_curv[i])[1])[0] +
+            '_avg.curv')
         for i in range(len(data_label))]
 
     if verbose:
@@ -350,8 +354,8 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
             print(("> Reading ROI #{} of subject ...".format(roi_idx)))
 
         with h5py.File(subject.mesh[mesh_idx]['fn_mesh_hdf5'], 'r') as f:
-            points_ROI = np.array(f['roi_surface/' + str(roi_idx) + '/node_coord_mid'])
-            con_ROI = np.array(f['roi_surface/' + str(roi_idx) + '/node_number_list'])
+            points_roi = np.array(f['roi_surface/' + str(roi_idx) + '/node_coord_mid'])
+            con_roi = np.array(f['roi_surface/' + str(roi_idx) + '/node_number_list'])
 
         # Read subject data
         if verbose:
@@ -363,7 +367,7 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
             if verbose:
                 print('> Transforming subject data from element centers to nodes')
             # data_sub_nodes = [data_elements2nodes(data_sub[i], con_ROI) for i in range(len(data_label))]
-            data_sub_nodes = data_elements2nodes(data_sub, con_ROI)
+            data_sub_nodes = data_elements2nodes(data_sub, con_roi)
 
         else:
             data_sub_nodes = data_sub
@@ -376,8 +380,8 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
             if verbose:
                 print('> Mapping point data of ROI to whole brain surface')
             data_sub_nodes_mapped = map_data_to_surface(datasets=data_sub_nodes,
-                                                        points_datasets=[points_ROI] * len(data_sub_nodes),
-                                                        con_datasets=[con_ROI] * len(data_sub_nodes),
+                                                        points_datasets=[points_roi] * len(data_sub_nodes),
+                                                        con_datasets=[con_roi] * len(data_sub_nodes),
                                                         fname_fsl_gm=os.path.join(mesh_folder_sub, fn_sub_gm),
                                                         fname_fsl_wm=os.path.join(mesh_folder_sub, fn_sub_wm),
                                                         delta=subject.roi[mesh_idx][roi_idx]['delta'],
@@ -462,3 +466,55 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
 
     if verbose:
         print('> Done!\n')
+
+
+def freesurfer2vtk(in_fns, out_folder, hem='lh', surf='pial', prefix=None, fs_subject='fsaverage', fs_subjects_dir=None):
+    f"""
+    Transform multiple freesurfer .mgh files into one .vtk file.
+    This can be read with Paraview and others.
+
+    Parameters
+    ----------
+    in_fns : list of str or list
+    out_folder : str
+        Output folder.
+    hem : str, default: 'lh'
+        Which hemisphere: 'lh' or 'rh'
+    surf : str, default:  'pial'
+        Which freesurfer surface: 'pial', 'inflated', ...
+    prefix : str, optional
+        Prefix to add to each filename
+    fs_subject : string, default: 'fsaverage'
+        FreeSurfer subject 
+    fs_subjects_dir : string, optional
+        FreeSurfer subjects directory. If not provided, read from environment
+
+    Returns
+    -------
+    out_folder/{prefix}_{hem}_{surf}.vtk : File
+        One .vtk file with data as overlays from all .mgh files provided
+    """
+    if not isinstance(in_fns, list):
+        in_fns = [in_fns]
+
+    if prefix is not None and not prefix.endswith('_'):
+        prefix += '_'
+
+    if fs_subjects_dir is None:
+        try:
+            fs_subjects_dir = os.environ['SUBJECTS_DIR']
+        except KeyError:
+            print(f"Freesurfer not loaded and 'fs_subjects_dir' not provided.")
+
+    output_fn = f"{out_folder}/{prefix}{hem}_{surf}.vtk"
+
+    # create a dict with fn as key, data as value. keys will be vtk data array names
+    point_data = {}
+    for fn in in_fns:
+        point_data[os.path.split(fn)[1].replace('.mgh', '')] = nib.load(fn).get_fdata()
+
+    # read freesurfer surface mesh
+    coords, faces = nib.freesurfer.read_geometry(f"{fs_subjects_dir}/{fs_subject}/surf/{hem}.{surf}")
+
+    # write all dataarrays + mesh geometry into one .vtk
+    meshio.Mesh(np.squeeze(coords), [('triangle', faces)], point_data=point_data).write(output_fn)
