@@ -1,19 +1,102 @@
-import os
-import sys
-from builtins import float
-
-import copy
 import numpy as np
-import multiprocessing
-from multiprocessing import shared_memory
-import re
-import h5py
 from pynibs import load_muaps
-
 from .exp.Mep import sigmoid
-from functools import partial
-from pynibs import compute_chunks
 from scipy.special import logsumexp
+from scipy.interpolate import LinearNDInterpolator
+
+
+def calc_e_sensitivity(theta, gradient):
+    """
+    Determine sensitivity map of electric field
+
+    Parameters
+    ----------
+    theta : np.ndarray
+        Theta angle (matrix) [N_stim x N_ele] of electric field with respect to surface normal
+    gradient : np.ndarray, optional, default: None
+        Electric field gradient (matrix) [N_stim x N_ele] between layer 1 and layer 6. Optional, the neuron mean field
+        model is more accurate when provided.
+
+    Returns
+    -------
+    e_sens : np.ndarray
+        Electric field sensitivity maps [N_stim x N_ele]
+    """
+    cell_simulation_data = []
+    combined_thresholds = []
+    thetas = []
+    rel_gradients = []
+
+    for cell_id in range(1, 6):
+        cell_thetas = []
+        cell_rel_grads = []
+        cell_thresholds = []
+        cell_simulation_data.append(np.genfromtxt(f'/data/pt_01756/studies/neuronibs/neuronibs_data/cells/Layer_5/L5_TTPC2_cADpyr232_{cell_id}/L5_TTPC2_cADpyr232_{cell_id}_41_9_6.csv', delimiter=','))
+
+        thresholds = cell_simulation_data[-1][:, 3]
+        theta_ = cell_simulation_data[-1][:, 0]
+        phi = cell_simulation_data[-1][:, 1]
+        rel_grad = cell_simulation_data[-1][:, 2]
+
+        unique_theta = np.unique(theta_)
+        unique_phi = np.unique(phi)
+        unique_rel_grad = np.unique(rel_grad)
+
+        theta_count = len(unique_theta)
+        phi_count = len(unique_phi)
+        rel_gradient_count = len(unique_rel_grad)
+
+        slice_width = ((theta_count - 2) * phi_count + 2)
+        for k in range(rel_gradient_count):
+            slice_start = k * slice_width
+            slice_end =  (k + 1) * slice_width - 1
+            cell_thresholds += [thresholds[slice_start]] + [np.mean(thresholds[slice_start + phi_count * j + 1: slice_start + phi_count * j + phi_count + 1]) for j in range(theta_count-2)] + [thresholds[slice_end]]
+            cell_thetas +=     [theta_[slice_start]] +      [np.mean(theta_[slice_start + phi_count * j + 1: slice_start + phi_count * j + phi_count + 1]) for j in range(theta_count-2)] + [theta_[slice_end]]
+            cell_rel_grads +=  [rel_grad[slice_start]] +   [np.mean(rel_grad[slice_start + phi_count * j + 1: slice_start + phi_count * j + phi_count + 1]) for j in range(theta_count-2)] + [rel_grad[slice_end]]
+
+        combined_thresholds.append(cell_thresholds)
+        thetas.append(cell_thetas)
+        rel_gradients.append(cell_rel_grads)
+
+    # average thresholds over cells
+    mean_threshold = np.mean(np.array(combined_thresholds), axis=0)
+
+    # create ND Interpolator
+    interp = LinearNDInterpolator(list(zip(thetas[0], rel_gradients[0])), mean_threshold)
+    e_sens = interp(theta, gradient) / 3
+
+    return e_sens
+
+
+def calc_e_effective(e, theta, gradient=None):
+    """
+    Determines the effective electric field using a neuron mean field model.
+    Transforms the electric field by subtracting the sensitivity map (in V/m) from the original electric field.
+    The remaining field is the effective electric field (e_eff), which is responsible for the stimulation effect,
+    i.e. behavioural effects start at e_eff > 0 because lower fields were not able to stimulate neurons.
+
+    Parameters
+    ----------
+    e : np.ndarray
+        Electric field (matrix) [N_stim x N_ele]
+    theta : np.ndarray
+        Theta angle (matrix) [N_stim x N_ele] of electric field with respect to surface normal
+    gradient : np.ndarray, optional, default: None
+        Electric field gradient (matrix) [N_stim x N_ele] between layer 1 and layer 6. Optional, the neuron mean field
+        model is more accurate when provided.
+
+    Returns
+    -------
+    e_eff : np.ndarray
+        Effective electric field (matrix) [N_stim x N_ele] the regression analysis can be performed with.
+    """
+    # determine sensitivity map
+    e_sens = calc_e_sensitivity(theta=theta, gradient=gradient)
+
+    # determine effective electric field
+    e_eff = e - e_sens
+
+    return e_eff
 
 
 def DI_wave(t, intensity, t0=5, dt=1.4, width=0.25):
@@ -62,10 +145,10 @@ def DI_wave(t, intensity, t0=5, dt=1.4, width=0.25):
     r["I3"] = 16.74855628350182
     r["I4"] = 17.85806255278076
 
-    y = np.zeros(len(t),dtype=np.float128)
+    y = np.zeros(len(t), dtype=np.float128)
 
     for i, w in enumerate(waves):
-        y_ = logsumexp(-(t - t0 - i * dt) ** 2 / (2 * width ** 2))
+        y_ = np.exp(-(t - t0 - i * dt) ** 2 / (2 * width ** 2))
         y_ = y_ / np.max(y_)
         y_ = y_ * sigmoid(intensity, amp=amp[w], r=r[w], x0=x0[w])
         y = y + y_
@@ -87,8 +170,6 @@ class Workhorse(object):
 
         Parameters
         ----------
-        alpha_motor_neurons: list of AlphaMotorNeuron instances
-            Alpha motor neurons
         input: ndarray of float [n_t]
             Input signal to alpha motor neurons
 
@@ -103,27 +184,23 @@ class Workhorse(object):
         delay = round(self.renshaw.delay / self.dt)
 
         for i, t_ in enumerate(self.t):
-
             feedback[i] = self.aMN.step(input, feedback, t_, i)
 
             if i + delay < len(self.t) - 1:
-
-               feedback[i+1] = self.renshaw.step(feedback, t_,i)  # TODO consider delay and length of signal
-
+                feedback[i + 1] = self.renshaw.step(feedback, t_, i)  # TODO consider delay and length of signal
 
         return self.aMN
 
 
 class Model(object):
     """
-    Our Model. Initialisation of all values and parallelisation process
+    Spinal cord and Muscle model containing alpha motor neurons and Renshaw cells.
     """
 
-    # init with default values
     def __init__(self, T, dt, N_MU=100, fn_muaps="/data/pt_01756/studies/MEP_modeling/results/muscle_model"
                                                  "/Dist1_Monopolar_Abduct_NormalCV_New.hdf5", **kwargs):
         """
-        Initialise all values of the Model
+        Initialize spinal cord and Muscle model
 
         Parameters
         ----------
@@ -138,51 +215,46 @@ class Model(object):
         self.N_MU = N_MU
         self.T = T
         self.dt = dt
-        self.t = np.arange(0, T + dt, dt)  # time array
+        self.t = np.arange(0, T + dt, dt)
 
         # Neuron
-        self.Vm_thr_array = kwargs['n_thr'] + kwargs['m_thr'] * np.arange(
-            self.N_MU)  # thresholds of alpha motor neurons
+        self.Vm_thr_array = kwargs['n_thr'] + kwargs['m_thr'] * np.arange(self.N_MU)  # thresholds of alpha MNs
         self.t_refrac = kwargs['t_refrac']
         self.tau_mem = kwargs['tau_mem']
         self.Vm_rest = kwargs['Vm_rest']
         self.Rm = kwargs['Rm']
 
-        #Renshaw Cell
-
+        # Renshaw cell (inhibitory)
         self.slope_renshaw = kwargs.get('slope_renshaw')
         self.Vm_thr_renshaw = kwargs.get('Vm_thr_renshaw')
         self.Vm_rest_renshaw = kwargs.get('Vm_rest_renshaw')
 
-
-        # Acetyl Synapse
+        # Acetyl synapse (excitatory)
         self.tau_syn_decay_ac = kwargs.get('tau_syn_decay_ac')
         self.tau_syn_rise_ac = kwargs.get('tau_syn_rise_ac')
         self.tau_syn_decay_fast_ac = kwargs.get('tau_syn_decay_fast_ac')
         self.tau_syn_decay_slow_ac = kwargs.get('tau_syn_decay_slow_ac')
         self.a_syn_decay_fast_ac = kwargs.get('a_syn_decay_fast_ac')
         self.a_syn_decay_slow_ac = kwargs.get('a_syn_decay_slow_ac')
-        self.in_weight_ac = kwargs.get('in_weight_ac')
-        self.out_weight_ac = kwargs.get('out_weight_ac')
 
-        # Glycin Synapse
+        # Glycin Synapse (inhibitory)
         self.tau_syn_decay_gl = kwargs.get('tau_syn_decay_gl')
         self.tau_syn_rise_gl = kwargs.get('tau_syn_rise_gl')
         self.tau_syn_decay_fast_gl = kwargs.get('tau_syn_decay_fast_gl')
         self.tau_syn_decay_slow_gl = kwargs.get('tau_syn_decay_slow_gl')
         self.a_syn_decay_fast_gl = kwargs.get('a_syn_decay_fast_gl')
         self.a_syn_decay_slow_gl = kwargs.get('a_syn_decay_slow_gl')
-        self.in_connectivity_gl = kwargs.get('in_weight_gl')
-        self.out_connectivity_gl = kwargs.get('out_connectivity_gl')
-        self.in_connectivity_renshaw = kwargs.get('in_connectivity_renshaw')
 
+        # Connectivity
+        self.connectivity_cortex_alpha = kwargs.get('connectivity_cortex_alpha')
+        self.connectivity_renshaw_alpha = kwargs.get('connectivity_renshaw_alpha')
+        self.connectivity_alpha_renshaw = kwargs.get('connectivity_alpha_renshaw')
 
-        # Load Muaps
+        # Load MUAPS
         self.fn_muaps = fn_muaps
         self.muaps = []
         self.t_muap = []
-        self.muaps, t_muap = load_muaps(
-            "/data/pt_01756/studies/MEP_modeling/results/muscle_model/Dist1_Monopolar_Abduct_NormalCV_New.hdf5")
+        self.muaps, t_muap = load_muaps(self.fn_muaps)
 
         # Workhorse Arrays
         self.alpha_motor_neurons = []
@@ -193,114 +265,105 @@ class Model(object):
         self.mep = None
         self.mep_p2p = None
 
-    def run(self, input, n_cpu=None):
+        # Synapse (Cortex -> Alpha Motor Neurons)
+        acetyl_synapse_1 = Synapse(dt=self.dt, T=self.T,
+                                   tau_syn_decay_fast=self.tau_syn_decay_fast_ac,
+                                   tau_syn_decay_slow=self.tau_syn_decay_slow_ac,
+                                   tau_syn_rise=self.tau_syn_rise_ac,
+                                   a_syn_decay_slow=self.a_syn_decay_slow_ac,
+                                   a_syn_decay_fast=self.a_syn_decay_fast_ac,
+                                   connectivity=kwargs.get('connectivity_cortex_alpha'),
+                                   N=self.N_MU)
+
+        # Synapse (Alpha Motor Neurons -> Renshaw Cells)
+        acetyl_synapse_2 = Synapse(dt=self.dt, T=self.T,
+                                   tau_syn_decay_fast=self.tau_syn_decay_fast_ac,
+                                   tau_syn_decay_slow=self.tau_syn_decay_slow_ac,
+                                   tau_syn_rise=self.tau_syn_rise_ac,
+                                   a_syn_decay_slow=self.a_syn_decay_slow_ac,
+                                   a_syn_decay_fast=self.a_syn_decay_fast_ac,
+                                   connectivity=self.connectivity_alpha_renshaw,
+                                   N=self.N_MU)
+
+        # Synapse (Renshaw Cells -> Alpha Motor Neurons)
+        glycine_synapse = Synapse(dt=self.dt, T=self.T,
+                                  tau_syn_decay_fast=self.tau_syn_decay_fast_gl,
+                                  tau_syn_decay_slow=self.tau_syn_decay_slow_gl,
+                                  tau_syn_rise=self.tau_syn_rise_gl,
+                                  a_syn_decay_slow=self.a_syn_decay_slow_gl,
+                                  a_syn_decay_fast=self.a_syn_decay_fast_gl,
+                                  connectivity=self.connectivity_renshaw_alpha,
+                                  N=self.N_MU)
+
+        self.aMNs = AlphaMotorNeuron(tau_mem=self.tau_mem, Vm_thr=self.Vm_thr_array, Vm_rest=self.Vm_rest, Rm=self.Rm,
+                                     t_refrac=self.t_refrac, T=self.T, dt=self.dt,
+                                     synapses=[acetyl_synapse_1, glycine_synapse], N=self.N_MU)
+
+        self.renshaw = RenshawCell(tau_mem=self.tau_mem, Vm_thr=self.Vm_thr_renshaw, Vm_rest=self.Vm_rest, Rm=self.Rm,
+                                   T=self.T, dt=self.dt, delay=0,
+                                   synapses=[acetyl_synapse_2],
+                                   slope=self.slope_renshaw)  # TODO own Vm_thr for optimisation
+
+        for i in range(self.N_MU):
+            self.motor_units.append(MotorUnit(muap=self.muaps[:, i], t=self.t))
+
+    def run(self, inp):
         """
         Starts the parallel processing.
         Creates Model can all workhorse process
 
         Parameters
         ----------
-        input: ndarray of float [n_t]
+        inp: ndarray of float [n_t]
             Input signal to alpha motor neurons
-        n_cpu : int
-            Number of CPU cores to use
-
-        Returns
-        -------
-        mep: Resulting MEP
-        mep_p2p: p2p value of MEP
         """
 
-        # Synapses
-        acetyl_synapse = Synapse(dt=self.dt, T=self.T, tau_syn_decay_fast=self.tau_syn_decay_fast_ac,
-                                 tau_syn_decay_slow=self.tau_syn_decay_slow_ac,
-                                 tau_syn_rise=self.tau_syn_rise_ac,
-                                 a_syn_decay_slow=self.a_syn_decay_slow_ac,
-                                 a_syn_decay_fast=self.a_syn_decay_fast_ac,
-                                 out_connectivity=1,
-                                 N=self.N_MU)
-        acetyl_synapse_2 = Synapse(dt=self.dt, T=self.T, tau_syn_decay_fast=self.tau_syn_decay_fast_ac,
-                                   tau_syn_decay_slow=self.tau_syn_decay_slow_ac,
-                                   tau_syn_rise=self.tau_syn_rise_ac,
-                                   a_syn_decay_slow=self.a_syn_decay_slow_ac,
-                                   a_syn_decay_fast=self.a_syn_decay_fast_ac,
-                                   out_connectivity=1, in_connectivity=self.in_connectivity_renshaw,
-                                   N=self.N_MU)
-        glycine_synapse = Synapse(dt=self.dt, T=self.T, tau_syn_decay_fast=self.tau_syn_decay_fast_gl,
-                                  # TODO get own values
-                                  tau_syn_decay_slow=self.tau_syn_decay_slow_gl,
-                                  tau_syn_rise=self.tau_syn_rise_gl,
-                                  a_syn_decay_slow=self.a_syn_decay_slow_gl,
-                                  a_syn_decay_fast=self.a_syn_decay_fast_gl,
-                                  N=self.N_MU,
-                                  out_connectivity=self.out_connectivity_gl)
-
-        aMNs = AlphaMotorNeuron(tau_mem=self.tau_mem, Vm_thr=self.Vm_thr_array, Vm_rest=self.Vm_rest, Rm=self.Rm,
-                                t_refrac=self.t_refrac, T=self.T, dt=self.dt,
-                                synapses=[acetyl_synapse, glycine_synapse], N=self.N_MU)
-
-        renshaw = RenshawCell(tau_mem=self.tau_mem, Vm_thr=self.Vm_thr_renshaw, Vm_rest=self.Vm_rest, Rm=self.Rm,
-                              T=self.T, dt=self.dt, delay=0,
-                              synapses=[acetyl_synapse_2],
-                              slope=self.slope_renshaw)#TODO own Vm_thr for optimisation
-
-        for i in range(self.N_MU):
-            self.motor_units.append(MotorUnit(muap=self.muaps[:, i], t=self.t))
-        # ==============================================================
-
-        workhorse = Workhorse(self.t, aMNs, renshaw)
-        self.alpha_motor_neurons = workhorse.run(input)
-
+        workhorse = Workhorse(self.t, self.aMNs, self.renshaw)
+        self.alpha_motor_neurons = workhorse.run(inp)
 
         muscle = Muscle(self.alpha_motor_neurons, self.motor_units)
         self.mep = muscle.get_mep()
         self.mep_p2p = np.max(self.mep) - np.min(self.mep)
 
 
-
-        return self.mep, self.mep_p2p,self.alpha_motor_neurons,renshaw  # TODO Return or just get the value yourself ?
-
-
 class Synapse(object):
     """
-    Synpase (conductance-based)
+    Synapse (conductance-based)
     """
 
     def __init__(self, T, dt, N, tau_syn_decay=None, tau_syn_rise=None,
                  tau_syn_decay_fast=None, tau_syn_decay_slow=None, a_syn_decay_fast=None, a_syn_decay_slow=None,
-                 in_connectivity=1,
-                 out_connectivity=1):
+                 connectivity=1):
         """
-                Initializes AlphaMotorNeuron instance
+        Initializes Synapse instance
 
-                Parameters
-                ----------
-                T : float
-                    Total simulation time in ms
-                dt : float
-                    Time-step to solve DEQ
-                tau_syn_decay : float
-                    Decay time constant of synapse in ms (set this parameter, if only one decay time constant is present and set
-                    tau_syn_decay_fast=None, tau_syn_decay_slow=None)
-                tau_syn_rise : float
-                    Rise time of synapse in ms
-                tau_syn_decay_fast : float
-                    Decay time constant of synapse of fast component in ms
-                tau_syn_decay_slow : float
-                    Decay time constant of synapse of slow component in ms
-                a_syn_decay_fast : float
-                    Proportion of fast decay time constant
-                a_syn_decay_slow : float
-                    Proportion of slow decay time constant
-                connectivity : float
-                    Scaling factor for synaptic strength
-                """
+        Parameters
+        ----------
+        T : float
+            Total simulation time in ms
+        dt : float
+            Time-step to solve DEQ
+        tau_syn_decay : float
+            Decay time constant of synapse in ms (set this parameter, if only one decay time constant is present and set
+            tau_syn_decay_fast=None, tau_syn_decay_slow=None)
+        tau_syn_rise : float
+            Rise time of synapse in ms
+        tau_syn_decay_fast : float
+            Decay time constant of synapse of fast component in ms
+        tau_syn_decay_slow : float
+            Decay time constant of synapse of slow component in ms
+        a_syn_decay_fast : float
+            Proportion of fast decay time constant
+        a_syn_decay_slow : float
+            Proportion of slow decay time constant
+        connectivity : float
+            Scaling factor for synaptic strength
+        """
 
         self.tau_syn_decay = tau_syn_decay
         self.T = T
         self.dt = dt
-        self.out_connectivity = out_connectivity
-        self.in_connectivity = in_connectivity
+        self.connectivity = connectivity
         self.tau_syn_rise = tau_syn_rise
         self.tau_syn_decay_fast = tau_syn_decay_fast
         self.tau_syn_decay_slow = tau_syn_decay_slow
@@ -327,7 +390,8 @@ class Synapse(object):
                 tau_syn_decay_slow is None and tau_syn_decay_fast is None):
             self.synapse_single_decay_rise = True
 
-        if self.synapse_single_decay + self.synapse_double_decay + self.synapse_single_decay_rise + self.synapse_double_decay_rise != 1:
+        if self.synapse_single_decay + self.synapse_double_decay + self.synapse_single_decay_rise + \
+                self.synapse_double_decay_rise != 1:
             raise AssertionError("Please check assignment of time constants...")
 
         # initialize arrays
@@ -345,16 +409,14 @@ class Synapse(object):
 
         Parameters
         ----------
-        input: ndarray of float [n_t]
+        inp: ndarray of float [n_t]
             Input signal
         i: int
             Current index
         """
 
-        inp[i] = inp[i] * self.in_connectivity
         # double exponential kernel with rise time and slow + fast decay times
         if self.synapse_double_decay_rise:
-
             self.g_f[:, i] = self.g_f[:, i - 1] + self.x_f[:, i - 1] * self.dt
             self.x_f[:, i] = self.x_f[:, i - 1] + (
                     (inp[i] - self.x_f[:, i - 1]) * (self.tau_syn_rise + self.tau_syn_decay_fast) -
@@ -364,7 +426,7 @@ class Synapse(object):
                     (inp[i] - self.x_s[:, i - 1]) * (self.tau_syn_rise + self.tau_syn_decay_slow) -
                     self.g_s[:, i - 1]) / (self.tau_syn_rise * self.tau_syn_decay_slow) * self.dt
             self.g_tot[:, i] = self.a_syn_decay_fast * self.g_f[:, i] + self.a_syn_decay_slow * self.g_s[:, i]
-            
+
         # single exponential kernel with slow + fast decay times
         if self.synapse_double_decay:
             self.g_s[:, i] = self.g_s[:, i - 1] + ((0 - self.g_s[:, i - 1]) / self.tau_syn_decay_slow) * self.dt + inp[
@@ -383,10 +445,9 @@ class Synapse(object):
             self.g_tot[:, i] = self.g_tot[:, i - 1] + self.x[:, i - 1] * self.dt
             self.x[:, i] = self.x[:, i - 1] + ((inp[i] - self.x[:, i - 1]) * (self.tau_syn_rise + self.tau_syn_decay) -
                                                self.g_tot[:, i - 1]) / (
-                                       self.tau_syn_rise * self.tau_syn_decay) * self.dt
+                                   self.tau_syn_rise * self.tau_syn_decay) * self.dt
 
-
-        return self.g_tot[:, i] * self.out_connectivity  # Apply weight to output
+        return self.g_tot[:, i] * self.connectivity  # Apply weight to output
 
 
 class RenshawCell(object):
@@ -394,7 +455,7 @@ class RenshawCell(object):
     Renshaw Cell (Leaky integrate and fire)
     """
 
-    def __init__(self, tau_mem, Vm_thr, Vm_rest, Rm, T, dt, delay, synapses,slope):
+    def __init__(self, tau_mem, Vm_thr, Vm_rest, Rm, T, dt, delay, synapses, slope):
         """
         Initializes Renshaw Cell instance
 
@@ -408,8 +469,6 @@ class RenshawCell(object):
             Resting potential
         Rm : float
             Cell membrane resistance
-        t_refrac : float
-            Refractory time
         T : float
             Total simulation time in ms
         dt : float
@@ -422,7 +481,7 @@ class RenshawCell(object):
             Slope of Threshold
         """
         self.tau_mem = tau_mem
-        self.Vm_thr = Vm_thr # TODO proper Threhshold
+        self.Vm_thr = Vm_thr  # TODO proper Threshold
         self.Vm_rest = Vm_rest
         self.Rm = Rm
 
@@ -430,31 +489,28 @@ class RenshawCell(object):
         self.dt = dt
         self.t_rest = 0
 
-        self.max_firerate = 23.12 #TODO Proper value
+        self.max_firerate = 23.12  # TODO Proper value
         self.slope = slope
-
-
         self.delay = delay
 
-
         # Synapses
         self.acetyl_synapse = synapses[0]  # excitatory (Alpha Motor Neuron)
 
         # initialize arrays
         self.t = np.linspace(0, self.T, int(self.T / self.dt + 1))
         self.g_tot = np.zeros(len(self.t))
-        self.Vm = Vm_rest * np.ones(len(self.t),dtype=np.float128)
+        self.Vm = Vm_rest * np.ones(len(self.t), dtype=np.float128)
         self.spike_times = np.zeros(len(self.t))
-        self.output = np.zeros(len(self.t),dtype=np.float128)
+        self.output = np.zeros(len(self.t), dtype=np.float128)
 
-    def step(self, input, t_, i):
+    def step(self, inp, t_, i):
         """
         Solves the DEQ and computes membrane potential.
         Saves membrane potential in self.Vm and spike times in self.spike_times
 
         Parameters
         ----------
-        input: ndarray of float [n_t]
+        inp: ndarray of float [n_t]
             Input signal
         t_ : floatf
             Current time step
@@ -462,16 +518,17 @@ class RenshawCell(object):
             Current index
         """
 
+        self.g_tot[i] = np.sum(self.acetyl_synapse.step(inp, i))
 
-        self.g_tot[i] = np.sum(self.acetyl_synapse.step(input, i))
+        self.Vm[i] = self.Vm[i - 1] + (-(self.Vm[i - 1] - self.Vm_rest) - self.Vm[i - 1] * self.g_tot[
+            i] * self.Rm) / self.tau_mem * self.dt
 
-        self.Vm[i] = self.Vm[i - 1] + (-(self.Vm[i - 1] - self.Vm_rest) - self.Vm[i - 1] * self.g_tot[i] * self.Rm)/self.tau_mem * self.dt
-
-        #A. Spiegler potential to Rate
-        #np.seterr('raise')
-        self.output[i] = 2*self.max_firerate / (1+(logsumexp(self.slope*(self.Vm_thr-self.Vm[i]))))
+        # A. Spiegler potential to Rate
+        # np.seterr('raise')
+        self.output[i] = 2 * self.max_firerate / (1 + (logsumexp(self.slope * (self.Vm_thr - self.Vm[i]))))
 
         return self.output[i]
+
         # if(self.Vm[i] >= self.Vm_thr):
         #     self.spike_times[i] = t_
         #     self.Vm[i] = self.Vm_rest
@@ -523,25 +580,25 @@ class AlphaMotorNeuron(object):
         self.is_in_refrac = np.ones(N)
 
         # Synapses
-        self.acetyl_synapse = synapses[0]  # excitatory (Motor Cortex)
-        self.glycin_synapse = synapses[1]  # inhibitory (Renshaw Cell)
+        self.acetyl_synapse = synapses[0]  # excitatory (from motor cortex)
+        self.glycin_synapse = synapses[1]  # inhibitory (from Renshaw cell)
 
         # initialize arrays
         self.t = np.linspace(0, self.T, int(self.T / self.dt + 1))
         self.g_tot = np.zeros((N, len(self.t)))
-        self.Vm = Vm_rest * np.ones((N, len(self.t)),dtype=np.float128)
-        self.spike_times = np.zeros((N,len(self.t)))
+        self.Vm = Vm_rest * np.ones((N, len(self.t)), dtype=np.float128)
+        self.spike_times = np.zeros((N, len(self.t)))
 
         self.anySpike = 0
 
-    def step(self, input, renshaw_feedback, t_, i):
+    def step(self, inp, renshaw_feedback, t_, i):
         """
         Solves the DEQ and computes membrane potential.
         Saves membrane potential in self.Vm and spike times in self.spike_times
 
         Parameters
         ----------
-        input: ndarray of float [n_t]
+        inp: ndarray of float [n_t]
             Input signal
         renshaw_feedback: ndarray of float
             Feedback from Renshaw Cell
@@ -550,39 +607,34 @@ class AlphaMotorNeuron(object):
         i : int
             Current index
         """
-        x = 0#self.glycin_synapse.step(renshaw_feedback, i)
-        y = self.acetyl_synapse.step(input, i)
+        x = 0  # self.glycin_synapse.step(renshaw_feedback, i)
+        y = self.acetyl_synapse.step(inp, i)
 
-        self.g_tot[:, i] = np.where(x+y <0, 0, x+y)
+        self.g_tot[:, i] = np.where(x + y < 0, 0, x + y)
 
         # if t_ > self.t_rest:
-        self.Vm[:, i] = self.Vm[:, i - 1] + self.is_in_refrac * ((-(self.Vm[:, i - 1] - self.Vm_rest) - self.Vm[:,i - 1] * self.g_tot[:,i] * self.Rm) / self.tau_mem * self.dt)
+        self.Vm[:, i] = self.Vm[:, i - 1] + self.is_in_refrac * \
+                        ((-(self.Vm[:, i - 1] - self.Vm_rest) - self.Vm[:, i - 1] * self.g_tot[:, i] * self.Rm) /
+                         self.tau_mem * self.dt)
         firing_neurons = zip(*np.where(self.Vm[:, i] >= self.Vm_thr))
 
         for f in firing_neurons:
-            if(self.is_in_refrac[f] == 1):
+            if self.is_in_refrac[f] == 1:
                 self.spike_times[f, i] = t_
                 self.Vm[f, i] = self.Vm_rest
                 self.t_rest[f] = t_ + self.t_refrac
                 self.is_in_refrac[f] = 0
                 self.anySpike = 1
 
-
-        for k,t in enumerate(self.t_rest):
+        for k, t in enumerate(self.t_rest):
             if t_ > self.t_rest[k]:
                 self.is_in_refrac[k] = 1
 
-
         # spike event (reset to Vm_rest and pause integrating during refractory period)
-
         if self.anySpike == 1:
-
             self.anySpike = 0
             return 1 / self.dt
 
-
-
-
         # if np.any(self.Vm[:, i] >= self.Vm_thr):
         #     # check if still in refract
         #     np.where(t_ > self.t_rest and self.Vm[:, i] >= self.Vm_thr, self.is_in_refrac, self.is_in_refrac + 1)
@@ -621,7 +673,7 @@ class MotorUnit(object):
         self.muap = muap
         self.t = t
 
-    def get_muap(self, spike_times,k):
+    def get_muap(self, spike_times, k):
         """
         Adds MUAPs of the motorunit at given spiketimes
 
@@ -637,10 +689,8 @@ class MotorUnit(object):
         """
         muap_total = np.zeros(len(self.t))
 
-
-        for tau in spike_times[k,:]:
-
-            if not (tau ==0):
+        for tau in spike_times[k, :]:
+            if not tau == 0:
                 try:
                     idx_tau = np.argmin(np.abs(self.t - tau))
                     if len(muap_total) - idx_tau < len(self.muap):
@@ -655,7 +705,7 @@ class MotorUnit(object):
 
 class Muscle(object):
     """
-    Muscle containing multiple motor units
+    Muscle containing multiple motor unit instances
     """
 
     def __init__(self, alpha_motor_neurons, motor_units):
@@ -685,105 +735,6 @@ class Muscle(object):
 
     def get_mep(self):
         for k in range(self.N):
-           self.mep += self.motor_units[k].get_muap(self.alpha_motor_neurons.spike_times,k)
+            self.mep += self.motor_units[k].get_muap(self.alpha_motor_neurons.spike_times, k)
 
         return self.mep
-
-# def spindle(self):
-#
-#     import matplotlib.pyplot as plt
-#     fig,ax = plt.subplots()
-# ax.plot(self.t,self.inp)
-# hz_500_signal = np.arange(30,self.T,2) #Delay auf 20-50ms
-# print(hz_500_signal)
-# print(self.t)
-# self.inp[np.nonzero(hz_500_signal[:, None] == self.t)[1]] = 1
-# ax.plot(self.t,self.inp)
-# plt.show()
-
-# Paper Values Lin 2002
-
-# out = np.zeros(len(self.t)+1)
-# self.g_g = 40 # impulse/s
-# self.g_f = 4 # N
-# self.f_me = np.ones(len(self.t)+1)*0.00001 # N (random choosen to test)
-# for i in range(len(self.t)):
-#     inp = self.g_g + np.log(self.f_me / self.g_f + 1)
-#
-#     if i == 0:
-#         A = 40 * ((1.7 / self.dt ** 2) * (inp[i + 1] - 2 * inp[i] +0) + (
-#                     2.58 / (2 * self.dt) * (inp[i + 1] - 0)) + 0.4 * inp[i])
-#         B = (-2.2 / (2 * self.dt) * (out[i + 1] - 0)) - (1 / self.dt ** 2) * out[i + 1] + (
-#                     -1 / self.dt ** 2) * 0
-#     else:
-#         A = 40 * ((1.7 / self.dt ** 2) * (inp[i + 1] - 2 * inp[i] + inp[i - 1]) + (
-#                     2.58 / (2 * self.dt) * (inp[i + 1] - inp[i - 1])) + 0.4 * inp[i])
-#         B = (-2.2 / (2 * self.dt) * (out[i + 1] - out[i - 1])) - (1 / self.dt ** 2) * out[i + 1] + (
-#                     -1 / self.dt ** 2) * out[i - 1]
-#
-#
-#     out[i] = ((-2/self.dt)+0.4)*(A+B)
-
-# ax.plot(out)
-# plt.show()
-
-
-# N=0, c1=48144, c2=45845, alpha1=0.0326, alpha2=0.034,
-# a=220, b_h=10, K_sh=35, K_ph=5
-
-# self.t = t
-# # Values from Patet2015 Paper
-# self.c1 = c1
-# self.c2 = c2
-# self.alpha1 = alpha1
-# self.alpha2 = alpha2
-# self.T0 = (t[1]-t[0])#/1000 # Has to be converted to seconds
-#
-# self.a = a  # Hz/cm
-# self.b_h = b_h  # g*s/cm
-# self.K_sh = K_sh  # g/cm spindle spring constants
-# self.K_ph = K_ph  # g/cm spindle spring constants TODO find suitable values
-# self.beta = (self.K_ph + self.K_sh) / self.b_h
-#
-# # Values from Patet2015 Paper
-#
-# self.w = np.zeros(len(t))
-# self.active_f = np.zeros(len(t))
-# self.spindle_stretch = np.zeros(len(t))
-# #self.spindle_stretch[np.flatnonzero(t == 50)[0]: np.flatnonzero(t == 150)[0]] = 0.1  # TODO Test value proberly implement it (simulation ?)
-# self.spindle_stretch[50:200] = 0.1
-# self.S_Ia = np.zeros(len(t))
-# self.S_II = np.zeros(len(t))
-#
-# self.number_spikes = 0
-#
-# #Only used to calculated the MEP
-# self.muap = muap
-# self.spike_times = spike_times
-# self.N = N
-
-# def spindle(self,i,spikes_t):
-#
-#     if not i == len(self.t) - 1:
-#         # TODO Move to Muscle
-#
-#         #for _, sp in enumerate(spikes_t):
-#            # print(self.t[i])
-#         sp = spikes_t[-1]
-#
-#         self.active_f[i] += self.c1 * np.exp(-(1 / self.alpha1) * (self.t[i] - sp)) - self.c2 * np.exp(
-#             -(1 / self.alpha2) * (self.t[i] - sp)) # Rember to add *1/1000 to the exponent
-#
-#
-#         self.Bh1 = -((self.K_sh ** 2 * (np.exp(self.beta * self.T0) - 1)) / (
-#                     (self.K_ph + self.K_sh) * np.exp(self.beta * self.T0)))
-#         self.Bh2 = ((self.K_sh * (np.exp(self.beta * self.T0) - 1)) / (
-#                     (self.K_ph + self.K_sh) * np.exp(self.beta * self.T0)))
-#         test = np.array([self.Bh1, self.Bh2], dtype=np.float64)
-#         test2 = np.array([self.spindle_stretch[i], self.active_f[i]], dtype=np.float64)
-#         self.w[i+1] = (self.w[i] + self.Bh1*self.spindle_stretch[i]+self.Bh2 * self.active_f[i])*(np.exp(-self.beta*self.T0))
-#
-#         self.S_Ia[i] = self.a / self.K_sh * self.w[i] + self.a * self.spindle_stretch[i]
-#         self.S_II[i] = -self.a / self.K_sh * self.w[i]
-#
-#     return self.w, self.active_f,self.spindle_stretch,self.S_Ia,self.S_II
