@@ -1,26 +1,16 @@
+import os
 import time
+import warnings
+
+import h5py
+import numpy as np
 import yaml
 import inspect
 
 from lmfit import Model
 
 from sklearn.neighbors import KernelDensity
-
-from .hdf5_io import write_data_hdf5_surf
-from .hdf5_io import write_geo_hdf5_surf
-from .main import map_data_to_surface
-from .roi import load_roi_surface_obj_from_hdf5
-from .roi import make_GM_WM_surface
-from .subject import load_subject
-from .util import find_nearest
-from .exp.exp import *
-from .exp.Mep import linear
-from .exp.Mep import exp0
-from .exp.Mep import sigmoid
-from .exp.Mep import sigmoid4
-from .exp.Mep import sigmoid_log
-from .exp.Mep import sigmoid4_log
-from .exp.Mep import Mep
+import pynibs
 
 
 def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep, mep_params, e, n_samples=100):
@@ -88,23 +78,6 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
 
         # calculate corresponding electric field values per condition
         for elm_idx, elmIdx in enumerate(elm_idx_list):
-
-            # if elmIdx == 2139:
-            #     print("M1 high c shift")
-            #     # points = np.array([[-23.143, -27.095, 72.450], [-22.933, -27.787, 71.839], [-23.703, -27.064, 71.996]])
-            # elif elmIdx == 4466:
-            #     print("M1 high c var")
-            #     # points = np.array([[-28.093, -20.740, 67.842], [-27.752, -21.2499, 67.2484], [-28.410, -20.538, 66.715]])
-            # elif elmIdx == 2007:
-            #     print("SS high c shift")
-            #     # points = np.array([[-30.2457, -28.1971, 67.3983], [-30.8065, -29.1365, 66.8159], [-31.062, -28.0206, 66.7044]])
-            # elif elmIdx == 5819:
-            #     print("M1 max c var")
-            #     # points = np.array([[-22.3091, -18.2346, 71.6757], [-22.0561, -18.2764, 71.5941], [-21.8947, -18.5464, 71.7875]])
-            # elif elmIdx == 3110:
-            #     print("M1 hotspot c var")
-            #     # points = np.array([[-21.2291, -24.2112, 73.524], [-21.3194, -25.2833, 73.4599], [-21.9384, -24.3019, 73.3017]])
-
             e_curve = []
             stepsize_local_shift = []
 
@@ -114,7 +87,6 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
                 stepsize_local_shift.append(e_curve[-1][1] - e_curve[-1][0])
 
             # KERNEL CODE STARTED HERE
-            ############################################################################################################
             e_min = np.min(e_curve, axis=1)  # minima of electric field for every condition
             # ceil to .stepsize
             e_min = np.ceil(e_min / stepsize) * stepsize
@@ -137,10 +109,10 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
 
                 for idx in range(n_conditions):
                     # lower boundary idx of e_x_cond in e_x_global
-                    start_ind[idx] = find_nearest(e_x_global, e_min[idx])
+                    start_ind[idx] = pynibs.find_nearest(e_x_global, e_min[idx])
 
                     # upper boundary idx of e_x_cond in e_x_global
-                    stop_ind[idx] = find_nearest(e_x_global, e_max[idx])
+                    stop_ind[idx] = pynibs.find_nearest(e_x_global, e_max[idx])
 
                 # get tau distances for all conditions vs reference condition
                 # distances for ref,i == i,ref. i,i == 0. So only compute upper triangle of matrix
@@ -187,8 +159,8 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
                                               intensities_max[idx]
 
                         # start index of initially shifted and stretched mep curve
-                        start_idx_mep_initial_shift = find_nearest(e_x_global,
-                                                                   e_mep_initial_shift * intensities_min[idx])
+                        start_idx_mep_initial_shift = pynibs.find_nearest(e_x_global,
+                                                                          e_mep_initial_shift * intensities_min[idx])
 
                         mep_shift = mep[idx].eval(intens_mep, mep_params_cond[idx])
 
@@ -204,14 +176,12 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
 
                         # length of shifted curve as a function of position (gets longer while shifting)
                         len_mep_idx_shift = np.round(np.linspace(
-                            len_e_mep_start,
-                            len_e_mep_end,
-                            len_e_mep_start + len_e_ref + 2 * stepsize_local_shift[reference_idx]))
+                                len_e_mep_start,
+                                len_e_mep_end,
+                                len_e_mep_start + len_e_ref + 2 * stepsize_local_shift[reference_idx]))
 
                         # construct shift array (there are less 0 at the beginning and more at the end because the mep
                         # curve is stretched during shifting)
-                        # shift_array = np.zeros(len_total)
-                        # shift_array[(len_e_mep_start + 1):(len_e_mep_start + 1 + len_e_ref)] = mep_curve[reference_idx]
                         stepsize_local_shift_intens = (intensities_max[reference_idx] -
                                                        intensities_min[reference_idx]) / \
                                                       n_samples
@@ -229,7 +199,7 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
                         slice_indices = np.outer(len_mep_idx_shift[:, np.newaxis],
                                                  np.linspace(0, 1, len_e_mep_start)[np.newaxis, :])
                         slice_indices = np.round(
-                            np.add(slice_indices, np.arange(slice_indices.shape[0])[:, np.newaxis])).astype(int)
+                                np.add(slice_indices, np.arange(slice_indices.shape[0])[:, np.newaxis])).astype(int)
 
                         # the error is y-difference between mep[idx] and mep[reference].zero_padded
                         err = np.sqrt(np.sum((shift_array[slice_indices] - mep_shift) ** 2, axis=1))
@@ -245,36 +215,6 @@ def congruence_factor_curveshift_workhorse_stretch_correction(elm_idx_list, mep,
     return congruence_factor
 
 
-# PLOTTING FUNCTIONS TO CHECK ABOVE CODE DURING DEBUGGING
-# import matplotlib.pyplot as plt
-#
-# # raw e-curves
-# label = []
-# for i in range(n_conditions):
-#     plt.plot(e_curve[i], mep_curve[i])
-#     label.append(str(i))
-# plt.legend(label)
-# plt.show()
-#
-# # error
-# plt.plot(err)
-# plt.show()
-#
-# # shifted e-curves (no stretch)
-# label = []
-# for i in range(n_conditions):
-#     x_mep_curve_shifted_stretched = (e_min[i] + t_cond[0, i]) / intensities_min[i] * np.linspace(intensities_min[i],
-#                                                                                                  intensities_max[i],
-#                                                                                                  100)
-#
-#     y_mep_curve_shifted_stretched = mep[i].eval(np.linspace(intensities_min[i], intensities_max[i], 100), mep_params_cond[i])
-#
-#     plt.plot(x_mep_curve_shifted_stretched, y_mep_curve_shifted_stretched)
-#     label.append(str(i))
-# plt.legend(label, loc=2)
-# plt.show()
-
-
 def congruence_factor_curveshift_workhorse_stretch_correction_new(mep, mep_params, e, n_samples=100, ref_idx=0):
     """
     Worker function for congruence factor computation - call from multiprocessing.pool
@@ -350,8 +290,8 @@ def congruence_factor_curveshift_workhorse_stretch_correction_new(mep, mep_param
         mep_c_shift = mep[idx].eval(i_c_shift, mep_params_c[i])
         # generate index shift list to compare curves
         slice_indices = np.outer(
-            np.round(np.linspace(n_c_before[i], n_c_after[i], n_c_before[i] + n_samples))[:, np.newaxis],
-            np.linspace(0, 1, n_c_before[i])[np.newaxis, :])
+                np.round(np.linspace(n_c_before[i], n_c_after[i], n_c_before[i] + n_samples))[:, np.newaxis],
+                np.linspace(0, 1, n_c_before[i])[np.newaxis, :])
         slice_indices = np.round(slice_indices + np.arange(slice_indices.shape[0])[:, np.newaxis])
         slice_indices = (slice_indices + (np.max(n_c_before) - n_c_before[i])).astype(int)
         # the error is y-difference between mep[idx] and mep[reference].zero_padded
@@ -476,8 +416,8 @@ def congruence_factor_curveshift_workhorse_stretch_correction_sign_new(mep, mep_
             mep_c_shift = mep[idx].eval(i_c_shift, mep_params_c[i])
             # generate index shift list to compare curves
             slice_indices = np.outer(
-                np.round(np.linspace(n_c_before[i], n_c_after[i], n_c_before[i] + n_samples))[:, np.newaxis],
-                np.linspace(0, 1, n_c_before[i])[np.newaxis, :])
+                    np.round(np.linspace(n_c_before[i], n_c_after[i], n_c_before[i] + n_samples))[:, np.newaxis],
+                    np.linspace(0, 1, n_c_before[i])[np.newaxis, :])
             slice_indices = np.round(slice_indices + np.arange(slice_indices.shape[0])[:, np.newaxis])
             slice_indices = (slice_indices + (np.max(n_c_before) - n_c_before[i])).astype(int)
             # the error is y-difference between mep[idx] and mep[reference].zero_padded
@@ -618,22 +558,6 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
         # calculate corresponding electric field values per condition
         for elm_idx, elmIdx in enumerate(elm_idx_list):
 
-            # if elmIdx == 2139:
-            #     print("M1 high c shift")
-            #     # points = np.array([[-23.143, -27.095, 72.450], [-22.933, -27.787, 71.839], [-23.703, -27.064, 71.996]])
-            # elif elmIdx == 4466:
-            #     print("M1 high c var")
-            #     # points = np.array([[-28.093, -20.740, 67.842], [-27.752, -21.2499, 67.2484], [-28.410, -20.538, 66.715]])
-            # elif elmIdx == 2007:
-            #     print("SS high c shift")
-            #     # points = np.array([[-30.2457, -28.1971, 67.3983], [-30.8065, -29.1365, 66.8159], [-31.062, -28.0206, 66.7044]])
-            # elif elmIdx == 5819:
-            #     print("M1 max c var")
-            #     # points = np.array([[-22.3091, -18.2346, 71.6757], [-22.0561, -18.2764, 71.5941], [-21.8947, -18.5464, 71.7875]])
-            # elif elmIdx == 3110:
-            #     print("M1 hotspot c var")
-            #     # points = np.array([[-21.2291, -24.2112, 73.524], [-21.3194, -25.2833, 73.4599], [-21.9384, -24.3019, 73.3017]])
-
             e_curve = []
             stepsize_local_shift = []
 
@@ -643,7 +567,6 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
                 stepsize_local_shift.append(e_curve[-1][1] - e_curve[-1][0])
 
             # KERNEL CODE STARTED HERE
-            ############################################################################################################
             e_min = np.min(e_curve, axis=1)  # minima of electric field for every condition
             # ceil to .stepsize
             e_min = np.ceil(e_min / stepsize) * stepsize
@@ -666,10 +589,10 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
 
                 for idx in range(n_conditions):
                     # lower boundary idx of e_x_cond in e_x_global
-                    start_ind[idx] = find_nearest(e_x_global, e_min[idx])
+                    start_ind[idx] = pynibs.find_nearest(e_x_global, e_min[idx])
 
                     # upper boundary idx of e_x_cond in e_x_global
-                    stop_ind[idx] = find_nearest(e_x_global, e_max[idx])
+                    stop_ind[idx] = pynibs.find_nearest(e_x_global, e_max[idx])
 
                 # get tau distances for all conditions vs reference condition
                 # distances for ref,i == i,ref. i,i == 0. So only compute upper triangle of matrix
@@ -716,8 +639,8 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
                                               intensities_max[idx]
 
                         # start index of initially shifted and stretched mep curve
-                        start_idx_mep_initial_shift = find_nearest(e_x_global,
-                                                                   e_mep_initial_shift * intensities_min[idx])
+                        start_idx_mep_initial_shift = pynibs.find_nearest(e_x_global,
+                                                                          e_mep_initial_shift * intensities_min[idx])
 
                         mep_shift = mep[idx].eval(intens_mep, mep_params_cond[idx])
 
@@ -733,9 +656,9 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
 
                         # length of shifted curve as a function of position (gets longer while shifting)
                         len_mep_idx_shift = np.round(np.linspace(
-                            len_e_mep_start,
-                            len_e_mep_end,
-                            len_e_mep_start + len_e_ref + 2 * stepsize_local_shift[reference_idx]))
+                                len_e_mep_start,
+                                len_e_mep_end,
+                                len_e_mep_start + len_e_ref + 2 * stepsize_local_shift[reference_idx]))
 
                         # construct shift array (there are less 0 at the beginning and more at the end because the mep
                         # curve is stretched during shifting)
@@ -758,7 +681,7 @@ def congruence_factor_curveshift_workhorse_stretch_correction_variance(elm_idx_l
                         slice_indices = np.outer(len_mep_idx_shift[:, np.newaxis],
                                                  np.linspace(0, 1, len_e_mep_start)[np.newaxis, :])
                         slice_indices = np.round(
-                            np.add(slice_indices, np.arange(slice_indices.shape[0])[:, np.newaxis])).astype(int)
+                                np.add(slice_indices, np.arange(slice_indices.shape[0])[:, np.newaxis])).astype(int)
 
                         # the error is y-difference between mep[idx] and mep[reference].zero_padded
                         err = np.sqrt(np.sum((shift_array[slice_indices] - mep_shift) ** 2, axis=1))
@@ -794,8 +717,8 @@ def congruence_factor_variance_workhorse(elm_idx_list, mep, mep_params, e, old_s
         - len(e) = n_cond
         - len(e[0]) = n_comp (e.g: e_mag = e[0]))
     old_style: boolean (default: True)
-        True:  Weight var(x_0_prime(r) with mean(e(r) * mean(Stimulator Intensity), taken from MEP object
-        False: Weight var(x_0_prime(r) with mean(E(r)), taken from e
+        True:  Weight var(x_0_prime(r)) with mean(e(r) * mean(Stimulator Intensity), taken from MEP object
+        False: Weight var(x_0_prime(r)) with mean(E(r)), taken from e
 
     Returns
     -------
@@ -1187,15 +1110,18 @@ def stimulation_threshold(elm_idx_list, mep, mep_params, n_samples, e, c_factor_
     c_function : fun
         Defines the function to use during c_gpc to calculate the congruence factor
         - congruence_factor_curveshift_workhorse: determines the average curve shift (without stretch correction)
-        - congruence_factor_curveshift_workhorse_stretch_correction: determines the average curve shift (with stretch correction)
-        - congruence_factor_curveshift_workhorse_stretch_correction_variance: determines the average curve shift (with stretch correction and variance)
+        - congruence_factor_curveshift_workhorse_stretch_correction: determines the average curve shift
+          (with stretch correction)
+        - congruence_factor_curveshift_workhorse_stretch_correction_variance: determines the average curve shift
+          (with stretch correction and variance)
         - congruence_factor_variance_workhorse: evaluates the variance of the shifting and stretching parameters
     t_function : fun
         Defines the function to determine the stimulation_threshold
         - stimulation_threshold_mean_mep_threshold: uses mep_threshold to determine the corresponding e_threshold over
         all conditions and takes the average values as the stimulation threshold
-        - stimulation_threshold_sigmoid: Fits a new sigmoid using all datapoints in the mep-vs-E space and evaluates
-        the threshold from the turning point or the intersection of the derivative in the crossing point with the e-axis
+        - stimulation_threshold_pynibs.sigmoid: Fits a new pynibs.sigmoid using all datapoints in the mep-vs-E space and
+           evaluates the threshold from the turning point or the intersection of the derivative in the crossing point
+           with the e-axis
 
     Returns
     -------
@@ -1325,23 +1251,16 @@ def stimulation_threshold_mean_mep_threshold(elm_idx, mep_curve, intensities, e,
 
     for i_data in range(n_datasets):
         stim_threshold_avg[i_data] = np.mean(
-            stim_threshold_cond[i_data][np.logical_not(np.isnan(stim_threshold_cond[i_data]))])
+                stim_threshold_cond[i_data][np.logical_not(np.isnan(stim_threshold_cond[i_data]))])
         stim_threshold_std[i_data] = np.std(
-            stim_threshold_cond[i_data][np.logical_not(np.isnan(stim_threshold_cond[i_data]))])
-
-    # ###
-    # import matplotlib.pyplot as plt
-    # i_data = 0
-    # for i_cond in range(n_conditions):
-    #     plt.plot(e[i_cond][i_data][elm_idx[i_data][50]]*intensities[i_cond], mep_curve[i_cond], linestyle='', marker='o', )
-    # ###
+                stim_threshold_cond[i_data][np.logical_not(np.isnan(stim_threshold_cond[i_data]))])
 
     return stim_threshold_avg, stim_threshold_std
 
 
 def stimulation_threshold_sigmoid(elm_idx, mep_curve, intensities, e, mep_threshold):
     """
-    Determines the stimulation threshold by calculating an equivalent sigmoid over all conditions. The
+    Determines the stimulation threshold by calculating an equivalent pynibs.sigmoid over all conditions. The
     stimulation threshold is the electric field value where the mep curves exceed the value of mep_threshold (in [mV]).
 
     Parameters
@@ -1392,8 +1311,8 @@ def stimulation_threshold_sigmoid(elm_idx, mep_curve, intensities, e, mep_thresh
             e_all = np.hstack(e_all)
 
             # fit data to function
-            mep = Mep(intensities=e_all, mep=mep_curve_all, intensity_min_threshold=0, mep_min_threshold=0)
-            mep.fit = mep.run_fit_multistart(sigmoid,
+            mep = pynibs.Mep(intensities=e_all, mep=mep_curve_all, intensity_min_threshold=0, mep_min_threshold=0)
+            mep.fit = mep.run_fit_multistart(pynibs.sigmoid,
                                              x=e_all,
                                              y=mep_curve_all,
                                              p0=[70, 0.6, 2],
@@ -1409,7 +1328,7 @@ def stimulation_threshold_sigmoid(elm_idx, mep_curve, intensities, e, mep_thresh
                 mep.popt = np.asarray(mep.popt)
                 mep.cvar = np.asarray(mep.fit.covar)
                 mep.pstd = np.sqrt(np.diag(mep.cvar))
-                mep.fun = sigmoid
+                mep.fun = pynibs.sigmoid
 
                 # determine stimulation threshold
                 e_fit = np.linspace(np.min(e_all), np.max(e_all), 200)
@@ -1426,7 +1345,7 @@ def stimulation_threshold_sigmoid(elm_idx, mep_curve, intensities, e, mep_thresh
                     stim_threshold_elm[i_data][i_elm] = e_fit[e_threshold_idx[0]]
 
             except (AttributeError, ValueError):
-                print(' > Warning: sigmoid in element could not be fitted!')
+                print(' > Warning: pynibs.sigmoid in element could not be fitted!')
                 stim_threshold_elm[i_data][i_elm] = np.nan
 
         # determine mean threshold over all elements
@@ -1613,10 +1532,11 @@ def e_cog_workhorse(elm_idx_list, mep, mep_params, e):
         mep_params_cond.append(mep_params[start_idx:(start_idx + mep[i_cond].popt.size)])
         start_idx = start_idx + mep[i_cond].popt.size
 
-        # stimulator intensity in [A/us] for mean MEP amplitude, i.e. turning point of sigmoid (1st para of sigmoid)
+        # stimulator intensity in [A/us] for mean MEP amplitude, i.e. turning point of pynibs.sigmoid (1st para of
+        # sigmoid)
         intensity_mep_mean_vec[0, i_cond] = mep[i_cond].popt[0]
 
-        # mean MEP amplitude (function value at 1st parameter of sigmoid)
+        # mean MEP amplitude (function value at 1st parameter of pynibs.sigmoid)
         mep_mean_vec[0, i_cond] = mep[i_cond].eval(mep_params_cond[-1][0], mep_params_cond[-1])
 
     e_arr = np.array(e)
@@ -1671,11 +1591,11 @@ def extract_condition_combination(fn_config_cfg, fn_results_hdf5, conds, fn_out_
     n_qoi = len(e_qoi)
 
     # load subject object
-    subject = load_subject(fn_subject)
+    subject = pynibs.load_subject(fn_subject)
     mesh_folder = subject.mesh[mesh_idx]["mesh_folder"]
 
     # loading roi
-    roi = load_roi_surface_obj_from_hdf5(subject.mesh[mesh_idx]['fn_mesh_hdf5'])
+    roi = pynibs.load_roi_surface_obj_from_hdf5(subject.mesh[mesh_idx]['fn_mesh_hdf5'])
 
     # load results file
     c_extracted = []
@@ -1697,55 +1617,55 @@ def extract_condition_combination(fn_config_cfg, fn_results_hdf5, conds, fn_out_
 
     # map data to whole brain surface
     print(" > Mapping c-factor map to brain surface...")
-    c_mapped = map_data_to_surface(datasets=[c_extracted[qoi_idx][:, np.newaxis] for qoi_idx in range(n_qoi)],
-                                   points_datasets=[roi[roi_idx].node_coord_mid] * n_qoi,
-                                   con_datasets=[roi[roi_idx].node_number_list] * n_qoi,
-                                   fname_fsl_gm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_gm']),
-                                   fname_fsl_wm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_wm']),
-                                   delta=subject.roi[mesh_idx][roi_idx]['delta'],
-                                   input_data_in_center=True,
-                                   return_data_in_center=True,
-                                   data_substitute=-1)
+    c_mapped = pynibs.map_data_to_surface(datasets=[c_extracted[qoi_idx][:, np.newaxis] for qoi_idx in range(n_qoi)],
+                                          points_datasets=[roi[roi_idx].node_coord_mid] * n_qoi,
+                                          con_datasets=[roi[roi_idx].node_number_list] * n_qoi,
+                                          fname_fsl_gm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_gm']),
+                                          fname_fsl_wm=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_wm']),
+                                          delta=subject.roi[mesh_idx][roi_idx]['delta'],
+                                          input_data_in_center=True,
+                                          return_data_in_center=True,
+                                          data_substitute=-1)
 
     # recreate complete midlayer surface to write in .hdf5 geo file
-    points_midlayer, con_midlayer = make_GM_WM_surface(
-        gm_surf_fname=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_gm']),
-        wm_surf_fname=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_wm']),
-        delta=subject.roi[mesh_idx][roi_idx]['delta'],
-        X_ROI=None,
-        Y_ROI=None,
-        Z_ROI=None,
-        layer=1,
-        fn_mask=None)
+    points_midlayer, con_midlayer = pynibs.make_GM_WM_surface(
+            gm_surf_fname=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_gm']),
+            wm_surf_fname=os.path.join(mesh_folder, subject.mesh[mesh_idx]['fn_lh_wm']),
+            delta=subject.roi[mesh_idx][roi_idx]['delta'],
+            x_roi=None,
+            y_roi=None,
+            z_roi=None,
+            layer=1,
+            fn_mask=None)
 
     # write output files
     # save .hdf5 _geo file
     print(" > Creating .hdf5 geo file of mapped and roi only data ...")
-    write_geo_hdf5_surf(out_fn=fn_out_prefix + '_geo.hdf5',
-                        points=points_midlayer,
-                        con=con_midlayer,
-                        replace=True,
-                        hdf5_path='/mesh')
+    pynibs.write_geo_hdf5_surf(out_fn=fn_out_prefix + '_geo.hdf5',
+                               points=points_midlayer,
+                               con=con_midlayer,
+                               replace=True,
+                               hdf5_path='/mesh')
 
-    write_geo_hdf5_surf(out_fn=fn_out_prefix + '_geo_roi.hdf5',
-                        points=roi[roi_idx].node_coord_mid,
-                        con=roi[roi_idx].node_number_list,
-                        replace=True,
-                        hdf5_path='/mesh')
+    pynibs.write_geo_hdf5_surf(out_fn=fn_out_prefix + '_geo_roi.hdf5',
+                               points=roi[roi_idx].node_coord_mid,
+                               con=roi[roi_idx].node_number_list,
+                               replace=True,
+                               hdf5_path='/mesh')
 
     # save .hdf5 _data file
     print(" > Creating .hdf5 data file of mapped and roi only data ...")
-    write_data_hdf5_surf(data=c_mapped,
-                         data_names=['c_' + e_qoi[qoi_idx] for qoi_idx in range(n_qoi)],
-                         data_hdf_fn_out=fn_out_prefix + '_data.hdf5',
-                         geo_hdf_fn=fn_out_prefix + '_geo.hdf5',
-                         replace=True)
+    pynibs.write_data_hdf5_surf(data=c_mapped,
+                                data_names=['c_' + e_qoi[qoi_idx] for qoi_idx in range(n_qoi)],
+                                data_hdf_fn_out=fn_out_prefix + '_data.hdf5',
+                                geo_hdf_fn=fn_out_prefix + '_geo.hdf5',
+                                replace=True)
 
-    write_data_hdf5_surf(data=[c_extracted[qoi_idx][:, np.newaxis] for qoi_idx in range(n_qoi)],
-                         data_names=['c_' + e_qoi[qoi_idx] for qoi_idx in range(n_qoi)],
-                         data_hdf_fn_out=fn_out_prefix + '_data_roi.hdf5',
-                         geo_hdf_fn=fn_out_prefix + '_geo_roi.hdf5',
-                         replace=True)
+    pynibs.write_data_hdf5_surf(data=[c_extracted[qoi_idx][:, np.newaxis] for qoi_idx in range(n_qoi)],
+                                data_names=['c_' + e_qoi[qoi_idx] for qoi_idx in range(n_qoi)],
+                                data_hdf_fn_out=fn_out_prefix + '_data_roi.hdf5',
+                                geo_hdf_fn=fn_out_prefix + '_geo_roi.hdf5',
+                                replace=True)
 
 
 def dvs_likelihood(params, x, y, verbose=True, normalize=False, bounds=[(1, 2), (1, 2)]):
@@ -1779,7 +1699,7 @@ def dvs_likelihood(params, x, y, verbose=True, normalize=False, bounds=[(1, 2),
     # determine posterior of DVS model with test data
     x_pre = np.linspace(np.min(x), np.max(x), 200000)
     x_post = x_pre + np.random.normal(loc=0., scale=sigma_x, size=len(x_pre))
-    y_post = sigmoid(x_post, p=p) + np.random.normal(loc=0., scale=sigma_y, size=len(x_pre))
+    y_post = pynibs.sigmoid(x_post, p=p) + np.random.normal(loc=0., scale=sigma_y, size=len(x_pre))
 
     # bin data
     n_bins = 50
@@ -1851,7 +1771,7 @@ def single_fit(x, y, fun):
     limits = dict()
     init_vals = dict()
 
-    if fun == linear:
+    if fun == pynibs.linear:
         log_scale = False
         limits["m"] = [-100, 100]
         limits["n"] = [-100, 100]
@@ -1859,7 +1779,7 @@ def single_fit(x, y, fun):
         init_vals["m"] = 0.3
         init_vals["n"] = -1
 
-    if fun == exp0:
+    if fun == pynibs.exp0:
         log_scale = False
         limits["x0"] = [0, 1000]
         limits["r"] = [1e-12, 100]
@@ -1867,7 +1787,7 @@ def single_fit(x, y, fun):
         init_vals["x0"] = np.mean(x)  # 40
         init_vals["r"] = 10 / np.max(x)
 
-    if fun == sigmoid:
+    if fun == pynibs.sigmoid:
         log_scale = False
 
         limits["x0"] = [0, 1000]
@@ -1878,7 +1798,7 @@ def single_fit(x, y, fun):
         init_vals["amp"] = np.max(y)
         init_vals["r"] = 10 / np.max(x)
 
-    if fun == sigmoid4:
+    if fun == pynibs.sigmoid4:
         log_scale = False
 
         limits["x0"] = [0, 1000]
@@ -1891,7 +1811,7 @@ def single_fit(x, y, fun):
         init_vals["r"] = 10 / np.max(x)
         init_vals["y0"] = 1e-2
 
-    if fun == sigmoid_log:
+    if fun == pynibs.sigmoid_log:
         log_scale = True
 
         limits["x0"] = [0, 1000]
@@ -1902,7 +1822,7 @@ def single_fit(x, y, fun):
         init_vals["amp"] = np.max(y)
         init_vals["r"] = 10 / np.max(x)
 
-    if fun == sigmoid4_log:
+    if fun == pynibs.sigmoid4_log:
         log_scale = True
 
         limits["x0"] = [0, 1000]
