@@ -1,4 +1,3 @@
-from . import main
 import numpy as np
 import platform
 import scipy
@@ -8,8 +7,8 @@ import h5py
 import nibabel as nib
 import trimesh
 import tqdm
-from pynibs.util import sample_sphere
 from itertools import product
+import pynibs
 
 
 def get_mask(areas, fn_annot, fn_inflated_fs, fn_out):
@@ -36,7 +35,7 @@ def get_mask(areas, fn_annot, fn_inflated_fs, fn_out):
     vertices, colortable, label = nib.freesurfer.io.read_annot(fn_annot)
 
     # convert label from numpy bytes str to str
-    label = [str(l.astype(str)) for l in label]
+    label = [str(lab.astype(str)) for lab in label]
 
     idx_areas = [i_l for i_a, a in enumerate(areas) for i_l, l in enumerate(label) if a == l]
     ourvertices = np.zeros(len(vertices)).astype(bool)
@@ -55,17 +54,17 @@ def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_t
     """
     Parameters
     ----------
-    chunk: list of int
+    chunk: np.ndarray
         Indices of points the CPU thread is computing the element indices for
-    points_out: nparray of float [N_points x 3]
+    points_out: np.ndarray of float [N_points x 3]
         Coordinates of points, the tetrahedra indices are computed for
-    P1_all : nparray of float [N_tet x 3]
+    P1_all : np.ndarray of float [N_tet x 3]
         Coordinates of first point of tetrahedra
-    P2_all : nparray of float [N_tet x 3]
+    P2_all : np.ndarray of float [N_tet x 3]
         Coordinates of second point of tetrahedra
-    P3_all : nparray of float [N_tet x 3]
+    P3_all : np.ndarray of float [N_tet x 3]
         Coordinates of third point of tetrahedra
-    P4_all : nparray of float [N_tet x 3]
+    P4_all : np.ndarray of float [N_tet x 3]
         Coordinates of fourth point of tetrahedra
     N_points_total : int
         Total number of points
@@ -74,7 +73,7 @@ def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_t
 
     Returns
     -------
-    tet_idx_local: nparray of int [N_points]
+    tet_idx_local: np.ndarray of int [N_points]
     """
 
     tet_idx_local = np.zeros([chunk.shape[0]])
@@ -83,32 +82,32 @@ def elem_workhorse(chunk, points_out, P1_all, P2_all, P3_all, P4_all, N_points_t
     for i in chunk:
         start = time.time()
 
-        Vtest1 = main.calc_tetrahedra_volume_cross(np.tile(points_out[i, :], (P1_all.shape[0], 1)),
-                                                   P2_all,
-                                                   P3_all,
-                                                   P4_all)
-        tet_idx_bool_1 = (Vtest1 >= 0)
+        vtest1 = pynibs.mesh.calc_tetrahedra_volume_cross(np.tile(points_out[i, :], (P1_all.shape[0], 1)),
+                                                          P2_all,
+                                                          P3_all,
+                                                          P4_all)
+        tet_idx_bool_1 = (vtest1 >= 0)
         tet_idx_1 = np.nonzero(tet_idx_bool_1)[0]
 
-        Vtest2 = main.calc_tetrahedra_volume_cross(P1_all[tet_idx_1, :],
-                                                   np.tile(points_out[i, :], (tet_idx_1.shape[0], 1)),
-                                                   P3_all[tet_idx_1, :],
-                                                   P4_all[tet_idx_1, :])
-        tet_idx_bool_2 = (Vtest2 >= 0)
+        vtest2 = pynibs.mesh.calc_tetrahedra_volume_cross(P1_all[tet_idx_1, :],
+                                                          np.tile(points_out[i, :], (tet_idx_1.shape[0], 1)),
+                                                          P3_all[tet_idx_1, :],
+                                                          P4_all[tet_idx_1, :])
+        tet_idx_bool_2 = (vtest2 >= 0)
         tet_idx_2 = tet_idx_1[np.nonzero(tet_idx_bool_2)[0]]
 
-        Vtest3 = main.calc_tetrahedra_volume_cross(P1_all[tet_idx_2, :],
-                                                   P2_all[tet_idx_2, :],
-                                                   np.tile(points_out[i, :], (tet_idx_2.shape[0], 1)),
-                                                   P4_all[tet_idx_2, :])
-        tet_idx_bool_3 = (Vtest3 >= 0)
+        vtest3 = pynibs.mesh.calc_tetrahedra_volume_cross(P1_all[tet_idx_2, :],
+                                                          P2_all[tet_idx_2, :],
+                                                          np.tile(points_out[i, :], (tet_idx_2.shape[0], 1)),
+                                                          P4_all[tet_idx_2, :])
+        tet_idx_bool_3 = (vtest3 >= 0)
         tet_idx_3 = tet_idx_2[np.nonzero(tet_idx_bool_3)[0]]
 
-        Vtest4 = main.calc_tetrahedra_volume_cross(P1_all[tet_idx_3, :],
-                                                   P2_all[tet_idx_3, :],
-                                                   P3_all[tet_idx_3, :],
-                                                   np.tile(points_out[i, :], (tet_idx_3.shape[0], 1)))
-        tet_idx_bool_4 = (Vtest4 >= 0)
+        vtest4 = pynibs.mesh.calc_tetrahedra_volume_cross(P1_all[tet_idx_3, :],
+                                                          P2_all[tet_idx_3, :],
+                                                          P3_all[tet_idx_3, :],
+                                                          np.tile(points_out[i, :], (tet_idx_3.shape[0], 1)))
+        tet_idx_bool_4 = (vtest4 >= 0)
 
         tet_idx_local[i_local] = tet_idx_3[
             np.nonzero(tet_idx_bool_4)[0]].astype(int)
@@ -218,17 +217,17 @@ def determine_element_idx_in_mesh(fname, msh, points, compute_baricentric=False)
     ----------
     fname : str or None
         Filename of saved .txt file containing the element indices (no data is saved when fname=None or fname='')
-    points : nparray [N x 3] or list of ndarray
+    points : np.ndarray [N x 3] or list of ndarray
         List of points to be queried
-    compute_baricenters : bool
+    compute_baricentric : bool
         Wether or not to compute baricentric coordinates of the points
 
     Returns
     -------
-    th_with_points : nparray
+    th_with_points : np.ndarray
         List with the tetrahedron that contains each point. If the point is outside
         the mesh, the value will be -1
-    baricentric : nparray [n x 4](if compute_baricentric == True)
+    baricentric : np.ndarray [n x 4](if compute_baricentric == True)
         Baricentric coordinates of point. If the point is outside, a list of zeros
 
     Notes
@@ -239,8 +238,14 @@ def determine_element_idx_in_mesh(fname, msh, points, compute_baricentric=False)
     """
 
     if platform.system() == 'Linux':
-        import simnibs.cython_code.cython_msh as cython_msh
-        # import pynibs.pckg.simnibs.cython_code.cython_msh as cython_msh
+        import simnibs
+
+        if int(simnibs.__version__[0]) < 4:
+            import simnibs.cython_code.cython_msh as cython_msh
+            # import pynibs.pckg.simnibs.cython_code.cython_msh as cython_msh
+        else:
+            from simnibs import cython_msh
+
     else:
         raise OSError('This function works currently only under Linux!')
 
@@ -312,7 +317,7 @@ def determine_element_idx_in_mesh(fname, msh, points, compute_baricentric=False)
 
 
 def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_fname=None, delta=0.5,
-                       X_ROI=None, Y_ROI=None, Z_ROI=None,
+                       x_roi=None, y_roi=None, z_roi=None,
                        layer=1,
                        fn_mask=None,
                        refine=False):
@@ -340,13 +345,13 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
         Distance parameter where surface is generated 0...1 (default: 0.5)
         - 0 -> WM surface
         - 1 -> GM surface
-    X_ROI : list of float or None
+    x_roi : list of float or None
         Region of interest [Xmin, Xmax], whole X range if empty [0,0] or None
         (left - right)
-    Y_ROI : list of float or None
+    y_roi : list of float or None
         Region of interest [Ymin, Ymax], whole Y range if empty [0,0] or None
         (anterior - posterior)
-    Z_ROI : list of float or None
+    z_roi : list of float or None
         Region of interest [Zmin, Zmax], whole Z range if empty [0,0] or None
         (inferior - superior)
     layer : int
@@ -361,14 +366,20 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
 
     Returns
     -------
-    surface_points_upper : nparray of float [N_points x 3]
-        Coordinates (x, y, z) of surface + epsilon (in GM surface direction)
-    surface_points_middle : nparray of float [N_points x 3]
-        Coordinates (x, y, z) of surface
-    surface_points_lower : nparray of float [N_points x 3]
-        Coordinates (x, y, z) of surface - epsilon (in WM surface direction)
-    connectivity : nparray of int [N_tri x 3]
-        Connectivity of triangles (indexation starts at 0!)
+    if layer == 3:
+        surface_points_upper : np.ndarray of float
+            (N_points, 3) Coordinates (x, y, z) of surface + epsilon (in GM surface direction)
+        surface_points_middle : np.ndarray of float
+            (N_points, 3) Coordinates (x, y, z) of surface
+        surface_points_lower : np.ndarray of float
+            (N_points, 3) Coordinates (x, y, z) of surface - epsilon (in WM surface direction)
+        connectivity : np.ndarray of int
+             (N_tri x 3) Connectivity of triangles (indexation starts at 0!)
+    else:
+        surface_points_middle : np.ndarray of float
+            (N_points, 3) Coordinates (x, y, z) of surface
+        connectivity : np.ndarray of int
+             (N_tri x 3) Connectivity of triangles (indexation starts at 0!)
 
     Example
     -------
@@ -413,7 +424,7 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
 
         if midlayer_surf_fname[i] is not None:
             points_mid[i], con_mid[i] = nib.freesurfer.read_geometry(
-                os.path.join(mesh_folder, midlayer_surf_fname[i]))
+                    os.path.join(mesh_folder, midlayer_surf_fname[i]))
             con_mid[i] = con_mid[i] + max_idx_mid
             max_idx_mid = max_idx_mid + points_mid[i].shape[0]  # np.max(con_wm[i]) + 2
 
@@ -466,19 +477,19 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
     x_roi_wb, y_roi_wb, z_roi_wb = (False,) * 3
     if fn_mask is None:
         # crop to region of interest
-        if X_ROI is None or X_ROI == [0, 0]:
-            X_ROI = [-np.inf, np.inf]
+        if x_roi is None or x_roi == [0, 0]:
+            x_roi = [-np.inf, np.inf]
             x_roi_wb = True
-        if Y_ROI is None or Y_ROI == [0, 0]:
-            Y_ROI = [-np.inf, np.inf]
+        if y_roi is None or y_roi == [0, 0]:
+            y_roi = [-np.inf, np.inf]
             y_roi_wb = True
-        if Z_ROI is None or Z_ROI == [0, 0]:
-            Z_ROI = [-np.inf, np.inf]
+        if z_roi is None or z_roi == [0, 0]:
+            z_roi = [-np.inf, np.inf]
             z_roi_wb = True
 
-        roi_mask_bool = (surface_points_middle[:, 0] > min(X_ROI)) & (surface_points_middle[:, 0] < max(X_ROI)) & \
-                        (surface_points_middle[:, 1] > min(Y_ROI)) & (surface_points_middle[:, 1] < max(Y_ROI)) & \
-                        (surface_points_middle[:, 2] > min(Z_ROI)) & (surface_points_middle[:, 2] < max(Z_ROI))
+        roi_mask_bool = (surface_points_middle[:, 0] > min(x_roi)) & (surface_points_middle[:, 0] < max(x_roi)) & \
+                        (surface_points_middle[:, 1] > min(y_roi)) & (surface_points_middle[:, 1] < max(y_roi)) & \
+                        (surface_points_middle[:, 2] > min(z_roi)) & (surface_points_middle[:, 2] < max(z_roi))
         roi_mask_idx = np.where(roi_mask_bool)
 
     else:
@@ -514,13 +525,13 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
                                faces=con_cropped_reform)
         mesh.export(os.path.join(mesh_folder, "roi", "tmp", "roi.stl"))
 
-        main.refine_surface(fn_surf=os.path.join(mesh_folder, "roi", "tmp", "roi.stl"),
-                            fn_surf_refined=os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"),
-                            center=[0, 0, 0],
-                            radius=np.inf,
-                            verbose=True,
-                            repair=False,
-                            remesh=False)
+        pynibs.mesh.refine_surface(fn_surf=os.path.join(mesh_folder, "roi", "tmp", "roi.stl"),
+                                   fn_surf_refined=os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"),
+                                   center=[0, 0, 0],
+                                   radius=np.inf,
+                                   verbose=True,
+                                   repair=False,
+                                   remesh=False)
 
         roi = trimesh.load(os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"))
         con_cropped_reform = roi.faces
@@ -534,12 +545,12 @@ def make_GM_WM_surface(gm_surf_fname, wm_surf_fname, mesh_folder, midlayer_surf_
                                        faces=con_cropped_reform)
                 mesh.export(os.path.join(mesh_folder, "roi", "tmp", "roi.stl"))
 
-                main.refine_surface(fn_surf=os.path.join(mesh_folder, "roi", "tmp", "roi.stl"),
-                                    fn_surf_refined=os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"),
-                                    center=[0, 0, 0],
-                                    radius=np.inf,
-                                    verbose=True,
-                                    repair=False)
+                pynibs.mesh.refine_surface(fn_surf=os.path.join(mesh_folder, "roi", "tmp", "roi.stl"),
+                                           fn_surf_refined=os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl"),
+                                           center=[0, 0, 0],
+                                           radius=np.inf,
+                                           verbose=True,
+                                           repair=False)
 
                 roi.append(trimesh.load(os.path.join(mesh_folder, "roi", "tmp", "roi_refined.stl")))
 
@@ -558,32 +569,32 @@ class RegionOfInterestSurface:
 
     Attributes
     ----------
-    node_coord_up : nparray [N_points x 3]
+    node_coord_up : np.ndarray [N_points x 3]
         Coordinates (x,y,z) of upper surface nodes
-    node_coord_mid : nparray [N_points x 3]
+    node_coord_mid : np.ndarray [N_points x 3]
         Coordinates (x,y,z) of middle surface nodes
-    node_coord_low : nparray [N_points x 3]
+    node_coord_low : np.ndarray [N_points x 3]
         Coordinates (x,y,z) of lower surface nodes
-    node_number_list : nparray [N_points x 3]
+    node_number_list : np.ndarray [N_points x 3]
         Connectivity matrix of triangles
     delta : float
         Distance parameter between WM and GM (0 -> WM, 1 -> GM)
-    tet_idx_tri_center_up : nparray [N_points]
+    tet_idx_tri_center_up : np.ndarray [N_points]
         Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
         upper surface are lying in
-    tet_idx_tri_center_mid : nparray [N_points]
+    tet_idx_tri_center_mid : np.ndarray [N_points]
         Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
         middle surface are lying in
-    tet_idx_tri_center_low : nparray [N_points]
+    tet_idx_tri_center_low : np.ndarray [N_points]
         Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
         lower surface are lying in
-    tet_idx_node_coord_mid : nparray [N_tri]
+    tet_idx_node_coord_mid : np.ndarray [N_tri]
         Tetrahedra indices of TetrahedraLinear object instance where the nodes of the middle surface are lying in
-    tri_center_coord_up : nparray [N_tri x 3]
+    tri_center_coord_up : np.ndarray [N_tri x 3]
         Coordinates of roi triangle center of upper surface
-    tri_center_coord_mid : nparray [N_tri x 3]
+    tri_center_coord_mid : np.ndarray [N_tri x 3]
         Coordinates of roi triangle center of middle surface
-    tri_center_coord_low : nparray [N_tri x 3]
+    tri_center_coord_low : np.ndarray [N_tri x 3]
         Coordinates of roi triangle center of lower surface
     fn_mask : string
         Filename for surface mask in subject space. .mgh file or freesurfer surface file.
@@ -685,7 +696,7 @@ class RegionOfInterestSurface:
         """
         # find midlayer node that is nearest to the target
         target_node_coord_mid = np.where(np.linalg.norm(self.node_coord_mid - target, axis=1) == np.min(
-            np.linalg.norm(self.node_coord_mid - target, axis=1)))[0][0]
+                np.linalg.norm(self.node_coord_mid - target, axis=1)))[0][0]
 
         # get coordinates of that node
         target_proj = self.node_coord_mid[target_node_coord_mid]
@@ -697,7 +708,7 @@ class RegionOfInterestSurface:
 
     def make_GM_WM_surface(self, gm_surf_fname=None, wm_surf_fname=None, midlayer_surf_fname=None, mesh_folder=None,
                            delta=0.5,
-                           X_ROI=None, Y_ROI=None, Z_ROI=None,
+                           x_roi=None, y_roi=None, z_roi=None,
                            layer=1,
                            fn_mask=None, refine=False):
         """
@@ -724,13 +735,13 @@ class RegionOfInterestSurface:
             Distance parameter where surface is generated 0...1 (default: 0.5)
             - 0 -> WM surface
             - 1 -> GM surface
-        X_ROI : list of float
+        x_roi : list of float
             Region of interest [Xmin, Xmax], whole X range if empty [0,0] or None
             (left - right)
-        Y_ROI : list of float
+        y_roi : list of float
             Region of interest [Ymin, Ymax], whole Y range if empty [0,0] or None
             (anterior - posterior)
-        Z_ROI : list of float
+        z_roi : list of float
             Region of interest [Zmin, Zmax], whole Z range if empty [0,0] or None
             (inferior - superior)
         layer : int
@@ -744,23 +755,23 @@ class RegionOfInterestSurface:
 
         Returns
         -------
-        node_coord_up : nparray of float [N_roi_points x 3]
+        node_coord_up : np.ndarray of float [N_roi_points x 3]
             Node coordinates (x, y, z) of upper epsilon layer of ROI surface
-        node_coord_mid : nparray of float [N_roi_points x 3]
+        node_coord_mid : np.ndarray of float [N_roi_points x 3]
             Node coordinates (x, y, z) of ROI surface
-        node_coord_low : nparray of float [N_roi_points x 3]
+        node_coord_low : np.ndarray of float [N_roi_points x 3]
             Node coordinates (x, y, z) of lower epsilon layer of ROI surface
-        node_number_list : nparray of int [N_roi_tri x 3]
+        node_number_list : np.ndarray of int [N_roi_tri x 3]
             Connectivity matrix of intermediate surface layer triangles
         delta : float
             Distance parameter where surface is generated 0...1 (default: 0.5)
             - 0 -> WM surface
             - 1 -> GM surface
-        tri_center_coord_up : nparray of float [N_roi_tri x 3]
+        tri_center_coord_up : np.ndarray of float [N_roi_tri x 3]
             Coordinates (x, y, z) of triangle center of upper epsilon layer of ROI surface
-        tri_center_coord_mid : nparray of float [N_roi_tri x 3]
+        tri_center_coord_mid : np.ndarray of float [N_roi_tri x 3]
             Coordinates (x, y, z) of triangle center of ROI surface
-        tri_center_coord_low : nparray of float [N_roi_tri x 3]
+        tri_center_coord_low : np.ndarray of float [N_roi_tri x 3]
             Coordinates (x, y, z) of triangle center of lower epsilon layer of ROI surface
         fn_mask : str
             Filename for freesurfer mask. If given, this is used instead of *_ROIs
@@ -787,9 +798,9 @@ class RegionOfInterestSurface:
         self.mesh_folder = mesh_folder
         self.fn_mask = fn_mask
         self.delta = delta
-        self.X_ROI = X_ROI
-        self.Y_ROI = Y_ROI
-        self.Z_ROI = Z_ROI
+        self.X_ROI = x_roi
+        self.Y_ROI = y_roi
+        self.Z_ROI = z_roi
         self.refine = refine
 
         if type(gm_surf_fname) is not list:
@@ -894,23 +905,23 @@ class RegionOfInterestSurface:
         x_roi_wb, y_roi_wb, z_roi_wb = (False,) * 3
         if fn_mask is None:
             # crop to region of interest
-            if X_ROI is None or X_ROI == [0, 0]:
-                X_ROI = [-np.inf, np.inf]
+            if x_roi is None or x_roi == [0, 0]:
+                x_roi = [-np.inf, np.inf]
                 x_roi_wb = True
-            if Y_ROI is None or Y_ROI == [0, 0]:
-                Y_ROI = [-np.inf, np.inf]
+            if y_roi is None or y_roi == [0, 0]:
+                y_roi = [-np.inf, np.inf]
                 y_roi_wb = True
-            if Z_ROI is None or Z_ROI == [0, 0]:
-                Z_ROI = [-np.inf, np.inf]
+            if z_roi is None or z_roi == [0, 0]:
+                z_roi = [-np.inf, np.inf]
                 z_roi_wb = True
 
-            roi_mask_bool = (surface_points_middle[:, 0] > min(X_ROI)) & (surface_points_middle[:, 0] < max(X_ROI)) & \
-                            (surface_points_middle[:, 1] > min(Y_ROI)) & (surface_points_middle[:, 1] < max(Y_ROI)) & \
-                            (surface_points_middle[:, 2] > min(Z_ROI)) & (surface_points_middle[:, 2] < max(Z_ROI))
+            roi_mask_bool = (surface_points_middle[:, 0] > min(x_roi)) & (surface_points_middle[:, 0] < max(x_roi)) & \
+                            (surface_points_middle[:, 1] > min(y_roi)) & (surface_points_middle[:, 1] < max(y_roi)) & \
+                            (surface_points_middle[:, 2] > min(z_roi)) & (surface_points_middle[:, 2] < max(z_roi))
             roi_mask_idx = np.where(roi_mask_bool)
 
         else:
-            if X_ROI is not None or Y_ROI is not None or Z_ROI is not None:
+            if x_roi is not None or y_roi is not None or z_roi is not None:
                 raise ValueError(f"Either provide X_ROI, Y_ROI, Z_ROI or fn_mask, not both.")
 
             # read mask from freesurfer mask file
@@ -950,12 +961,12 @@ class RegionOfInterestSurface:
                                    faces=con_cropped_reform)
             mesh.export(os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"))
 
-            main.refine_surface(fn_surf=os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"),
-                                fn_surf_refined=os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl"),
-                                center=[0, 0, 0],
-                                radius=np.inf,
-                                verbose=True,
-                                repair=False)
+            pynibs.mesh.refine_surface(fn_surf=os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"),
+                                       fn_surf_refined=os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl"),
+                                       center=[0, 0, 0],
+                                       radius=np.inf,
+                                       verbose=True,
+                                       repair=False)
 
             roi = trimesh.load(os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl"))
             con_cropped_reform = roi.faces
@@ -969,12 +980,13 @@ class RegionOfInterestSurface:
                                            faces=con_cropped_reform)
                     mesh.export(os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"))
 
-                    main.refine_surface(fn_surf=os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"),
-                                        fn_surf_refined=os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl"),
-                                        center=[0, 0, 0],
-                                        radius=np.inf,
-                                        verbose=True,
-                                        repair=False)
+                    pynibs.mesh.refine_surface(fn_surf=os.path.join(self.mesh_folder, "roi", "tmp", "roi.stl"),
+                                               fn_surf_refined=os.path.join(self.mesh_folder, "roi", "tmp",
+                                                                            "roi_refined.stl"),
+                                               center=[0, 0, 0],
+                                               radius=np.inf,
+                                               verbose=True,
+                                               repair=False)
 
                     roi.append(trimesh.load(os.path.join(self.mesh_folder, "roi", "tmp", "roi_refined.stl")))
 
@@ -1004,21 +1016,21 @@ class RegionOfInterestSurface:
 
         Parameters
         ----------
-        msh : object
+        msh : pynibs.TetrahedraLinear
             Object instance of TetrahedraLinear class
 
         Returns
         -------
-        RegionOfInterestSurface.tet_idx_tri_center_up : nparray [N_points]
+        RegionOfInterestSurface.tet_idx_tri_center_up : np.ndarray [N_points]
             Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
             upper surface are lying in
-        RegionOfInterestSurface.tet_idx_tri_center_mid : nparray [N_points]
+        RegionOfInterestSurface.tet_idx_tri_center_mid : np.ndarray [N_points]
             Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
             middle surface are lying in
-        RegionOfInterestSurface.tet_idx_tri_center_low : nparray [N_points]
+        RegionOfInterestSurface.tet_idx_tri_center_low : np.ndarray [N_points]
             Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
             lower surface are lying in
-        RegionOfInterestSurface.tet_idx_node_coord_mid : nparray [N_tri]
+        RegionOfInterestSurface.tet_idx_node_coord_mid : np.ndarray [N_tri]
             Tetrahedra indices of TetrahedraLinear object instance where the nodes of the middle surface are lying in
      """
         # determine tetrahedra indices of triangle center points of upper, middle and lower surface
@@ -1071,22 +1083,14 @@ class RegionOfInterestSurface:
 
         # sample sphere equally
         r = np.linalg.norm(sphere_coords[0, :])
-        A0 = 4 * np.pi * r ** 2
+        a0 = 4 * np.pi * r ** 2
 
-        sphere_points_sampled = sample_sphere(n_points=int(np.ceil(A0 / dist ** 2) // 2 * 2 + 1),
-                                              r=np.linalg.norm(sphere_coords[0, :]))
-
-        # from mpl_toolkits.mplot3d import Axes3D  # noqa: F401 unused import
-        # import matplotlib.pyplot as plt
-
-        # fig = plt.figure()
-        # ax = fig.add_subplot(111, projection='3d')
-        # ax.scatter(sphere_points_sampled[:, 0], sphere_points_sampled[:, 1], sphere_points_sampled[:, 2])
-        # triangular_mesh(sphere_coords[:, 0], sphere_coords[:, 1], sphere_coords[:, 2], sphere_faces, tube_radius=None, line_width=0.01, scale_factor=0.0001)
+        sphere_points_sampled = pynibs.sample_sphere(n_points=int(np.ceil(a0 / dist ** 2) // 2 * 2 + 1),
+                                                     r=np.linalg.norm(sphere_coords[0, :]))
 
         # read mask from freesurfer mask file
         mask = nib.freesurfer.mghformat.MGHImage.from_filename(
-            os.path.join(self.mesh_folder, self.fn_mask)).dataobj[:].flatten()
+                os.path.join(self.mesh_folder, self.fn_mask)).dataobj[:].flatten()
         roi_mask_idx = np.where(mask > 0.5)
 
         con_row_idx = [i for i in range(sphere_faces.shape[0]) if
@@ -1116,18 +1120,18 @@ class RegionOfInterestVolume:
 
     Attributes
     ----------
-    node_coord : nparray [N_points x 3]
+    node_coord : np.ndarray [N_points x 3]
         Coordinates (x,y,z) of ROI tetrahedra nodes
-    tet_node_number_list : nparray [N_tet_roi x 3]
+    tet_node_number_list : np.ndarray [N_tet_roi x 3]
         Connectivity matrix of ROI tetrahedra
-    tri_node_number_list : nparray [N_tri_roi x 3]
+    tri_node_number_list : np.ndarray [N_tri_roi x 3]
         Connectivity matrix of ROI tetrahedra
-    tet_idx_node_coord : nparray [N_points]
+    tet_idx_node_coord : np.ndarray [N_points]
         Tetrahedra indices of TetrahedraLinear object instance where the ROI nodes are lying in
-    tet_idx_tetrahedra_center : nparray [N_tet_roi]
+    tet_idx_tetrahedra_center : np.ndarray [N_tet_roi]
         Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI tetrahedra are
         lying in
-    tet_idx_triangle_center : nparray [N_tri_roi]
+    tet_idx_triangle_center : np.ndarray [N_tri_roi]
         Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI triangle are
         lying in. If the ROI is directly generated from the msh instance using "make_roi_volume_from_msh", these
         indices are the triangle indices of the head mesh since the ROI mesh and the head mesh are overlapping. If
@@ -1145,40 +1149,40 @@ class RegionOfInterestVolume:
         self.tet_idx_tetrahedra_center = []
         self.tet_idx_triangle_center = []
 
-    def make_roi_volume_from_msh(self, msh, volume_type='box', X_ROI=None, Y_ROI=None, Z_ROI=None):
+    def make_roi_volume_from_msh(self, msh, volume_type='box', x_roi=None, y_roi=None, z_roi=None):
         """
         Generate region of interest (volume) and extract nodes, triangles and tetrahedra from msh instance.
 
         Parameters
         ----------
-        msh: object
+        msh: pynibs.TetrahedraLinear
             Mesh object instance of type TetrahedraLinear (see main.py)
         volume_type: str
             Type of ROI ('box' or 'sphere')
-        X_ROI: list of float
+        x_roi: list of float
             - type = 'box': [Xmin, Xmax] (in mm), whole X range if empty [0,0] or None (left - right)
             - type = 'sphere': origin [x,y,z]
-        Y_ROI: list of float
+        y_roi: list of float
             - type = 'box': [Ymin, Ymax] (in mm), whole Y range if empty [0,0] or None (anterior - posterior)
             - type = 'sphere': radius (in mm)
-        Z_ROI: list of float
+        z_roi: list of float
             - type = 'box': [Zmin, Zmax] (in mm), whole Z range if empty [0,0] or None (inferior - superior)
             - type = 'sphere': None
 
         Returns
         -------
-        RegionOfInterestVolume.node_coord : nparray [N_points x 3]
+        RegionOfInterestVolume.node_coord : np.ndarray [N_points x 3]
             Coordinates (x,y,z) of ROI tetrahedra nodes
-        RegionOfInterestVolume.tet_node_number_list : nparray [N_tet_roi x 3]
+        RegionOfInterestVolume.tet_node_number_list : np.ndarray [N_tet_roi x 3]
             Connectivity matrix of ROI tetrahedra
-        RegionOfInterestVolume.tri_node_number_list : nparray [N_tri_roi x 3]
+        RegionOfInterestVolume.tri_node_number_list : np.ndarray [N_tri_roi x 3]
             Connectivity matrix of ROI tetrahedra
-        RegionOfInterestVolume.tet_idx_node_coord : nparray [N_points]
+        RegionOfInterestVolume.tet_idx_node_coord : np.ndarray [N_points]
             Tetrahedra indices of TetrahedraLinear object instance where the ROI nodes are lying in
-        RegionOfInterestVolume.tet_idx_tetrahedra_center : nparray [N_tet_roi]
+        RegionOfInterestVolume.tet_idx_tetrahedra_center : np.ndarray [N_tet_roi]
             Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI tetrahedra are
             lying in
-        RegionOfInterestVolume.tet_idx_triangle_center : nparray [N_tri_roi]
+        RegionOfInterestVolume.tet_idx_triangle_center : np.ndarray [N_tri_roi]
             Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI triangle are
             lying in. If the ROI is directly generated from the msh instance using "make_roi_volume_from_msh", these
             indices are the triangle indices of the head mesh since the ROI mesh and the head mesh are overlapping. If
@@ -1187,13 +1191,13 @@ class RegionOfInterestVolume:
         """
 
         if volume_type == 'box':
-            roi_mask_bool = (msh.points[:, 0] > min(X_ROI)) & (msh.points[:, 0] < max(X_ROI)) & \
-                            (msh.points[:, 1] > min(Y_ROI)) & (msh.points[:, 1] < max(Y_ROI)) & \
-                            (msh.points[:, 2] > min(Z_ROI)) & (msh.points[:, 2] < max(Z_ROI))
+            roi_mask_bool = (msh.points[:, 0] > min(x_roi)) & (msh.points[:, 0] < max(x_roi)) & \
+                            (msh.points[:, 1] > min(y_roi)) & (msh.points[:, 1] < max(y_roi)) & \
+                            (msh.points[:, 2] > min(z_roi)) & (msh.points[:, 2] < max(z_roi))
             roi_mask_idx = np.where(roi_mask_bool)[0]
 
         elif volume_type == 'sphere':
-            roi_mask_bool = np.linalg.norm(msh.points - np.array(X_ROI), axis=1) <= Y_ROI
+            roi_mask_bool = np.linalg.norm(msh.points - np.array(x_roi), axis=1) <= y_roi
             roi_mask_idx = np.where(roi_mask_bool)[0]
 
         else:
@@ -1231,7 +1235,7 @@ class RegionOfInterestVolume:
     #     ----------
     #     msh: object instance
     #         Mesh object instance of type TetrahedraLinear (see main.py)
-    #     points: nparray [N_points_roi x 3]
+    #     points: np.ndarray [N_points_roi x 3]
     #         Coordinates of points in ROI x, y, z)
     #     """
     #     a=1
@@ -1407,36 +1411,37 @@ def get_sphere_in_nii(center, radius, nii=None, out_fn=None,
     roi_data[:] = val_out
 
     center = np.atleast_1d(center)
-    # radius = np.array(radius).astype(int)
-
-    if outside_radius != np.inf:
-        # searach space is whole nifti space
-        x_min, y_min, z_min = nib.affines.apply_affine(nii.affine, [0, 0, 0]).astype(int)
-        x_max, y_max, z_max = nib.affines.apply_affine(nii.affine, list(nii.shape[:3])).astype(int)
-    else:
-        # if outside_val should not be set we only need to search within center-radius
-        x_min, y_min, z_min = center - radius
-        x_max, y_max, z_max = center + radius
 
     # speed up the long loop below
     affine_inv = np.linalg.inv(nii.affine)
-    apply_affine = nib.affines.apply_affine
     norm = np.linalg.norm
 
+    center_vox = nib.affines.apply_affine(affine_inv, center)
+
+    if outside_radius != np.inf:
+        # search space is whole nifti space
+        x_min, y_min, z_min = 0, 0, 0
+        x_max, y_max, z_max = nii.shape[:3]
+
+    else:
+        # if outside_val should not be set we only need to search within center-radius
+        x_min, y_min, z_min = center_vox - radius
+        x_max, y_max, z_max = center_vox + radius
+
     # compute spherical roi around nifti coordinates
     # do this in original coordinates to have correct mm radius
+
+    pixdim = nii.get_header()['pixdim'][1:4]
     for xyz in tqdm.tqdm(product(range(x_min, x_max), range(y_min, y_max), range(z_min, z_max)),
-                         total=(x_max-x_min)*(y_max-y_min)*(z_max-z_min),
+                         total=x_max * y_max * z_max,
                          desc="Finding ROI elements."):
         # set to val_in if within radius
-        dist = norm(center - xyz)
+        dist = norm(np.multiply(pixdim, center_vox - xyz))
         if dist < radius:
-            vox_x, vox_y, vox_z = apply_affine(affine_inv, xyz).astype(int)
-            roi_data[vox_x, vox_y, vox_z] = val_in
+            roi_data[xyz] = val_in
         elif dist > outside_radius:
             # Set values outside of outside_radius to outside_val
-            vox_x, vox_y, vox_z = apply_affine(affine_inv, xyz).astype(int)
-            roi_data[vox_x, vox_y, vox_z] = outside_val
+            roi_data[xyz] = outside_val
 
     # check if there's something left in the ROI
     if np.sum(roi_data) == 0:
@@ -1493,12 +1498,12 @@ def create_refine_spherical_roi(center, radius, final_tissues_nii, out_fn, targe
     """
     # get spherical ROI, masked to all tissues
     roi_img = get_sphere_in_nii(
-        center=center,
-        radius=radius,
-        nii=final_tissues_nii,
-        val_in=target_size,
-        out_fn=out_spher_fn,
-        outside_val=outside_size, outside_radius=radius * outside_factor)
+            center=center,
+            radius=radius,
+            nii=final_tissues_nii,
+            val_in=target_size,
+            out_fn=out_spher_fn,
+            outside_val=outside_size, outside_radius=radius * outside_factor)
 
     # get tissue_types data
     if isinstance(final_tissues_nii, str):
