@@ -1,11 +1,10 @@
-import os
+import gc
 import re
 import warnings
 import copy
 import numpy as np
 import trimesh
-
-from .util import calc_tri_surface, calc_tet_volume
+import pynibs
 
 
 def calc_e_in_midlayer_roi(phi_dadt, roi, subject, phi_scaling=1., mesh_idx=0, mesh=None, roi_hem='lh', depth=.5,
@@ -29,15 +28,26 @@ def calc_e_in_midlayer_roi(phi_dadt, roi, subject, phi_scaling=1., mesh_idx=0, m
     -------
     (roi.n_tris,4) : np.vstack((e_mag, e_norm, e_tan, e_angle)).transpose()
     """
-    from .main import data_nodes2elements
-    import simnibs.msh.mesh_io as mesh_io
-    import simnibs.simulation.fem as fem
-    from simnibs.msh.transformations import get_surface_names_from_folder_structure
-    from simnibs.msh.mesh_io import Msh
-    from simnibs.msh.mesh_io import Nodes
-    from simnibs.msh.mesh_io import Elements
+    if depth != .5:
+        raise NotImplementedError(f"depth={depth} != 0.5 not implemented")
+    import simnibs
+    v_simnibs = int(simnibs.__version__[0])
+    if v_simnibs == 3:
+        # SimNIBS < 4 imports
+        import simnibs.msh.mesh_io as mesh_io
+        from simnibs.msh.mesh_io import Msh
+        from simnibs.msh.mesh_io import Nodes
+        from simnibs.msh.mesh_io import Elements
+        from simnibs.msh.transformations import get_surface_names_from_folder_structure
+    else:
+        # SimNIBS >= 4 imports
+        import simnibs.mesh_tools.mesh_io as mesh_io
+        from simnibs.mesh_tools.mesh_io import Msh
+        from simnibs.mesh_tools.mesh_io import Nodes
+        from simnibs.mesh_tools.mesh_io import Elements
 
-    mesh_folder = subject.mesh[mesh_idx]["mesh_folder"]
+    import simnibs.simulation.fem as fem
+    # mesh_folder = subject.mesh[mesh_idx]["mesh_folder"]
 
     # set default return quantities
     if qoi is None:
@@ -52,7 +62,8 @@ def calc_e_in_midlayer_roi(phi_dadt, roi, subject, phi_scaling=1., mesh_idx=0, m
 
     # simnibs calls this with empty data so find out about the results dimensions
     if isinstance(phi_dadt, tuple):
-        phi = phi_dadt[0] * phi_scaling
+        # SimNIBS 4 includes 'phi/v' last in the tuple of the to-be computed fields
+        phi = phi_dadt[-1] * phi_scaling
         # dadt = phi_dadt[1].elm_data2node_data().value
     else:
         return np.zeros((roi.n_tris, ret_arr_shape))
@@ -79,9 +90,9 @@ def calc_e_in_midlayer_roi(phi_dadt, roi, subject, phi_scaling=1., mesh_idx=0, m
     # write phi and dAdt in msh
     # dadt = mesh_io.NodeData(dadt, name='D', mesh=mesh_local)
     try:
-        dadt = phi_dadt[1].value
+        dadt = phi_dadt[0].value
     except AttributeError:
-        dadt = phi_dadt[1]
+        dadt = phi_dadt[0]
     dadt = mesh_io.ElementData(dadt, name='D', mesh=mesh_local)
     phi = mesh_io.NodeData(phi.flatten(), name='v', mesh=mesh_local)
 
@@ -89,22 +100,25 @@ def calc_e_in_midlayer_roi(phi_dadt, roi, subject, phi_scaling=1., mesh_idx=0, m
     mesh_local = mesh_local.crop_mesh(2)
 
     # get folder names and such
-    m2m_folder = os.path.join(mesh_folder, "m2m_" + subject.id)
-    m2m_folder = os.path.abspath(os.path.normpath(m2m_folder))
-    names, segtype = get_surface_names_from_folder_structure(m2m_folder)
-    # middle_surf = {}
+    # m2m_folder = os.path.join(mesh_folder, "m2m_" + subject.id)
+    # m2m_folder = os.path.abspath(os.path.normpath(m2m_folder))
 
-    # get midlayer
-    if segtype == 'mri2mesh':
-        # for hemi in ['lh', 'rh']:
-        wm_surface = mesh_io.read_freesurfer_surface(names[roi_hem + '_wm'])
-        gm_surface = mesh_io.read_freesurfer_surface(names[roi_hem + '_gm'])
-        middle_surf = mesh_io._middle_surface(wm_surface, gm_surface, depth)
-    elif segtype == 'headreco':
-        # for hemi in ['lh', 'rh']:
-        middle_surf = mesh_io.read_gifti_surface(names[roi_hem + '_midgm'])
-    else:
-        raise NotImplementedError(f"segtype {segtype} unknown.")
+    # if v_simnibs == 3:
+    #     names, segtype = get_surface_names_from_folder_structure(m2m_folder)
+    #     # middle_surf = {}
+    #
+    #     # get midlayer
+    #     if segtype == 'mri2mesh':
+    #         # for hemi in ['lh', 'rh']:
+    #         wm_surface = mesh_io.read_freesurfer_surface(names[roi_hem + '_wm'])
+    #         gm_surface = mesh_io.read_freesurfer_surface(names[roi_hem + '_gm'])
+    #         # middle_surf = mesh_io._middle_surface(wm_surface, gm_surface, depth)
+    #     elif segtype == 'headreco':
+    #         pass
+    #         # for hemi in ['lh', 'rh']:
+    #         # middle_surf = mesh_io.read_gifti_surface(names[roi_hem + '_midgm'])
+    #     else:
+    #         raise NotImplementedError(f"segtype {segtype} unknown.")
 
     # initialize ROI surface
     surface = Msh(nodes=Nodes(node_coord=roi.node_coord_mid),
@@ -125,13 +139,16 @@ def calc_e_in_midlayer_roi(phi_dadt, roi, subject, phi_scaling=1., mesh_idx=0, m
             qois_dict[q_name] = interpolated.value
 
         # transform point data to element data
-        qois_dict[q_name] = data_nodes2elements(data=qois_dict[q_name], con=roi.node_number_list)
+        qois_dict[q_name] = pynibs.data_nodes2elements(data=qois_dict[q_name], con=roi.node_number_list)
         # crop results data to ROI
         qois_dict[q_name] = qois_dict[q_name]
         if qois_dict[q_name].ndim == 1:
             qois_dict[q_name] = qois_dict[q_name][:, np.newaxis]
 
     res = tuple(qois_dict[q_name] for q_name in qoi)
+    del qois_dict, surface, interpolated, mesh_local, phi, mesh, simnibs, data
+    gc.collect()
+
     return np.hstack(res)
 
 
@@ -325,30 +342,27 @@ def read_coil_geo(fn_coil_geo):
     dipole_mag : ndarray of float [n_dip x 1]
         Dipole magnitude
     """
-    regex = r"\((.*?)\)\{(.*?)\}"
-    file = open(fn_coil_geo)
-    read = True
-    dipole_pos = []
-    dipole_mag = []
+    regex = r"SP\((.*?)\)\{(.*?)\}"
+    with open(fn_coil_geo, 'r') as file:
+        dipole_pos = []
+        dipole_mag = []
+        while file:
+            te = file.readline()
 
-    while read:
-        te = file.readline()
+            if te == "":
+                break
 
-        if te == "":
-            break
+            try:
+                pos, mag = re.findall(regex, te)[0]
+                pos = [float(x) for x in pos.split(',')]
+                mag = float(mag)
+                dipole_pos.append(pos)
+                dipole_mag.append(mag)
+            except IndexError:
+                pass
 
-        matches = re.finditer(regex, te, re.MULTILINE)
-
-        for matchNum, match in enumerate(matches, start=1):
-            for groupNum in range(0, len(match.groups())):
-                groupNum = groupNum + 1
-                if groupNum == 1:
-                    dipole_pos.append([float(x) for x in match.group(groupNum).split(',')])
-                if groupNum == 2:
-                    dipole_mag.append(float(match.group(groupNum)))
-
-    dipole_pos = np.vstack(dipole_pos)
-    dipole_mag = np.array(dipole_mag)[:, np.newaxis]
+        dipole_pos = np.vstack(dipole_pos)
+        dipole_mag = np.array(dipole_mag)[:, np.newaxis]
 
     return dipole_pos, dipole_mag
 
@@ -384,20 +398,20 @@ def check_mesh(mesh, verbose=False):
         mesh = read_msh(mesh)
     tris = mesh.elm.node_number_list[mesh.elm.triangles - 1][:, :3]
     points_tri = mesh.nodes[tris]
-    tri_area = calc_tri_surface(points_tri)
-    zero_tris = np.argwhere(np.isclose(tri_area, 0, atol=3e-17))
+    tri_area = pynibs.calc_tri_surface(points_tri)
+    zero_tris = np.argwhere(np.isclose(tri_area, 0, atol=1e-13))
     if verbose:
         print(f"{len(zero_tris)} zero surface triangles found.")
 
     tets = mesh.elm.node_number_list[mesh.elm.tetrahedra - 1]
     points_tets = mesh.nodes[tets]
-    tets_volume = calc_tet_volume(points_tets)
-    zero_tets = np.argwhere(np.isclose(tets_volume, 0, atol=3e-17))
+    tets_volume = pynibs.calc_tet_volume(points_tets)
+    zero_tets = np.argwhere(np.isclose(tets_volume, 0, atol=1e-13))
     if verbose:
         print(f"{len(zero_tets)} zero volume tetrahedra found.")
 
     tet_idx = mesh.elm.node_number_list[mesh.elm.tetrahedra - 1]
-    vol = calc_tet_volume(mesh.nodes.node_coord[tet_idx - 1], abs=False)
+    vol = pynibs.calc_tet_volume(mesh.nodes.node_coord[tet_idx - 1], abs=False)
     neg_idx = np.argwhere(vol > 0)
     if verbose:
         print(f"{len(neg_idx)} negative tets found.")
