@@ -1,12 +1,10 @@
-import json
 import os
 import warnings
 
 import h5py
 import pickle
 import numpy as np
-from pynibs.util import bash_call
-from .hdf5_io import write_dict_to_hdf5, read_dict_from_hdf5
+import pynibs
 
 
 class Subject:
@@ -230,7 +228,7 @@ class Subject:
 
         Parameters
         ----------
-        exp_dict : dictionary or list of dictionaries
+        exp_dict : dict of dict or list of dict
             Dictionary containing information about the experiment
 
         Notes
@@ -295,175 +293,6 @@ def fill_from_dict(self, d):
     return self
 
 
-class Mesh:
-    """"
-    Mesh class to initialize default attributes.
-    """
-
-    def __init__(self, mesh_name, subject_id, subject_folder):
-        self.subject_id = subject_id
-        self.subject_folder = subject_folder
-        self.name = mesh_name
-        self.info = None
-        self.approach = None  # 'mri2mesh', 'headreco', 'charm'
-        self.mri_idx = None
-
-        # default parameters
-        self.mesh_folder = os.path.join(subject_folder, 'mesh', mesh_name)
-        self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{subject_id}.msh")
-        self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{subject_id}.hdf5")
-        self.fn_tensor_vn = f"d2c_{subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
-        self.fn_mri_conform = f"{subject_id}_T1fs_conform.nii.gz"
-        self.fn_lh_midlayer = f"fs_{subject_id}{os.sep}surf{os.sep}lh.central"
-        self.fn_rh_midlayer = f"fs_{subject_id}{os.sep}surf{os.sep}rh.central"
-        self.vertex_density = 1.0 # headreco
-        self.numvertices = 100000 # mri2mesh
-
-        # refinement parameters
-        self.center = None
-        self.radius = None
-        self.element_size = None
-        self.refine_domains = None
-        self.smooth_domains = None
-
-        self.fn_lh_wm = None
-        self.fn_rh_wm = None
-        self.fn_lh_gm = None
-        self.fn_rh_gm = None
-        self.fn_lh_gm_curv = None
-        self.fn_rh_gm_curv = None
-
-        # charm meshes
-        self.smooth_skin = None
-        self.refinement_roi = None
-        self.refinemement_element_size = None
-
-    def fill_defaults(self, approach):
-        """
-        Initializes attributes for a headreco mesh.
-
-        Parameters:
-        -----------
-        approach: str
-            'headreco'
-            'mri2mesh'
-            'charm'
-        """
-        self.approach = approach
-        if approach == 'headreco':
-            self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{self.subject_id}.msh")
-            self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{self.subject_id}.hdf5")
-            self.fn_tensor_vn = f"d2c_{self.subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
-            self.fn_mri_conform = f"{self.subject_id}_T1fs_conform.nii.gz"
-            self.fn_lh_midlayer = f"fs_{self.subject_id}{os.sep}surf{os.sep}lh.central"
-            self.fn_rh_midlayer = f"fs_{self.subject_id}{os.sep}surf{os.sep}rh.central"
-
-        elif approach == 'mri2mesh':
-            self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{self.subject_id}.msh")
-            self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{self.subject_id}.hdf5")
-            self.fn_tensor_vn = f"d2c_{self.subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
-            self.fn_mri_conform = f"{self.subject_id}_T1fs_conform.nii.gz"
-            self.fn_lh_gm_curv = f"fs_{self.subject_id}{os.sep}surf{os.sep}lh.curv.pial"
-            self.fn_rh_gm_curv = f"fs_{self.subject_id}{os.sep}surf{os.sep}rh.curv.pial"
-
-        elif approach == 'charm':
-            self.fn_mesh_msh = os.path.join(self.mesh_folder, f"{self.subject_id}.msh")
-            self.fn_mesh_hdf5 = os.path.join(self.mesh_folder, f"{self.subject_id}.hdf5")
-            self.fn_tensor_vn = f"d2c_{self.subject_id}{os.sep}dti_results_T1space{os.sep}DTI_conf_tensor.nii.gz"
-            self.fn_mri_conform = f"{self.subject_id}_T1.nii.gz"
-            self.fn_lh_midlayer = f"m2m_{self.subject_id}{os.sep}surfaces{os.sep}lh.central.gii"
-            self.fn_rh_midlayer = f"m2m_{self.subject_id}{os.sep}surfaces{os.sep}rh.central.gii"
-
-        else:
-            raise NotImplementedError(f"Approach {approach} not implemented.")
-
-    def write_to_hdf5(self, fn_hdf5, check_file_exist=False, verbose=False):
-        """
-        Write this mesh' attributes to .hdf5 file.
-
-        Parameters:
-        -----------
-        fn_hdf5 : str
-        check_file_exist : bool
-            Check if provided filenames exist, warn if not.
-        verbose : bool
-            Print self information
-        """
-
-        write_dict_to_hdf5(fn_hdf5=fn_hdf5, data=self.__dict__, folder=f"mesh/{self.name}",
-                           check_file_exist=check_file_exist)
-        if verbose:
-            self.print()
-
-    def print(self):
-        """
-        Print self information.
-        """
-        n_left, n_right = int(32 - np.floor((len(self.name) + 10) / 2)), int(32 - np.ceil((len(self.name) + 10) / 2))
-        n_left, n_right = np.max(n_left, 0), np.max(n_right, 0)
-        print("    " + "=" * n_left + f"  Mesh {self.name}:  " + "=" * n_right)
-        print("\t" + json.dumps(self.__dict__, sort_keys=False, indent="\t", ))
-        print("    " + "=" * 64 + "\n")
-
-
-class ROI:
-    """"
-    Region of interest class to initialize default attributes.
-
-    """
-
-    def __init__(self, subject_id, roi_name, mesh_name):
-        self.subject_id = subject_id
-        self.name = roi_name
-        self.mesh_name = mesh_name
-        self.type = None  # 'surface' or 'volume'
-        self.info = None
-        self.template = None  # None, 'MNI', 'fsaverage', 'subject'
-        self.gm_surf_fname = None
-        self.wm_surf_fname = None
-        self.midlayer_surf_fname = None
-        self.delta = 0.5
-        self.refine = False
-        self.X_ROI = None
-        self.Y_ROI = None
-        self.Z_ROI = None
-        self.center = None
-        self.radius = None
-        self.layer = 3
-        self.fn_mask = None
-        self.fn_mask_avg = None
-        self.hemisphere = None
-        self.midlayer_surf_fname = None
-
-    def write_to_hdf5(self, fn_hdf5, check_file_exist=False, verbose=False):
-        """
-        Write this mesh' attributes to .hdf5 file.
-
-        Parameters:
-        -----------
-        fn_hdf5 : str
-        check_file_exist : bool
-            Check if provided filenames exist, warn if not.
-        verbose : bool
-            Print self information
-        """
-
-        write_dict_to_hdf5(fn_hdf5=fn_hdf5, data=self.__dict__, folder=f"roi/{self.mesh_name}/{self.name}",
-                           check_file_exist=check_file_exist)
-        if verbose:
-            self.print()
-
-    def print(self):
-        """
-        Print self information.
-        """
-        n_left, n_right = int(32 - np.floor((len(self.name) + 10) / 2)), int(32 - np.ceil((len(self.name) + 10) / 2))
-        n_left, n_right = np.max(n_left, 0), np.max(n_right, 0)
-        print("    " + "=" * n_left + f"  ROI {self.name}:  " + "=" * n_right)
-        print("\t" + json.dumps(self.__dict__, sort_keys=False, indent="\t", ))
-        print("    " + "=" * 64 + "\n")
-
-
 def save_subject(subject_id, subject_folder, fname, mri_dict=None, mesh_dict=None, roi_dict=None,
                  exp_dict=None, ps_dict=None, **kwargs):
     """
@@ -512,26 +341,27 @@ def save_subject(subject_id, subject_folder, fname, mri_dict=None, mesh_dict=Non
                           **kwargs)
 
     elif filetype == ".pkl":
-
-        # create and initialize subject
-        subject = Subject(subject_id=subject_id)
-
-        # add mri information
-        subject.add_mri_info(mri_dict=mri_dict)
-
-        # add mesh information
-        subject.add_mesh_info(mesh_dict=mesh_dict)
-
-        # add roi info to mesh
-        subject.add_roi_info(roi_dict=roi_dict)
-
-        # add experiment info
-        subject.add_experiment_info(exp_dict=exp_dict)
-
-        # add plotsettings
-        subject.add_plotsettings(ps_dict=ps_dict)
-
-        save_subject_pkl(sobj=subject, fname=fname)
+        raise NotImplementedError
+        #
+        # # create and initialize subject
+        # subject = Subject(subject_id=subject_id)
+        #
+        # # add mri information
+        # subject.add_mri_info(mri_dict=mri_dict)
+        #
+        # # add mesh information
+        # subject.add_mesh_info(mesh_dict=mesh_dict)
+        #
+        # # add roi info to mesh
+        # subject.add_roi_info(roi_dict=roi_dict)
+        #
+        # # add experiment info
+        # subject.add_experiment_info(exp_dict=exp_dict)
+        #
+        # # add plotsettings
+        # subject.add_plotsettings(ps_dict=ps_dict)
+        #
+        # save_subject_pkl(sobj=subject, fname=fname)
 
 
 def save_subject_pkl(sobj, fname):
@@ -610,14 +440,14 @@ def save_subject_hdf5(subject_id, subject_folder, fname, mri_dict=None, mesh_dic
             mri_dict = {i: mri_dict[i] for i in range(len(mri_dict))}
 
         for i in mri_dict.keys():
-            write_dict_to_hdf5(fn_hdf5=fname, data=mri_dict[i], folder=f"mri/{i}", check_file_exist=True)
+            pynibs.write_dict_to_hdf5(fn_hdf5=fname, data=mri_dict[i], folder=f"mri/{i}", check_file_exist=True)
 
     if mesh_dict is not None:
         if isinstance(mesh_dict, list):
             mesh_dict = {i: mesh_dict[i] for i in range(len(mesh_dict))}
 
         for mesh_name, mesh_dict in mesh_dict.items():
-            mesh = Mesh(mesh_name=mesh_name, subject_id=subject_id, subject_folder=subject_folder)
+            mesh = pynibs.Mesh(mesh_name=mesh_name, subject_id=subject_id, subject_folder=subject_folder)
             mesh.fill_defaults(mesh_dict['approach'])
             mesh = fill_from_dict(mesh, mesh_dict)
             mesh.write_to_hdf5(fn_hdf5=fname, check_file_exist=check_file_exist, verbose=verbose)
@@ -625,7 +455,7 @@ def save_subject_hdf5(subject_id, subject_folder, fname, mri_dict=None, mesh_dic
     if roi_dict is not None:
         for mesh_name in roi_dict.keys():
             for roi_name, roi_dict_i in roi_dict[mesh_name].items():
-                roi = ROI(subject_id=subject_id, roi_name=roi_name, mesh_name=mesh_name)
+                roi = pynibs.ROI(subject_id=subject_id, roi_name=roi_name, mesh_name=mesh_name)
                 roi = fill_from_dict(roi, roi_dict_i)
                 roi.write_to_hdf5(fn_hdf5=fname, check_file_exist=check_file_exist, verbose=verbose)
 
@@ -633,11 +463,13 @@ def save_subject_hdf5(subject_id, subject_folder, fname, mri_dict=None, mesh_dic
         if isinstance(exp_dict, list):
             exp_dict = {i: exp_dict[i] for i in range(len(exp_dict))}
         for i in exp_dict.keys():
-            write_dict_to_hdf5(fn_hdf5=fname, data=exp_dict[i], folder=f"exp/{i}", check_file_exist=check_file_exist)
+            pynibs.write_dict_to_hdf5(fn_hdf5=fname, data=exp_dict[i], folder=f"exp/{i}",
+                                      check_file_exist=check_file_exist)
 
     if ps_dict is not None:
         for i in range(len(ps_dict)):
-            write_dict_to_hdf5(fn_hdf5=fname, data=ps_dict[i], folder=f"ps/{i}", check_file_exist=check_file_exist)
+            pynibs.write_dict_to_hdf5(fn_hdf5=fname, data=ps_dict[i], folder=f"ps/{i}",
+                                      check_file_exist=check_file_exist)
 
     with h5py.File(fname, 'a') as f:
         for key, value in kwargs.items():
@@ -676,7 +508,7 @@ def load_subject_hdf5(fname):
             mri = []
 
             for key in mri_keys:
-                mri.append(read_dict_from_hdf5(fn_hdf5=fname, folder=f"mri/{key}"))
+                mri.append(pynibs.read_dict_from_hdf5(fn_hdf5=fname, folder=f"mri/{key}"))
 
         except KeyError:
             mri = None
@@ -689,7 +521,7 @@ def load_subject_hdf5(fname):
             mesh = {}
 
             for key in mesh_keys:
-                mesh[key] = read_dict_from_hdf5(fn_hdf5=fname, folder=f"mesh/{key}")
+                mesh[key] = pynibs.read_dict_from_hdf5(fn_hdf5=fname, folder=f"mesh/{key}")
 
         except KeyError:
             mesh = None
@@ -707,7 +539,8 @@ def load_subject_hdf5(fname):
                     roi[mesh_idx] = dict()
 
                     for roi_idx in roi_idx_keys:
-                        roi[mesh_idx][roi_idx] = read_dict_from_hdf5(fn_hdf5=fname, folder=f"roi/{mesh_idx}/{roi_idx}")
+                        roi[mesh_idx][roi_idx] = pynibs.read_dict_from_hdf5(fn_hdf5=fname,
+                                                                            folder=f"roi/{mesh_idx}/{roi_idx}")
 
                 except KeyError:
                     roi[mesh_idx] = None
@@ -723,7 +556,7 @@ def load_subject_hdf5(fname):
             exp = {}
 
             for key in exp_keys:
-                exp[key] = read_dict_from_hdf5(fn_hdf5=fname, folder=f"exp/{key}")
+                exp[key] = pynibs.read_dict_from_hdf5(fn_hdf5=fname, folder=f"exp/{key}")
 
         except KeyError:
             exp = None
@@ -736,7 +569,7 @@ def load_subject_hdf5(fname):
             ps = []
 
             for key in ps_keys:
-                ps.append(read_dict_from_hdf5(fn_hdf5=fname, folder=f"ps/{key}"))
+                ps.append(pynibs.read_dict_from_hdf5(fn_hdf5=fname, folder=f"ps/{key}"))
 
         except KeyError:
             ps = None
@@ -807,7 +640,7 @@ def load_subject_pkl(fname):
         fn_scipt = os.path.join(os.path.split(fname)[0],
                                 "create_subject_" +
                                 os.path.splitext(os.path.split(fname)[1])[0] + ".py")
-        bash_call("python {}".format(fn_scipt))
+        pynibs.bash_call("python {}".format(fn_scipt))
 
         with open(fname, 'rb') as f:
             return pickle.load(f)
