@@ -10,77 +10,62 @@ import numpy as np
 import h5py
 
 from scipy.special import binom
-from scipy.spatial import Delaunay
-from scipy.interpolate import griddata
 from sklearn.neighbors import KernelDensity
 
 
-def calc_tet_volume(points, abs=True):
+def tal2mni(coords, direction='tal2mni', style='nonlinear'):
     """
-    Calculate tetrahedra volumes.
+    Transform Talairach coordinates into (SPM) MNI space and vice versa.
 
-    Parameter
-    ---------
-    points: np.ndarray
-        shape: (n_tets,4,3)
-        [[[Ax, Ay, Az],
-          [Bx, By, Bz],
-          [Cx, Cy, Cz],
-          [Dx, Dy, Dy]],
+    This is taken from https://imaging.mrc-cbu.cam.ac.uk/imaging/MniTalairach and
+    http://gibms.mc.ntu.edu.tw/bmlab/tools/data-analysis-codes/mni2tal-m/
 
-         [[Ax, Ay, Az],
-          [Bx, By, Bz],
-          [Cx, Cy, Cz],
-          [Dx, Dy, Dy]],
-
-        ...
-
-        ]
+    Parameters
+    ----------
+    coords : np.ndarray or list
+        x,y,z coordinates
+    direction : str, default: 'tal2mni
+        Transformation direction. One of ('tal2mni', 'mni2tal')
+    style : str, default: 'nonlinear'
+        Transformation style. One of ('linear', 'nonlinear')
 
     Returns
     -------
-    volume: np.ndarray
-        shape: (n_tets)
+    coords_trans : np.ndarray
 
-    Other Parameters
-    ----------------
-    abs : bool
-        Return magnitude. Default: True.
     """
-    if points.ndim == 2:
-        points = np.atleast_3d(points).reshape(1,4,3)
-    if points.ndim != 3:
-        raise ValueError(f"Wrong dimensions for points: ({points.shape}). Expected: (n_tets,4,3))")
+    assert direction in ['tal2mni',
+                         'mni2tal'], f"direction parameter '{direction}' invalid. Choose 'tal2mni' or 'mni2tal'."
+    assert style in ['linear', 'nonlinear'], f"style parameter '{style}' invalid. Choose 'linear' or 'nonlinear'."
+    if len(coords) == 3:
+        coords = np.hstack((coords, 1))
+    if style == 'linear':
+        mat = np.array([[0.88, 0, 0, -0.8], [0, 0.97, 0, -3.32], [0, 0.05, 0.88, -0.44], [0, 0, 0, 1]])
+        if direction == 'mni2tal':
+            return np.dot(mat, coords)[:3]
+        elif direction == 'tal2mni':
+            return np.dot(np.linalg.inv(mat), coords)[:3]
+    elif style == 'nonlinear':
+        upper_mat = np.array([[0.99, 0, 0, 0],
+                              [0, 0.9688, 0.0460, 0],
+                              [0, -0.0485, 0.9189, 0],
+                              [0, 0, 0, 1]])
 
-    a = np.zeros((points.shape[0], 4, 4))
-    a[:, :, 3] = 1
-    a[:, :, :3] = points
-    a = a.swapaxes(1, 2)
-
-    if abs:
-        return np.abs(1 / 6 * np.linalg.det(a))
-    else:
-        return 1 / 6 * np.linalg.det(a)
-
-
-def calc_tri_surface(points):
-    """
-    Calculate triangle surface areas.
-
-    Parameter
-    ---------
-    points : np.ndarray
-        (n_triangles,3,3)
-
-    Returns
-    -------
-    triange_area : np.ndarray
-    """
-    a = np.linalg.norm(points[:, 0] - points[:, 1], axis=1)
-    b = np.linalg.norm(points[:, 1] - points[:, 2], axis=1)
-    c = np.linalg.norm(points[:, 0] - points[:, 2], axis=1)
-    s = np.sum((a, b, c), axis=0) / 2
-    return (s * (s - a) * (s - b) * (s - c)) ** 0.5
+        lower_mat = np.array([[0.99, 0, 0, 0],
+                              [0, 0.9688, 0.042, 0],
+                              [0, -0.0485, 0.839, 0],
+                              [0, 0, 0, 1]])
+        if direction == 'mni2tal':
+            pass
+        elif direction == 'tal2mni':
+            upper_mat = np.linalg.inv(upper_mat)
+            lower_mat = np.linalg.inv(lower_mat)
+        if coords[2] > 0:
+            # above AC
+            return np.dot(upper_mat, coords)[:3]
+        else:
+            # below AC
+            return np.dot(lower_mat, coords)[:3]
 
 
 def rd(array, array_ref):
@@ -104,115 +89,6 @@ def rd(array, array_ref):
     return np.linalg.norm(array - array_ref) / np.linalg.norm(array_ref)
 
 
-def get_sphere(mesh=None, mesh_fn=None, target=None, radius=None, roi_idx=None, roi=None, elmtype='tris'):
-    """
-    Return element idx of elements within a certain distance to provided target.
-    Elements might be 'tris' (default) or 'tets'
-
-    If roi object / idx and mesh fn is provided, the roi is expected to have midlayer information and the roi
-    geometry is used.
-
-
-
-    """
-
-    from ..hdf5_io import load_roi_surface_obj_from_hdf5
-    from pynibs import load_mesh_hdf5
-
-    if mesh_fn is not None:
-        if mesh is not None:
-            raise ValueError("Either provide mesh or mesh_fn")
-        if mesh_fn.endswith('.hdf5'):
-            mesh = load_mesh_hdf5(mesh_fn)
-    if roi is None and roi_idx is not None:
-        if mesh_fn is None:
-            raise ValueError("Provide mesh_fn to load roi from")
-        roi = load_roi_surface_obj_from_hdf5(mesh_fn)[roi_idx]
-    if elmtype == 'tris':
-        return tris_in_sphere(mesh=mesh, target=target, radius=radius, roi=roi)
-    elif elmtype == 'tets':
-        return tets_in_sphere(mesh=mesh, target=target, radius=radius, roi=roi)
-    else:
-        raise ValueError(f"Unknown elmtpye '{elmtype}'")
-
-
-def tets_in_sphere(mesh, target, radius, roi):
-    """
-    Worker function for get_sphere()
-
-    """
-    if roi is None:
-        if radius is None or radius == 0:
-            return np.where(np.linalg.norm(mesh.tetrahedra_center - target, axis=1) ==
-                            np.min(np.linalg.norm(mesh.tetrahedra_center - target, axis=1)))[0]
-    else:
-
-        if radius is not None and radius > 0:
-            tri_target_idx = np.where(np.linalg.norm(roi.tri_center_coord_mid - target, axis=1) <= radius)[0]
-        else:
-            tri_target_idx = np.where(np.linalg.norm(roi.tri_center_coord_mid - target, axis=1) == np.min(
-                np.linalg.norm(roi.tri_center_coord_mid - target, axis=1)))[0]
-        tet_target_idx = roi.tet_idx_tri_center_mid[tri_target_idx]
-        return tet_target_idx
-
-
-def tris_in_sphere(mesh, target, radius, roi):
-    """
-    Worker function for get_sphere()
-
-    """
-    if roi is None:
-        if radius is None or radius == 0:
-            return np.where(np.linalg.norm(mesh.triangles_center - target, axis=1) ==
-                            np.min(np.linalg.norm(mesh.triangles_center - target, axis=1)))[0]
-
-    else:
-        if radius is not None and radius > 0:
-            tri_target_idx = np.where(np.linalg.norm(roi.tri_center_coord_mid - target, axis=1) <= radius)[0]
-        else:
-            tri_target_idx = np.where(np.linalg.norm(roi.tri_center_coord_mid - target, axis=1) == np.min(
-                np.linalg.norm(roi.tri_center_coord_mid - target, axis=1)))[0]
-
-        return tri_target_idx
-
-
-def sample_sphere(n_points, r):
-    """
-    Creates n_points evenly spread in a sphere of radius r.
-
-    Parameters
-    ----------
-    n_points: int
-        Number of points to be spread, must be odd
-    r: float
-        Radius of sphere
-
-    Returns
-    -------
-    points: ndarray of float [N x 3]
-        Evenly spread points in a unit sphere
-    """
-
-    assert n_points % 2 == 1, "The number of points must be odd"
-    points = []
-
-    # The golden ratio
-    phi = (1 + math.sqrt(5)) / 2.
-    n = int((n_points - 1) / 2)
-
-    for i in range(-n, n + 1):
-        lat = math.asin(2 * i / n_points)
-        lon = 2 * math.pi * i / phi
-        x = r * math.cos(lat) * math.cos(lon)
-        y = r * math.cos(lat) * math.sin(lon)
-        z = r * math.sin(lat)
-        points.append((x, y, z))
-
-    points = np.array(points, dtype=float)
-
-    return points
-
-
 def generalized_extreme_value_distribution(x, mu, sigma, k):
     """
     Generalized extreme value distribution
@@ -238,8 +114,6 @@ def generalized_extreme_value_distribution(x, mu, sigma, k):
     return y
 
 
-# Differential Evolution (single core)
-############################################################################################################
 def differential_evolution(fobj, bounds, mut=0.8, crossp=0.7, popsize=20, its=1000, **kwargs):
     """
     Differential evolution optimization algorithm
@@ -371,8 +245,8 @@ def likelihood_posterior(x, y, fun, bounds=None, verbose=True, normalized_params
         x data
     y : ndarray of float [n_points]
         y data
-    fun : function object
-        Function object to fit the data to (e.g. sigmoid)
+    fun : function
+        Function to fit the data to (e.g. sigmoid)
     bounds : dict, optional, default: None
         Dictionary containing the bounds of "sigma_x" and "sigma_y" and the free parameters of fun
     verbose : bool, optional, default: True
@@ -429,7 +303,7 @@ def likelihood_posterior(x, y, fun, bounds=None, verbose=True, normalized_params
     # determine probabilities of observations
     kde = KernelDensity(bandwidth=0.01, kernel='gaussian')
 
-    l = []
+    likelihood = []
 
     for i in range(n_bins):
         mask = np.logical_and(x_pre >= (x_bins_loc[i] - dx_bins / 2), x_pre < (x_bins_loc[i] + dx_bins / 2))
@@ -448,24 +322,24 @@ def likelihood_posterior(x, y, fun, bounds=None, verbose=True, normalized_params
         # get probability densities at data
         kde_y_post_bins = np.exp(kde_bins.score_samples(y[mask_data][:, np.newaxis]))
 
-        l.append(kde_y_post_bins)
+        likelihood.append(kde_y_post_bins)
 
-    l = np.concatenate(l)
+    likelihood = np.concatenate(likelihood)
 
     # mask out zero probabilities
-    l[l == 0] = 1e-100
+    likelihood[likelihood == 0] = 1e-100
 
     # determine log likelihood
-    l = np.sum(np.log10(l))
+    likelihood = np.sum(np.log10(likelihood))
 
     stop = time.time()
 
     if verbose:
         parameter_str = [f"{p_}={params[p_]:.5f}" for p_ in params]
-        print(f"Likelihood: {l:.1f} / sigma_x={sigma_x:.5f}, sigma_y={sigma_y:.5f}, " +
+        print(f"Likelihood: {likelihood:.1f} / sigma_x={sigma_x:.5f}, sigma_y={sigma_y:.5f}, " +
               ", ".join(parameter_str) + f"({stop - start:.2f} sec)")
 
-    return -l
+    return -likelihood
 
 
 def mutual_coherence(array):
@@ -597,175 +471,6 @@ def compute_chunks(seq, num):
     return out
 
 
-def get_indices_discontinuous_data(data, con, neighbor=False, deviation_factor=2,
-                                   min_val=None, not_fitted_elms=None, crit='median', neigh_style='point'):
-    """
-    Get element indices (and the best neighbor index), where the data is discontinuous
-
-    Parameters
-    ----------
-    data : ndarray of float [n_data]
-        Data array to analyze given in the element center
-    con : ndarray of float [n_data, 3 or 4]
-        Connectivity matrix
-    neighbor : boolean, optional, default=False
-        Return also the element index of the "best" neighbor (w.r.t. median of data)
-    deviation_factor : float
-        Allows data deviation from 1/deviation_factor < data[i]/median < deviation_factor
-    min_val : float, optional
-        If given, only return elements which have a neighbor with data higher than min_val.
-    not_fitted_elms : ndarray
-        If given, these elements are not used as neighbors
-    crit: str, default: median
-        Criterium for best neighbor. Either median or max value
-    neigh_style : str, default: 'point'
-        Should neighbors share point or 'edge'
-
-    Returns
-    -------
-    idx_disc : list of int [n_disc]
-        Index list containing the indices of the discontinuous elements
-    idx_neighbor : list of int [n_disc]
-        Index list containing the indices of the "best" neighbors of the discontinuous elements
-    """
-
-    n_ele = con.shape[0]
-    idx_disc, idx_neighbor = [], []
-
-    data[data == 0] = 1e-12
-
-    if neigh_style == 'point':
-        def get_neigh(m):
-            return np.logical_and(0 < mask, mask < 3)
-    elif neigh_style == 'edge':
-        def get_neigh(m):
-            return mask == 2
-    else:
-        raise NotImplementedError(f"neigh_style {neigh_style} unknown.")
-
-    if crit == 'median':
-        def is_neigh():
-            if not (1 / deviation_factor < data_i / median < deviation_factor):
-                neighbor_indices = np.where(mask_neighbor)[0]
-                best_neigh = neighbor_indices[(np.abs(data[neighbor_indices] - median)).argmin()]
-                if min_val is None or data[best_neigh] > min_val:
-                    idx_disc.append(elm_i)
-                # if neighbor:
-                idx_neighbor.append(best_neigh)
-    elif crit == 'max':
-        def is_neigh():
-            if data_i / median < 1 / deviation_factor:
-                neighbor_indices = np.where(mask_neighbor)[0]
-                best_neigh = neighbor_indices[(data[neighbor_indices]).argmax()]
-                if min_val is None or data[best_neigh] > min_val:
-                    idx_disc.append(elm_i)
-
-                # if neighbor:
-                idx_neighbor.append(best_neigh)
-    elif crit == 'randmax':
-        def is_neigh():
-            if data_i / median < 1 / deviation_factor:
-                neighbor_indices = np.where(mask_neighbor)[0]
-                best_neigh = np.random.choice(neighbor_indices[(data[neighbor_indices]) > 0], 1)
-                if min_val is None or data[best_neigh] > min_val:
-                    idx_disc.append(elm_i)
-
-                # if neighbor:
-                idx_neighbor.append(best_neigh)
-    else:
-        raise NotImplementedError(f"Criterium {crit} unknown. ")
-
-    # start = time.time()
-    for elm_i, data_i in zip(range(n_ele), data):
-        if elm_i in not_fitted_elms:
-            continue
-
-        # find neighbors
-        mask = np.sum(np.isin(con, con[elm_i, :]), axis=1)
-        mask_neighbor = get_neigh(mask)
-
-        # best_values are set to 0 for bad elements and unfittable ones. do not use these as neighbors
-        if not_fitted_elms is not None and len(not_fitted_elms) != 0:
-            mask_neighbor[not_fitted_elms] = False
-
-        # if the element is lonely floating and has no neighbors ... continue
-        if not np.sum(mask_neighbor):
-            continue
-
-        # check if current value does not fit to neighbors
-        median = np.median(data[mask_neighbor])
-
-        if not median:
-            median = 1e-12
-
-        is_neigh()
-
-        # if not (1 / deviation_factor < data_i / median < deviation_factor):
-        #     # find best neighbor idx
-        #     neighbor_indices = np.where(mask_neighbor)[0]
-        #
-        #     if crit == 'max':
-        #         best_neigh = neighbor_indices[(data[neighbor_indices]).argmax()]
-        #     elif crit == 'median':
-        #         best_neigh = neighbor_indices[(np.abs(data[neighbor_indices] - median)).argmin()]
-        #     else:
-        #         raise  NotImplementedError(f"Criterium {crit} unknown. ")
-        #     if min_val is None or data[best_neigh] > min_val:
-        #         idx_disc.append(elm_i)
-        #
-        #     if neighbor:
-        #         idx_neighbor.append(best_neigh)
-
-        # stop = time.time()
-        # print(stop-start)
-
-    if neighbor:
-        return idx_disc, idx_neighbor
-    else:
-        return idx_disc
-
-
-def find_nearest(array, value):
-    """
-    Given an "array", and given a "value" , returns an index j such that "value" is between array[j]
-    and array[j+1]. "array" must be monotonic increasing. j=-1 or j=len(array) is returned
-    to indicate that "value" is out of range below and above respectively.
-
-    Parameters
-    ----------
-    array : nparray of float
-        Monotonic increasing array
-    value : float
-        Target value the nearest neighbor index in "array" is computed for
-
-    Returns
-    -------
-    idx : int
-        Index j such that "value" is between array[j] and array[j+1]
-
-    """
-    n = len(array)
-    if value < array[0]:
-        return -1
-    elif value > array[n - 1]:
-        return n
-    jl = 0  # Initialize lower
-    ju = n - 1  # and upper limits.
-    while ju - jl > 1:  # If we are not yet done,
-        jm = (ju + jl) >> 1  # compute a midpoint with a bitshift
-        if value >= array[jm]:
-            jl = jm  # and replace either the lower limit
-        else:
-            ju = jm  # or the upper limit, as appropriate.
-    # Repeat until the test condition is satisfied.
-    if value == array[0]:  # edge cases at bottom
-        return 0
-    elif value == array[n - 1]:  # and top
-        return n - 1
-    else:
-        return jl
-
-
 def bash(command):
     """
     Executes bash command and returns output message in stdout (uses os.popen).
@@ -889,9 +594,9 @@ def quat_rotation_angle(q):
     """
 
     if len(q) == 3:
-        alpha = 2 * np.arcsin(np.linalg.norm(q))
+        return 2 * np.arcsin(np.linalg.norm(q))
     elif len(q) == 4:
-        alpha = q[0]
+        return q[0]
     else:
         raise ValueError('Please check size of quaternion')
 
@@ -1050,7 +755,7 @@ def rotation_matrix_to_euler_angles(r):
 
     Parameters
     ----------
-    M : np.array [3 x 3]
+    r : np.array [3 x 3]
         Rotation matrix (z, y', x'')
 
     Returns
@@ -1081,7 +786,7 @@ def rotation_matrix_to_euler_angles(r):
 
 def recursive_len(item):
     """
-    Determine len of list of list (recursively).
+    Determine len of list of lists (recursively).
 
     Parameters
     ----------
@@ -1091,7 +796,7 @@ def recursive_len(item):
     Returns
     -------
     len : int
-        Total length of list of list
+        Total length of list of lists
     """
     if type(item) == list:
         return sum(recursive_len(subitem) for subitem in item)
@@ -1139,34 +844,6 @@ def unique_rows(a):
     # surface_points=np.vstack({tuple(row) for row in surface_points})
 
 
-def in_hull(points, hull):
-    """
-    Test if points in `points` are in `hull`.
-    `points` should be a [N x K] coordinates of N points in K dimensions.
-    `hull` is either a scipy.spatial.Delaunay object or the [M x K] array of the
-    coordinates of M points in Kdimensions for which Delaunay triangulation
-    will be computed.
-
-    Parameters
-    ----------
-    points : nparray [N_points x 3]
-        Set of floating point data to test whether they are lying inside the hull or not
-    hull : Delaunay instance or nparray [M x K]
-        Surface data
-
-    Returns
-    -------
-    inside : boolean array
-        TRUE: point inside the hull
-        FALSE: point outside the hull
-    """
-
-    if not isinstance(hull, Delaunay):
-        hull = Delaunay(hull)
-
-    return hull.find_simplex(points) >= 0
-
-
 def calc_n_network_combs(n_e, n_c, n_i):
     """
     Determine number of combinations if all conditions may be replaced between N_i elements (mixed interaction)
@@ -1189,31 +866,6 @@ def calc_n_network_combs(n_e, n_c, n_i):
     return binom(n_e, n_i) * np.sum([((n_i - 1) ** k) * binom(n_c, k) for k in range(1, n_c)])
 
 
-def cell_data_to_point_data(tris, data_tris, nodes, method='nearest'):
-    """
-    A wrapper for scipy.interpolate.griddata to interpolate cell data to node data.
-
-    Parameters
-    ----------
-
-    tris : np.ndarray
-        element number list, (n_tri, 3)
-    data_tris : np.ndarray
-        data in tris, , (n_tri x 3)
-    nodes : np.ndarray
-        nodes coordinates, (n_nodes, 3
-    method: str, default: 'nearest'
-        Which method to use for interpolation. Default uses NearestNDInterpolator
-
-    Returns
-    -------
-    data_nodes : np.ndarray
-        Data in nodes
-    """
-    elms_center = np.mean(nodes[tris], axis=1)
-    return griddata(elms_center, data_tris, nodes, method)
-
-
 def load_muaps(fn_muaps, fs=1e6, fs_downsample=1e5):
     # load MUAPs and downsample
     with h5py.File(fn_muaps, "r") as f:
@@ -1228,3 +880,51 @@ def load_muaps(fn_muaps, fs=1e6, fs_downsample=1e5):
         muaps[:, i] = np.interp(t_muap, t_muap_orig, muaps_orig[:, i])
 
     return muaps, t_muap
+
+
+def cross_product(A, B):
+    """
+    Evaluates the cross product between the vector pairs in a and b using pure Python.
+
+    Parameters
+    ----------
+    A : nparray of float 2 x [N x 3]
+    B : nparray of float 2 x [N x 3]
+        Input vectors, the cross product is evaluated between
+
+    Returns
+    -------
+    c : nparray of float [N x 3]
+        Cross product between vector pairs in a and b
+    """
+
+    c1 = np.multiply(A[:, 1], B[:, 2]) - np.multiply(A[:, 2], B[:, 1])
+    c2 = np.multiply(A[:, 2], B[:, 0]) - np.multiply(A[:, 0], B[:, 2])
+    c3 = np.multiply(A[:, 0], B[:, 1]) - np.multiply(A[:, 1], B[:, 0])
+    # C=np.array(np.multiply(A[:, 1], B[:, 2]) - np.multiply(A[:, 2], B[:, 1]),
+    #            np.multiply(A[:, 2], B[:, 0]) - np.multiply(A[:, 0], B[:, 2]),
+    #            np.multiply(A[:, 0], B[:, 1]) - np.multiply(A[:, 1], B[:, 0]))
+    return np.vstack([c1, c2, c3]).transpose()
+
+
+def cross_product_einsum2(a, b):
+    """
+    Evaluates the cross product between the vector pairs in a and b using the double Einstein sum.
+
+    Parameters
+    ----------
+    a : nparray of float 2 x [N x 3]
+    b : nparray of float 2 x [N x 3]
+        Input vectors, the cross product is evaluated between
+
+    Returns
+    -------
+    c : nparray of float [N x 3]
+        Cross product between vector pairs in a and b
+    """
+
+    eijk = np.zeros((3, 3, 3))
+    eijk[0, 1, 2] = eijk[1, 2, 0] = eijk[2, 0, 1] = 1
+    eijk[0, 2, 1] = eijk[2, 1, 0] = eijk[1, 0, 2] = -1
+
+    return np.einsum('iak,ak->ai', np.einsum('ijk,aj->iak', eijk, a), b)
